package db

import (
	"errors"
	"github.com/go-xorm/xorm"
	"leit.com/leit_seat_aps/common"
	"xorm.io/core"
)

//Grammer 的ASN头信息表
type Jit_asnhead_landing struct {
	Finr           int                   `xorm:"pk" json:"jit_asnhead_landing.finr"`
	Asnnr          string                `xorm:"pk" json:"jit_asnhead_landing.asnnr"`
	Asndate        string                `json:"jit_asnhead_landing.asndate"`
	Asntime        string                `json:"jit_asnhead_landing.asntime"`
	Shipdate       string                `json:"jit_asnhead_landing.shipdate"`
	Shiptime       string                `json:"jit_asnhead_landing.shiptime"`
	Tradepartnerid string                `json:"jit_asnhead_landing.tradepartnerid"`
	Doctype        string                `json:"jit_asnhead_landing.doctype"`
	Senderappid    string                `json:"jit_asnhead_landing.senderappid"`
	Recieverappid  string                `json:"jit_asnhead_landing.recieverappid"`
	Supplierid     string                `json:"jit_asnhead_landing.supplierid"`
	Shipfromid     string                `json:"jit_asnhead_landing.shipfromid"`
	Shiptoid       string                `json:"jit_asnhead_landing.shiptoid"`
	Dockcode       string                `json:"jit_asnhead_landing.dockcode"`
	Parsed         int                   `json:"jit_asnhead_landing.parsed"`
	Asnfile        string                `json:"jit_asnhead_landing.asnfile"`
	Lastmodif      string                `json:"jit_asnhead_landing.lastmodif"`
	Lastuser       string                `json:"jit_asnhead_landing.lastuser"`
	Credatuz       string                `json:"jit_asnhead_landing.credatuz"`
	Itemlst        []Jit_asnitem_landing `json:"itemlst" xorm:"-"`
}

func (t *Jit_asnhead_landing) Clipped() {
	common.TrimStruct(t, *t)
}

func (t *Jit_asnhead_landing) TableName() string {
	return "jit_asnhead_landing"
}

//增
func (t *Jit_asnhead_landing) Add() (err error) {
	var (
		affw       int64
		asnheadtab Jit_asnhead_landing
	)

	e := G_DbEngine
	asnheadtab = Jit_asnhead_landing{}
	affw, err = e.Table("jit_asnhead_landing").ID(core.PK{G_FINR, t.Asnnr}).Count(asnheadtab)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table("jit_asnhead_landing").Insert(t)
	if err != nil {
		return err
	}

	return nil
}

//增
func (t *Jit_asnhead_landing) Insert(session *xorm.Session) (err error) {
	var (
		affw       int64
		asnheadtab Jit_asnhead_landing
	)

	asnheadtab = Jit_asnhead_landing{}
	affw, err = session.Table("jit_asnhead_landing").ID(core.PK{G_FINR, t.Asnnr}).Count(asnheadtab)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = session.Table("jit_asnhead_landing").Insert(t)
	if err != nil {
		return err
	}

	return nil
}

//删
func (t *Jit_asnhead_landing) Del() bool {
	e := G_DbEngine
	_, err := e.ID(core.PK{G_FINR, t.Asnnr}).Delete(&Jit_asnhead_landing{})
	if err != nil {
		return false
	}
	return true
}

//删
func (t *Jit_asnhead_landing) Delelte(session *xorm.Session) bool {
	var (
		asnitemtab Jit_asnitem_landing
		asncachetab Jit_asnorder_cache
		asnordtab  Jit_asnorder_landing
		err        error
	)
	// 删除订单数据
	asnordtab = Jit_asnorder_landing{Finr: G_FINR, Asnnr: t.Asnnr}
	if _, err = session.Delete(asnordtab); err != nil {
		return false
	}
	// 删除零件数据
	asnitemtab = Jit_asnitem_landing{Finr: G_FINR, Asnnr: t.Asnnr}
	if _, err = session.Delete(asnitemtab); err != nil {
		return false
	}
	// 删除缓存数据
	asncachetab = Jit_asnorder_cache{Finr: G_FINR, Asnnr: t.Asnnr}
	if _, err = session.Delete(asncachetab); err != nil {
		return false
	}
	// 删除ASN头
	if _, err = session.ID(core.PK{G_FINR, t.Asnnr}).Delete(&Jit_asnhead_landing{}); err != nil {
		return false
	}
	return true
}

//改
func (t *Jit_asnhead_landing) Update() bool {
	e := G_DbEngine
	_, err := e.ID(core.PK{G_FINR, t.Asnnr}).Update(t)
	if err != nil {
		return false
	}
	return true
}

//更新指定字段
func (t *Jit_asnhead_landing) UpdateFields( fields string) (err error) {
	e := G_DbEngine
	if _, err = e.Table("jit_asnhead_landing").ID(core.PK{G_FINR, t.Asnnr}).Cols(fields).Update(t); err != nil {
		return
	}
	return
}

//查
func (t *Jit_asnhead_landing) SelectOne() (data Jit_asnhead_landing, err error) {
	var i, j int

	e := G_DbEngine
	_, err = e.ID(core.PK{G_FINR, t.Asnnr}).Get(&data)
	if err != nil {
		return data, err
	}

	// 获取子项列表
	if err = e.Where("finr = ? and asnnr = ?",
		G_FINR, data.Asnnr).OrderBy("partid").Find(&data.Itemlst); err != nil {
		return
	}
	for i, _ = range data.Itemlst {
		data.Itemlst[i].Clipped()
		// 获取订单列表
		if err = e.Where("finr = ? and asnnr = ? and partid = ?",
			G_FINR, data.Itemlst[i].Asnnr, data.Itemlst[i].Partid).OrderBy("ordernr").Find(&data.Itemlst[i].Orderlst); err != nil {
			return
		}
		for j, _ = range data.Itemlst[i].Orderlst {
			data.Itemlst[i].Orderlst[j].Clipped()
		}
	}

	return data, nil
}

//查
func (t *Jit_asnhead_landing) GetUnparsedAsnLandingData() (datalst []Jit_asnhead_landing, err error) {
	var i, j, k int

	e := G_DbEngine

	// 获取未解析的ASN头列表
	if err = e.Where("finr = ? and parsed = ?",
		G_FINR, 0).OrderBy("asnnr").Find(&datalst); err != nil {
		return
	}

	for i, _ = range datalst {
		datalst[i].Clipped()
		// 获取子项列表
		if err = e.Where("finr = ? and asnnr = ?",
			G_FINR, datalst[i].Asnnr).OrderBy("partid").Find(&datalst[i].Itemlst); err != nil {
			return
		}
		for j, _ = range datalst[i].Itemlst {
			datalst[i].Itemlst[j].Clipped()
			// 获取订单列表
			if err = e.Table("jit_asnorder_landing").Join("INNER","jit_asnorder_cache","jit_asnorder_cache.finr = jit_asnorder_landing.finr and jit_asnorder_cache.asnnr = jit_asnorder_landing.asnnr and jit_asnorder_cache.partid = jit_asnorder_landing.partid and jit_asnorder_cache.ordernr = jit_asnorder_landing.ordernr").Where("jit_asnorder_landing.finr = ? and jit_asnorder_landing.asnnr = ? and jit_asnorder_landing.partid = ? and jit_asnorder_cache.flag = 0",
				G_FINR, datalst[i].Itemlst[j].Asnnr, datalst[i].Itemlst[j].Partid).OrderBy("jit_asnorder_landing.ordernr").Find(&datalst[i].Itemlst[j].Orderlst); err != nil {
				return
			}
			for k, _ = range datalst[i].Itemlst[j].Orderlst {
				datalst[i].Itemlst[j].Orderlst[k].Clipped()
			}
		}
	}

	return datalst, nil
}
