package db

import (
	"errors"
	"github.com/go-xorm/xorm"
	"leit.com/leit_seat_aps/common"
	"xorm.io/core"
)

// 发运车模板
type Jit_shiptemplate struct {
	Finr           int    `xorm:"pk"`
	Shiptemplateid string `xorm:"pk"`
	Active         int
	Defaultcreate  int
	Descr          string
	Projnr         string
	Shiptype       string
	Planqty        int
	Printerid      string
	Templatefile   string
	Printqty       int
	Orientation    int
	Lastmodif      string
	Lastuser       string
	Credatuz       string
	Itemlst        []Jit_shiptemplate_itemlst  `json:"itemlst" xorm:"-"`
	Printlst       []Jit_shiptemplate_printlst `json:"printlst" xorm:"-"`
}

func (t *Jit_shiptemplate) Clipped() {
	common.TrimStruct(t, *t)
}

func (t *Jit_shiptemplate) TableName() string {
	return "jit_shiptemplate"
}

//增
func (t *Jit_shiptemplate) Add() error {
	e := G_DbEngine
	sptmptab := new(Jit_shiptemplate)
	affw, err := e.Table("jit_shiptemplate").ID(core.PK{G_FINR, t.Shiptemplateid}).Count(sptmptab)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table("jit_shiptemplate").Insert(t)

	if err != nil {
		return err
	}
	return nil
}

//增
func (t *Jit_shiptemplate) Insert(session *xorm.Session) error {
	sptmptab := new(Jit_shiptemplate)
	affw, err := session.Table("jit_shiptemplate").ID(core.PK{G_FINR, t.Shiptemplateid}).Count(sptmptab)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = session.Table("jit_shiptemplate").Insert(t)

	if err != nil {
		return err
	}
	return nil
}

//删
func (t *Jit_shiptemplate) Del() bool {
	e := G_DbEngine
	_, err := e.ID(core.PK{G_FINR, t.Shiptemplateid}).Delete(&Jit_shiptemplate{})
	if err != nil {
		return false
	}
	return true
}

//改
func (t *Jit_shiptemplate) Update() bool {
	e := G_DbEngine
	_, err := e.ID(core.PK{G_FINR, t.Shiptemplateid}).Update(t)
	if err != nil {
		return false
	}
	return true
}

//更新指定字段
func (t *Jit_shiptemplate) UpdateFields(session *xorm.Session, fields string) (err error) {
	if _, err = session.Table("jit_shiptemplate").ID(core.PK{G_FINR, t.Shiptemplateid}).Cols(fields).Update(t); err != nil {
		return
	}
	return
}

//查
func (t *Jit_shiptemplate) SelectOne() (data Jit_shiptemplate, err error) {
	var i int

	e := G_DbEngine
	_, err = e.ID(core.PK{G_FINR, t.Shiptemplateid}).Get(&data)
	if err != nil {
		return data, err
	}

	// 获取值列表
	if err = e.Where("finr = ? and shiptemplateid = ? ",
		G_FINR, data.Shiptemplateid).OrderBy("pos").Find(&data.Itemlst); err != nil {
		return
	}
	for i, _ = range data.Itemlst {
		data.Itemlst[i].Clipped()
	}

	return data, nil
}

// 获取指定项目的发运车模板列表
func (t *Jit_shiptemplate) GetProjectAll(projnr string) (datalst []Jit_shiptemplate, err error) {
	var i, j, k int

	e := G_DbEngine
	if err = e.Where("finr = ? and projnr = ?", G_FINR, projnr).OrderBy("shiptemplateid").Find(&datalst); err != nil {
		return
	}

	// 遍历模板获取模板子项数据
	for i, _ = range datalst {
		datalst[i].Clipped()
		datalst[i].Itemlst = []Jit_shiptemplate_itemlst{}
		// 获取属模板子项数据
		if err = e.Where("finr = ? and shiptemplateid = ?", G_FINR, datalst[i].Shiptemplateid).OrderBy("pos").Find(&datalst[i].Itemlst); err != nil {
			return
		}
		for j, _ = range datalst[i].Itemlst {
			datalst[i].Itemlst[j].Clipped()
		}
		// 获取模板打印设置数据
		if err = e.Where("finr = ? and shiptemplateid = ?", G_FINR, datalst[i].Shiptemplateid).OrderBy("pos").Find(&datalst[i].Printlst); err != nil {
			return
		}
		for k, _ = range datalst[i].Printlst {
			datalst[i].Printlst[k].Clipped()
		}
	}

	return
}

// 获取指定项目的发运车模板列表
func (t *Jit_shiptemplate) GetAll() (datalst []Jit_shiptemplate, err error) {
	var i, j, k int

	e := G_DbEngine
	if err = e.Where("finr = ?", G_FINR).OrderBy("shiptemplateid").Find(&datalst); err != nil {
		return
	}

	// 遍历模板获取模板子项数据
	for i, _ = range datalst {
		datalst[i].Clipped()
		datalst[i].Itemlst = []Jit_shiptemplate_itemlst{}
		// 获取属模板子项数据
		if err = e.Where("finr = ? and shiptemplateid = ?", G_FINR, datalst[i].Shiptemplateid).OrderBy("pos").Find(&datalst[i].Itemlst); err != nil {
			return
		}
		for j, _ = range datalst[i].Itemlst {
			datalst[i].Itemlst[j].Clipped()
		}
		// 获取模板打印设置数据
		if err = e.Where("finr = ? and shiptemplateid = ?", G_FINR, datalst[i].Shiptemplateid).OrderBy("pos").Find(&datalst[i].Printlst); err != nil {
			return
		}
		for k, _ = range datalst[i].Printlst {
			datalst[i].Printlst[k].Clipped()
		}
	}

	return
}
