package db

import (
	"errors"
	"github.com/go-xorm/xorm"
	"leit.com/leit_seat_aps/common"
	"xorm.io/core"
)

//MES消息表
type Mes_msg struct {
	Finr        int    `xorm:"pk"`
	Msgid       string `xorm:"pk"`
	Msgtype     string
	Msgobjid    string
	Msgevent    string
	Status      int
	Printheadsn string
	Printcopies int
	Errinfo     string
	Msg_ipara1  int
	Msg_ipara2  int
	Msg_cpara1  string
	Msg_cpara2  string
	Lastmodif   string
	Lastuser    string
	Credatuz    string
}

func (t *Mes_msg) Clipped() {
	common.TrimStruct(t, *t)
}
func (t *Mes_msg) TableName() string {
	return "mes_msg"
}

//增
func (t *Mes_msg) Add() error {
	e := G_DbEngine
	ordmsgtab := new(Order_msg)
	affw, err := e.Table("mes_msg").ID(core.PK{G_FINR, t.Msgid}).Count(ordmsgtab)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table("mes_msg").Insert(t)

	if err != nil {
		return err
	}
	return nil
}

//增
func (t *Mes_msg) Insert(session *xorm.Session) error {
	msgtab := new(Mes_msg)
	affw, err := session.Table("mes_msg").ID(core.PK{G_FINR, t.Msgid}).Count(msgtab)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = session.Table("mes_msg").Insert(t)

	if err != nil {
		return err
	}
	return nil
}

//删
func (t *Mes_msg) Del() bool {
	e := G_DbEngine
	_, err := e.ID(core.PK{G_FINR, t.Msgid}).Delete(&Mes_msg{})
	if err != nil {
		return false
	}
	return true
}

//改
func (t *Mes_msg) Update() bool {
	e := G_DbEngine
	_, err := e.ID(core.PK{G_FINR, t.Msgid}).Update(t)
	if err != nil {
		return false
	}
	return true
}

//更新指定字段
func (t *Mes_msg) UpdateFields(session *xorm.Session, fields string) (err error) {
	if _, err = session.Table("mes_msg").ID(core.PK{G_FINR, t.Msgid}).Cols(fields).Update(t); err != nil {
		return
	}
	return
}

//更新指定字段
func (t *Mes_msg) UpdateFieldsWithoutSession(fields string) (err error) {
	e := G_DbEngine
	if _, err = e.Table("mes_msg").ID(core.PK{G_FINR, t.Msgid}).Cols(fields).Update(t); err != nil {
		return
	}
	return
}

//查
func (t *Mes_msg) SelectOne() (data Order_msg, err error) {
	e := G_DbEngine
	_, err = e.ID(core.PK{G_FINR, t.Msgid}).Get(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}

//获取开口状态的打印消息
func (t *Mes_msg) GetOpenPrintMessages() (datalst []Mes_msg, err error) {
	var (
		i int
	)

	e := G_DbEngine
	if err = e.Where("finr = ? and status <= ? and msgevent = ?", G_FINR, common.MESMSG_STATUS_PLAN, common.MESMSG_EVENT_PRINT).OrderBy("credatuz").Find(&datalst); err != nil {
		return
	}
	for i = 0; i < len(datalst); i++ {
		datalst[i].Clipped()
	}
	return
}

//获取开口状态的订单消息，包括新建和重打
func (t *Mes_msg) GetProdPackReleaseMessages(msgtypes []string) (datalst []Mes_msg, err error) {
	var (
		i int
	)

	e := G_DbEngine
	if err = e.Where("finr = ? and msgtype = ? and status <= ? and msgevent = ?", G_FINR, common.MESMSG_TYPE_PACK, common.MESMSG_STATUS_PLAN, common.MESMSG_EVENT_PRINT).In("msgtype", msgtypes).OrderBy("credatuz").Find(&datalst); err != nil {
		return
	}
	for i = 0; i < len(datalst); i++ {
		datalst[i].Clipped()
	}
	return
}
