package db

import (
	"errors"
	"github.com/go-xorm/xorm"
	"leit.com/leit_seat_aps/common"
	"xorm.io/core"
)

//订单消息表
type Order_msg struct {
	Finr        int    `xorm:"pk"`
	Msgid       string `xorm:"pk"`
	Msgtype     string
	Msgobjid    string
	Msgevent    string
	Status      int
	Printheadsn string
	Printcopies int
	Errinfo     string
	Msg_ipara1  int
	Msg_ipara2  int
	Msg_cpara1  string
	Msg_cpara2  string
	Lastmodif   string
	Lastuser    string
	Credatuz    string
}

func (t *Order_msg) Clipped() {
	common.TrimStruct(t, *t)
}
func (t *Order_msg) TableName() string {
	return "order_msg"
}

//增
func (t *Order_msg) Add() error {
	e := G_DbEngine
	ordmsgtab := new(Order_msg)
	affw, err := e.Table("order_msg").ID(core.PK{G_FINR, t.Msgid}).Count(ordmsgtab)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table("order_msg").Insert(t)

	if err != nil {
		return err
	}
	return nil
}

//增
func (t *Order_msg) Insert(session *xorm.Session) error {
	ordmsgtab := new(Order_msg)
	affw, err := session.Table("order_msg").ID(core.PK{G_FINR, t.Msgid}).Count(ordmsgtab)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = session.Table("order_msg").Insert(t)

	if err != nil {
		return err
	}
	return nil
}

//删
func (t *Order_msg) Del() bool {
	e := G_DbEngine
	_, err := e.ID(core.PK{G_FINR, t.Msgid}).Delete(&Order_msg{})
	if err != nil {
		return false
	}
	return true
}

//改
func (t *Order_msg) Update() bool {
	e := G_DbEngine
	_, err := e.ID(core.PK{G_FINR, t.Msgid}).Update(t)
	if err != nil {
		return false
	}
	return true
}

//更新指定字段
func (t *Order_msg) UpdateFields(session *xorm.Session, fields string) (err error) {
	if _, err = session.Table("order_msg").ID(core.PK{G_FINR, t.Msgid}).Cols(fields).Update(t); err != nil {
		return
	}
	return
}

//更新指定字段
func (t *Order_msg) UpdateFieldsWithoutSession(fields string) (err error) {
	e := G_DbEngine
	if _, err = e.Table("order_msg").ID(core.PK{G_FINR, t.Msgid}).Cols(fields).Update(t); err != nil {
		return
	}
	return
}

//查
func (t *Order_msg) SelectOne() (data Order_msg, err error) {
	e := G_DbEngine
	_, err = e.ID(core.PK{G_FINR, t.Msgid}).Get(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}

//获取开口状态的订单消息，包括新建和重打
func (t *Order_msg) GetOpenPrintMessages(msgtypes []string) (datalst []Order_msg, err error) {
	var (
		i int
	)

	e := G_DbEngine
	if err = e.Where("finr = ? and status <= ? and msgevent = ?", G_FINR, common.MSG_STATUS_PLAN, common.MSG_EVENT_PRINT).In("msgtype", msgtypes).OrderBy("credatuz").Find(&datalst); err != nil {
		return
	}
	for i = 0; i < len(datalst); i++ {
		datalst[i].Clipped()
	}
	return
}

//获取开口状态的需求计算消息
func (t *Order_msg) GetOpenDemandMessages() (datalst []Order_msg, err error) {
	var (
		i int
	)

	e := G_DbEngine
	if err = e.Where("finr = ? and status <= ? and msgtype = ? and msgevent = ?", G_FINR, common.MSG_STATUS_PLAN,
		common.MSG_TYPE_DEMAND, common.MSG_EVENT_NEW).OrderBy("msgid").Find(&datalst); err != nil {
		return
	}
	for i = 0; i < len(datalst); i++ {
		datalst[i].Clipped()
	}
	return
}

//获取指定项目的开口状态的车型解析消息
func (t *Order_msg) GetProjectOpenCarmodelMessages(projnr string) (datalst []Order_msg, err error) {
	var i int

	e := G_DbEngine
	if err = e.Where("finr = ? and status <= ? and msg_cpara1 = ? and msgtype = ?", G_FINR,
		common.MSG_STATUS_PLAN, projnr, common.MSG_TYPE_CARMODEL).OrderBy("msgid").Find(&datalst); err != nil {
		return
	}
	for i = 0; i < len(datalst); i++ {
		datalst[i].Clipped()
	}
	return
}

//获取指定项目的开口状态的车型解析消息
func (t *Order_msg) GetOemorderOpenCarmodelMessages(oemordid string) (datalst []Order_msg, err error) {
	var i int

	e := G_DbEngine
	if err = e.Where("finr = ? and status <= ? and msgobjid = ? and msgtype = ?", G_FINR,
		common.MSG_STATUS_PLAN, oemordid, common.MSG_TYPE_CARMODEL).OrderBy("msgid").Find(&datalst); err != nil {
		return
	}
	for i = 0; i < len(datalst); i++ {
		datalst[i].Clipped()
	}
	return
}

//获取开口状态的类型为发运订单的订单消息
func (t *Order_msg) GetShipCloseMessages() (datalst []Order_msg, err error) {
	var i int

	e := G_DbEngine
	if err = e.Where("finr = ? and msgtype = ? and status <= ? and msgevent = ?", G_FINR, common.MSG_TYPE_SHIP, common.MSG_STATUS_PLAN, common.MSG_EVENT_CLOSE).OrderBy("credatuz").Find(&datalst); err != nil {
		return
	}
	for i = 0; i < len(datalst); i++ {
		datalst[i].Clipped()
	}
	return
}
