package db

import (
	"errors"
	"github.com/go-xorm/xorm"
	"leit.com/leit_seat_aps/common"
	"xorm.io/core"
)

// 计划原材料独立需求
type Pln_material_demand struct {
	Finr        int    `xorm:"pk"`
	Materialid  string `xorm:"pk"`
	Workordernr string `xorm:"pk"`
	Demanddate  string `xorm:"pk"`
	Descr       string
	Custmatid   string
	Demandtype  string
	Demandqty   int
	Demandyear  int
	Demandmonth int
	Demandweek  int
	Demandday   int
	Demandhour  int
	Swet        string
	Usestatus   int // 0=未使用；1=已使用
	Lastmodif   string
	Lastuser    string
	Credatuz    string
}

func (t *Pln_material_demand) Clipped() {
	common.TrimStruct(t, *t)
}

func (t *Pln_material_demand) TableName() string {
	return "pln_material_demand"
}

//增
func (t *Pln_material_demand) Add() error {
	e := G_DbEngine
	madtab := new(Pln_material_demand)
	affw, err := e.Table("pln_material_demand").ID(core.PK{G_FINR, t.Materialid, t.Workordernr, t.Demanddate}).Count(madtab)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table("pln_material_demand").Insert(t)

	if err != nil {
		return err
	}
	return nil
}

//增
func (t *Pln_material_demand) Insert(session *xorm.Session) error {
	madtab := new(Pln_material_demand)
	affw, err := session.Table("pln_material_demand").ID(core.PK{G_FINR, t.Materialid, t.Workordernr, t.Demanddate}).Count(madtab)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = session.Table("pln_material_demand").Insert(t)

	if err != nil {
		return err
	}
	return nil
}

//增，如果存在则更新
func (t *Pln_material_demand) InsertAndUpdate(session *xorm.Session) (err error) {
	var affw int64

	madtab := new(Pln_material_demand)
	if affw, err = session.Table("pln_material_demand").ID(core.PK{G_FINR, t.Materialid, t.Workordernr, t.Demanddate}).Count(madtab); err != nil {
		return
	}
	if affw > 0 {
		if _, err = session.ID(core.PK{G_FINR, t.Materialid, t.Workordernr, t.Demanddate}).Update(t); err != nil {
			return
		}
	}else{
		if _, err = session.Table("pln_material_demand").Insert(t); err != nil {
			return
		}
	}
	return
}

//删
func (t *Pln_material_demand) Del() bool {
	e := G_DbEngine
	_, err := e.ID(core.PK{G_FINR, t.Materialid, t.Workordernr, t.Demanddate}).Delete(&Pln_material_demand{})
	if err != nil {
		return false
	}
	return true
}

//删除指定订单的物料需求
func (t *Pln_material_demand) DelOrderMaterial(session *xorm.Session, wonr string, matid string) bool {
	_, err := session.Delete(&Pln_material_demand{Finr: G_FINR, Workordernr: wonr, Materialid: matid})
	if err != nil {
		return false
	}
	return true
}

//查
func (t *Pln_material_demand) SelectOne() (data Pln_material_demand, err error) {
	e := G_DbEngine
	_, err = e.ID(core.PK{G_FINR, t.Materialid, t.Workordernr, t.Demanddate}).Get(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}

//改
func (t *Pln_material_demand) Update() bool {
	e := G_DbEngine
	_, err := e.ID(core.PK{G_FINR, t.Materialid, t.Workordernr, t.Demanddate}).Update(t)
	if err != nil {
		return false
	}
	return true
}

//更新指定字段
func (t *Pln_material_demand) UpdateFields(session *xorm.Session, fields string) (err error) {
	if _, err = session.ID(core.PK{G_FINR, t.Materialid, t.Workordernr, t.Demanddate}).Cols(fields).Update(t); err != nil {
		return
	}
	return
}

//批量更新信息
func (t *Pln_material_demand) BatchUpdateFields(session *xorm.Session,Workordernrs []string, fields string) (err error) {
	if _, err = session.In("workordernr",Workordernrs).Where("finr = ? and materialid = ? and demanddate =?",G_FINR, t.Materialid,  t.Demanddate).Cols(fields).Update(t); err != nil {
		return
	}
	return
}

//按日汇聚需求
func (t *Pln_material_demand) SelectByMaterialDuedate(matid, ddt string, session *xorm.Session) (datalst []Pln_material_demand, err error) {
	var i int

	if err = session.Where("finr = ? and materialid = ? and demanddate = ?", G_FINR, matid, ddt).OrderBy("workordernr").Find(&datalst);
		err != nil {
		return
	}
	for i = 0; i < len(datalst); i++ {
		datalst[i].Clipped()
	}

	return datalst, nil
}

//按日汇聚需求
func (t *Pln_material_demand) GetUnusedDemand(demandtypes []string) (datalst []Pln_material_demand, err error) {
	var i int

	e := G_DbEngine
	if err = e.Where("finr = ? and usestatus = ?", G_FINR, 0).In("demandtype", demandtypes).OrderBy("materialid, demanddate").Find(&datalst);
		err != nil {
		return
	}
	for i = 0; i < len(datalst); i++ {
		datalst[i].Clipped()
	}

	return datalst, nil
}
