package db

import (
	"errors"
	"github.com/go-xorm/xorm"
	"leit.com/leit_seat_aps/common"
	"strings"
	"time"
	"xorm.io/core"
)

//生产订单表
type Pln_workorder struct {
	Finr          int                      `xorm:"pk" json:"pln_workorder.finr"`
	Workordernr   string                   `xorm:"pk" json:"pln_workorder.workordernr"`
	Custordernr   string                   `json:"pln_workorder.custordernr"`
	Projnr        string                   `json:"pln_workorder.projnr"`
	Orderinfo     string                   `json:"pln_workorder.orderinfo"`
	Supplygroupid string                   `json:"pln_workorder.supplygroupid"`
	Partfamilyid  string                   `json:"pln_workorder.partfamilyid"`
	Partid        string                   `json:"pln_workorder.partid"`
	Status        int                      `json:"pln_workorder.status"`
	Shippable     int                      `json:"pln_workorder.shippable"`
	Pickstatus    int                      `json:"pln_workorder.pickstatus"` // 拣料状态：0=未拣料；1=已拣料
	Mrpstatus     int                      `json:"pln_workorder.mrpstatus"`  // 物料需求计算状态：0=未计算；1=已计算
	Packstatus    int                      `json:"pln_workorder.packstatus"` // 发运包装状态：0=未包装；1=已包装
	Ordertype     string                   `json:"pln_workorder.ordertype"`
	Worklineid    string                   `json:"pln_workorder.worklineid"`
	Preschedseq   float64                  `json:"pln_workorder.preschedseq"`
	Schedseq      int                      `json:"pln_workorder.schedseq"`
	Oemseq        float64                  `json:"pln_workorder.oemseq"`
	Schedbegtime  string                   `json:"pln_workorder.schedbegtime"`
	Swet          string                   `json:"pln_workorder.swet"`
	Planqty       int                      `json:"pln_workorder.planqty"`
	Priority      int                      `json:"pln_workorder.priority"`
	Lastmodif     string                   `json:"pln_workorder.lastmodif"`
	Lastuser      string                   `json:"pln_workorder.lastuser"`
	Credatuz      string                   `json:"pln_workorder.credatuz"`
	Attrlst       []Pln_workorder_atcodlst `xorm:"-"`
	Partlst       []Pln_workorder_partlst  `xorm:"-"`
}

// 生产订单和客户订单的综合视图
type VWorkorderCustorder struct {
	Pln_workorder `xorm:"extends"`
	Pln_custorder `xorm:"extends"`
}

// 生产订单和零件的综合视图
type VWorkorderCustorderPartlst struct {
	Pln_workorder         `xorm:"extends"`
	Pln_custorder         `xorm:"extends"`
	Pln_workorder_partlst `xorm:"extends"`
}

func (t *Pln_workorder) Clipped() {
	common.TrimStruct(t, *t)
}
func (t *Pln_workorder) TableName() string {
	return "pln_workorder"
}

//增
func (t *Pln_workorder) Add() error {
	e := G_DbEngine
	wotab := new(Pln_workorder)
	affw, err := e.Table("pln_workorder").ID(core.PK{G_FINR, t.Workordernr}).Count(wotab)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table("pln_workorder").Insert(t)

	if err != nil {
		return err
	}
	return nil
}

//增
func (t *Pln_workorder) Insert(session *xorm.Session) error {
	wotab := new(Pln_workorder)
	affw, err := session.Table("pln_workorder").ID(core.PK{G_FINR, t.Workordernr}).Count(wotab)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = session.Table("pln_workorder").Insert(t)

	if err != nil {
		return err
	}
	return nil
}

//删
func (t *Pln_workorder) Del() bool {
	e := G_DbEngine
	_, err := e.ID(core.PK{G_FINR, t.Workordernr}).Delete(&Pln_workorder{})
	if err != nil {
		return false
	}
	return true
}

//改
func (t *Pln_workorder) Update(fields string) (err error) {
	e := G_DbEngine
	_, err = e.ID(core.PK{G_FINR, t.Workordernr}).Cols(fields).Update(t)
	if err != nil {
		return
	}
	return
}

//更新指定字段
func (t *Pln_workorder) UpdateFields(session *xorm.Session, fields string) (err error) {
	if _, err = session.ID(core.PK{G_FINR, t.Workordernr}).Cols(fields).Update(t); err != nil {
		return
	}
	return
}

//更新指定字段
func (t *Pln_workorder) UpdateFieldsWo(session *xorm.Session, fields string) (ok int64, err error) {
	ok, err = session.ID(core.PK{G_FINR, t.Workordernr}).Cols(fields).Update(t)
	if err != nil {
		return
	}
	return
}

//根据生产订单状态，改变客户订单
func (t *Pln_workorder) UpdateCustorderByWorkorder(session *xorm.Session, custordernr string) error {
	var plc Pln_workorder
	var co Pln_custorder
	ok, err := session.Table("pln_workorder").Where("finr = ? and custordernr = ? and status < ?", G_FINR, custordernr, common.WO_STATUS_CLOSED).Get(&plc)
	if err != nil {
		return err
	}
	if !ok {
		result, err := session.Table("pln_custorder").Where("finr = ? and custordernr = ?", G_FINR, custordernr).Get(&co)
		if err != nil {
			return err
		}
		if !result {
			return errors.New("没有该记录!")
		}
		if co.Status == common.WO_STATUS_CLOSED {
			return nil
		}
		//生产订单不存在小于80的状态，更新生产订单
		res, err := session.Table("pln_custorder").Cols("status", "lastuser").Where("finr = ? and custordernr = ?", G_FINR, custordernr).Update(&map[string]interface{}{"status": common.WO_STATUS_CLOSED, "lastuser": "asn_service"})
		if err != nil {
			return err
		}
		if res == 0 {
			return errors.New("更新失败!")
		}
	}
	return err
}

//根据客户订单下的GRAMMER生产订单是否有小于80的状态
func (t *Pln_workorder) SelectByCustorder(session *xorm.Session, custordernr string) bool {
	var plc Pln_workorder
	ok, err := session.Table("pln_workorder").Where("finr = ? and custordernr = ? and worklineid = ? and status < ?", G_FINR, custordernr, "GRAMMER", common.WO_STATUS_CLOSED).Get(&plc)
	if err != nil{
		return false
	}
	if ok{
		return false
	}
	return true
}

//查生产订单数据，包括属性和零件清单
func (t *Pln_workorder) SelectOne() (data Pln_workorder, err error) {
	var i int

	e := G_DbEngine
	_, err = e.ID(core.PK{G_FINR, t.Workordernr}).Get(&data)
	if err != nil {
		return data, err
	}
	data.Clipped()

	// 获取属性列表
	if err = e.Where("finr = ? and workordernr = ? ",
		G_FINR, t.Workordernr).OrderBy("pos").Find(&data.Attrlst); err != nil {
		return
	}
	for i, _ = range data.Attrlst {
		data.Attrlst[i].Clipped()
	}

	// 获取零件列表
	if err = e.Where("finr = ? and workordernr = ? ",
		G_FINR, t.Workordernr).OrderBy("partid").Find(&data.Partlst); err != nil {
		return
	}
	for i, _ = range data.Partlst {
		data.Partlst[i].Clipped()
	}

	return data, nil
}

//查生产订单数据，包括属性和零件清单
func (t *Pln_workorder) SelectOneBySession(session *xorm.Session) (data Pln_workorder, err error) {
	var i int

	_, err = session.ID(core.PK{G_FINR, t.Workordernr}).Get(&data)
	if err != nil {
		return data, err
	}
	data.Clipped()

	// 获取属性列表
	if err = session.Where("finr = ? and workordernr = ? ",
		G_FINR, t.Workordernr).OrderBy("pos").Find(&data.Attrlst); err != nil {
		return
	}
	for i, _ = range data.Attrlst {
		data.Attrlst[i].Clipped()
	}

	// 获取零件列表
	if err = session.Where("finr = ? and workordernr = ? ",
		G_FINR, t.Workordernr).OrderBy("partid").Find(&data.Partlst); err != nil {
		return
	}
	for i, _ = range data.Partlst {
		data.Partlst[i].Clipped()
	}

	return data, nil
}

// 查询生产订单的零件清单
func (t *Pln_workorder) GetPartlst() (datalst []Pln_workorder_partlst, err error) {
	var i int

	e := G_DbEngine
	err = e.Where("finr = ? and workordernr = ?", G_FINR, t.Workordernr).OrderBy("partid").Find(&datalst)
	for i, _ = range datalst {
		datalst[i].Clipped()
	}
	return
}

// 查询生产订单的属性清单
func (t *Pln_workorder) GetAttributelst() (datalst []Pln_workorder_atcodlst, err error) {
	var i int

	e := G_DbEngine
	err = e.Where("finr = ? and workordernr = ?", G_FINR, t.Workordernr).OrderBy("pos").Find(&datalst)
	for i, _ = range datalst {
		datalst[i].Clipped()
	}
	return
}

// 为生产订单添加零件清单
func (t *Pln_workorder) AddPartlst(session *xorm.Session, datalst []Pln_workorder_partlst) (err error) {
	var wopartab Pln_workorder_partlst

	for i := 0; i < len(datalst); i++ {
		wopartab = Pln_workorder_partlst{}
		wopartab = datalst[i]
		if _, err = session.Insert(wopartab); err != nil {
			return
		}
	}
	return
}

// 为生产订单添加属性清单
func (t *Pln_workorder) AddAttributelst(session *xorm.Session, datalst []Pln_workorder_atcodlst) (err error) {
	var woattrtab Pln_workorder_atcodlst

	for i := 0; i < len(datalst); i++ {
		woattrtab = Pln_workorder_atcodlst{}
		woattrtab = datalst[i]
		if _, err = session.Insert(woattrtab); err != nil {
			return
		}
	}
	return
}

// 删除生产订单的零件清单
func (t *Pln_workorder) DelPartlst(session *xorm.Session) bool {
	if _, err := session.Exec("DELETE FROM `pln_workorder_partlst` WHERE finr = ? AND workordernr = ?",
		G_FINR, t.Workordernr); err != nil {
		return false
	}
	return true
}

// 删除生产订单的属性清单
func (t *Pln_workorder) DelAttributelst(session *xorm.Session) bool {
	if _, err := session.Exec("DELETE FROM `pln_workorder_atcodlst` WHERE finr = ? AND workordernr = ?",
		G_FINR, t.Workordernr); err != nil {
		return false
	}
	return true
}

// 基于客户订单号和supplygroup检索生产订单，如果有则返回生产订单号
func (t *Pln_workorder) GetByCOAndSG() (exist bool, err error) {
	var wotablst []Pln_workorder

	e := G_DbEngine
	exist = false

	if err = e.Where("finr = ? and custordernr = ? and supplygroupid = ?", G_FINR, t.Custordernr,
		t.Supplygroupid).Desc("credatuz").Limit(1, 0).Find(&wotablst); err != nil {
		return
	}
	if len(wotablst) > 0 {
		t = &wotablst[0]
		t.Clipped()
		exist = true
		return
	}

	return
}

// 基于客户订单号和supplygroup检索生产订单，如果有则返回生产订单号
func (t *Pln_workorder) GetByCOAndSGBySession(session *xorm.Session, custordernr string, sgid string) (exist bool, wotablst []Pln_workorder, err error) {
	var i int

	exist = false

	if err = session.Where("finr = ? and custordernr = ? and supplygroupid = ?", G_FINR, custordernr,
		sgid).Desc("credatuz").Find(&wotablst); err != nil {
		return
	}
	if len(wotablst) > 0 {
		for i = 0; i < len(wotablst); i++ {
			wotablst[i].Clipped()
		}
		exist = true
		return
	}

	return
}

// 查询指定项目和产线的已计划的生产订单(预排序号必须>0）
func (t *Pln_workorder) GetLineProjectPlannedTasks(projnr string, lineid string) (datalst []Pln_workorder, err error) {
	e := G_DbEngine
	if err = e.Where("finr = ? and projnr = ? and worklineid = ? and status = ? and preschedseq > ?", G_FINR, projnr,
		lineid, common.WO_STATUS_PLANNED, 0).Asc("preschedseq", "supplygroupid").Find(&datalst); err != nil {
		return
	}
	return
}

// 查询指定产线的已计划的生产订单
func (t *Pln_workorder) GetLinePlannedTasks(lineid string) (datalst []Pln_workorder, err error) {
	e := G_DbEngine
	if err = e.Where("finr = ? and worklineid = ? and status = ? and preschedseq > ?", G_FINR,
		lineid, common.WO_STATUS_PLANNED, 0).Asc("preschedseq", "supplygroupid").Find(&datalst); err != nil {
		return
	}
	return
}

// 查询指定产线的已计划的生产订单（26）
func (t *Pln_workorder) GetLineReleasedTaskQty(lineid string) (qty int, err error) {
	var totalQty int64

	e := G_DbEngine

	//Ordertype为SEQ状态去重
	if totalQty, err = e.Distinct("oemseq").Where("finr = ? and worklineid = ? and ordertype = ? and status = ?",
		G_FINR, lineid, common.WO_TYPE_SEQ, common.WO_STATUS_RELEASED).Count(t); err != nil {
		return
	}

	qty = int(totalQty)
	return
}

// 查询指定产线的已计划的生产订单（26）
func (t *Pln_workorder) GetLineReleasedTaskQtyBySwet(lineid string) (qty int, err error) {
	var totalQty int64

	e := G_DbEngine

	if totalQty, err = e.Where("finr = ? and worklineid = ? and ordertype = ? and status = ?",
		G_FINR, lineid, common.WO_TYPE_SEQ, common.WO_STATUS_RELEASED).Count(t); err != nil {
		return
	}

	qty = int(totalQty)
	return
}

// 查询指定产线需要下达的生产订单（20）
func (t *Pln_workorder) GetLineUnfinishedTaskQty(lineid string) (qty int, err error) {
	var totalQty int64

	e := G_DbEngine

	//Ordertype为SEQ状态去重
	if totalQty, err = e.Distinct("oemseq").Where("finr = ? and worklineid = ? and ordertype = ? and status >= ? and status < ?",
		G_FINR, lineid, common.WO_TYPE_SEQ, common.WO_STATUS_RELEASED, common.WO_STATUS_CLOSED).Count(t); err != nil {
		return
	}

	qty = int(totalQty)
	return
}

// 查询指定产线需要下达的生产订单（20）
func (t *Pln_workorder) GetLineUnfinishedTaskWorkorders(lineid string, qty int) (wotabs []Pln_workorder, err error) {

	e := G_DbEngine
	wotem := make([]Pln_workorder, 0)
	//Ordertype为SEQ状态去重
	if err = e.Distinct("oemseq").Where("finr = ? and worklineid = ? and status = ? and oemseq < ?",
		G_FINR, lineid, common.WO_STATUS_PLANNED, 9999999999).Asc("oemseq").Limit(qty).Find(&wotem); err != nil {
		return
	}
	var oemseqarr []float64
	for _, v := range wotem {
		oemseqarr = append(oemseqarr, v.Oemseq)
	}
	//Ordertype为SEQ状态去重
	if err = e.Where("finr = ? and worklineid = ? and status = ? and oemseq < ?",
		G_FINR, lineid, common.WO_STATUS_PLANNED, 9999999999).In("oemseq", oemseqarr).Asc("preschedseq", "supplygroupid").Find(&wotabs); err != nil {
		return
	}

	return
}

// 查询指定产线需要下达的生产订单（20）
func (t *Pln_workorder) GetLineProjectUnfinishedTaskWorkorders(projnr string, lineid string, qty int) (wotabs []Pln_workorder, err error) {

	e := G_DbEngine

	wotem := make([]Pln_workorder, 0)
	//Ordertype为SEQ状态去重
	if err = e.Distinct("oemseq").Where("finr = ? and projnr = ? and worklineid = ? and status = ? and oemseq < ?",
		G_FINR, projnr, lineid, common.WO_STATUS_PLANNED, 9999999999).Asc("oemseq").Limit(qty).Find(&wotem); err != nil {
		return
	}
	var oemseqarr []float64
	for _, v := range wotem {
		oemseqarr = append(oemseqarr, v.Oemseq)
	}
	//Ordertype为SEQ状态去重
	if err = e.Where("finr = ? and projnr = ? and worklineid = ?  and status = ? and oemseq < ?",
		G_FINR, projnr, lineid, common.WO_STATUS_PLANNED, 9999999999).In("oemseq", oemseqarr).Asc("preschedseq", "supplygroupid").Find(&wotabs); err != nil {
		return
	}

	return
}

// 查询指定产线需要下达的生产订单（20,基于混线时间swet逻辑）
func (t *Pln_workorder) GetLineProjectUnfinishedTaskWorkordersBySwet(projnr string, lineid string, qty int) (wotabs []Pln_workorder, err error) {

	e := G_DbEngine
	if err = e.Where("finr = ? and projnr = ? and worklineid = ? and status = ?",
		G_FINR, projnr, lineid, common.WO_STATUS_PLANNED).Asc("preschedseq", "supplygroupid").Limit(qty).Find(&wotabs); err != nil {
		return
	}

	return
}

// 查询指定产线需要下达的生产订单（20,基于混线时间swet逻辑）但是有oemseq限制
func (t *Pln_workorder) GetLineProjectUnfinishedTaskWorkordersBySwetLimitOemseq(projnr string, lineid string, qty int, oemseq float64) (wotabs []Pln_workorder, err error) {

	e := G_DbEngine
	if err = e.Where("finr = ? and projnr = ? and worklineid = ? and status = ? and oemseq <= ? and oemseq < ?",
		G_FINR, projnr, lineid, common.WO_STATUS_PLANNED, oemseq, 9999999999).Asc("preschedseq", "supplygroupid").Limit(qty).Find(&wotabs); err != nil {
		return
	}

	return
}

// 查询指定产线需要下达的生产订单（20）,基于swet
func (t *Pln_workorder) GetLineUnfinishedTaskWorkordersBySwet(lineid string) (wotem []Pln_workorder, err error) {

	e := G_DbEngine
	//查询100条数据，并且排序
	if err = e.Where("finr = ? and worklineid = ? and status = ? and oemseq < ?",
		G_FINR, lineid, common.WO_STATUS_PLANNED, 9999999999).Asc("preschedseq", "supplygroupid").Limit(100).Find(&wotem); err != nil {
		return
	}
	return
}

// 查询指定产线的已计划的生产订单
func (t *Pln_workorder) GetLineUnpickedTasksFullData(lineid string) (datalst []Pln_workorder, err error) {
	var (
		i, j, k    int
		attrtablst []Pln_workorder_atcodlst
		parttablst []Pln_workorder_partlst
	)

	e := G_DbEngine
	if err = e.Where("finr = ? and worklineid = ? and status >= ? and pickstatus = ? and priority < ? and ordertype = ?",
		G_FINR, lineid, common.WO_STATUS_RELEASED, common.PK_STATUS_UNPICKED, 9, "SEQ").Asc("schedseq").Find(&datalst);
		err != nil {
		return
	}

	for i = 0; i < len(datalst); i++ {
		datalst[i].Clipped()
		// 加载属性值列表
		attrtablst = []Pln_workorder_atcodlst{}
		if err = e.Where("finr = ? and workordernr = ?", G_FINR, datalst[i].Workordernr).OrderBy("pos").Find(&attrtablst); err != nil {
			return
		}
		for j, _ = range attrtablst {
			attrtablst[j].Clipped()
		}
		datalst[i].Attrlst = attrtablst
		// 加载零件列表
		parttablst = []Pln_workorder_partlst{}
		if err = e.Where("finr = ? and workordernr = ?", G_FINR, datalst[i].Workordernr).OrderBy("partid").Find(&parttablst); err != nil {
			return
		}
		for k, _ = range parttablst {
			parttablst[k].Clipped()
		}
		datalst[i].Partlst = parttablst
	}

	return
}

// 查询mrpstatus = 0的生产订单
func (t *Pln_workorder) GetUndemandedData() (datalst []Pln_workorder, err error) {
	var (
		i, j       int
		parttablst []Pln_workorder_partlst
	)

	e := G_DbEngine
	if err = e.Where("finr = ? and mrpstatus = ?",
		G_FINR, common.WO_STATUS_UNRUNMRP).OrderBy("workordernr").Find(&datalst);
		err != nil {
		return
	}

	for i = 0; i < len(datalst); i++ {
		datalst[i].Clipped()
		// 加载零件列表
		parttablst = []Pln_workorder_partlst{}
		if err = e.Where("finr = ? and workordernr = ?", G_FINR, datalst[i].Workordernr).OrderBy("partid").Find(&parttablst); err != nil {
			return
		}
		for j, _ = range parttablst {
			parttablst[j].Clipped()
		}
		datalst[i].Partlst = parttablst
	}

	return
}

// 设置生产订单的MRP运行状态（零件需求计算）
func (t *Pln_workorder) SetRunMrpStatusBySession(session *xorm.Session, mrpstatus int) (err error) {
	t.Mrpstatus = mrpstatus
	t.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	if err = t.UpdateFields(session, "mrpstatus, lastmodif"); err != nil {
		return
	}
	return
}

//获取指定同步号包装单关联的客户订单
func (t *Pln_workorder) GetListByOemseqAndSupplygroup(session *xorm.Session, projnr string, oemseqlist []int, sglist []string) (datalst []Pln_workorder, err error) {
	var i int

	if err = session.Where("finr = ? and projnr = ?", G_FINR, projnr).In("oemseq",
		oemseqlist).In("supplygroupid", sglist).Find(&datalst); err != nil {
		return
	}
	for i = 0; i < len(datalst); i++ {
		datalst[i].Clipped()
	}

	return
}

// 获取指定OEMSET和SGID的工单
func (t *Pln_workorder) GetByProjectOemSeqSg(session *xorm.Session, projnr string, oemseq int, sgid string) (exist bool, data Pln_workorder, err error) {
	var (
		oemseqf float64
		datalst []Pln_workorder
	)

	oemseqf = common.ValueToFloat(oemseq, 0.0)
	exist = false
	if err = session.Where("finr = ? and projnr = ? and oemseq = ? and supplygroupid = ?", G_FINR, projnr, oemseqf, sgid).Desc("credatuz").Find(&datalst); err != nil {
		return
	}
	if len(datalst) > 0 {
		exist = true
		data = datalst[0]
		return
	}

	return
}

//获取已下达生产订单及其客户订单信息
func (t *Pln_workorder) GetReleasedWorkorder(projnr string) (vdatalst []VWorkorderCustorder, err error) {
	var i int

	e := G_DbEngine
	query := e.Table("pln_workorder")
	query = query.Join("INNER", "pln_custorder", "pln_workorder.finr = pln_custorder.finr and pln_workorder.custordernr = pln_custorder.custordernr")
	query = query.Where("pln_workorder.finr = ? and pln_workorder.projnr = ? and pln_workorder.status = ?", G_FINR, projnr, common.WO_STATUS_RELEASED)

	if err = query.Find(&vdatalst); err != nil {
		return
	}
	for i = 0; i < len(vdatalst); i++ {
		vdatalst[i].Pln_workorder.Clipped()
		vdatalst[i].Pln_custorder.Clipped()
	}

	return
}

//获取生产订单某属性的位置值
func (t *Pln_workorder) GetAttributePos(iattrcode int) int {
	var i int

	for i = 0; i < len(t.Attrlst); i++ {
		if iattrcode == t.Attrlst[i].Attrcode {
			return i
		}
	}
	return -1
}

// 查询外部二级件的排序订单数据
func (t *Pln_workorder) GetExternalTier2SEQOrders() (datalst []Pln_workorder, err error) {
	var (
		i, j, k    int
		linelst    []Workline
		lines      []string
		wopartlst  []Pln_workorder_partlst
		woatcodlst []Pln_workorder_atcodlst
	)

	e := G_DbEngine
	// 查询供应商类别的外部产线
	if err = e.Where("finr = ? and linetype = ?",
		G_FINR, common.LINE_TYPE_VENDOR).Find(&linelst);
		err != nil {
		return
	}

	lines = []string{}
	for i = 0; i < len(linelst); i++ {
		lines = append(lines, strings.TrimSpace(linelst[i].Worklineid))
	}

	// 查询关联产线的生产订单
	if err = e.Where("finr = ? and ordertype = ? and status != ?",
		G_FINR, common.WO_TYPE_SEQ, common.WO_STATUS_CLOSED).In("worklineid", lines).OrderBy("workordernr").Find(&datalst);
		err != nil {
		return
	}

	for i = 0; i < len(datalst); i++ {
		datalst[i].Clipped()
		// 加载零件列表
		wopartlst = []Pln_workorder_partlst{}
		if err = e.Where("finr = ? and workordernr = ?", G_FINR, datalst[i].Workordernr).OrderBy("partid").Find(&wopartlst); err != nil {
			return
		}
		for j, _ = range wopartlst {
			wopartlst[j].Clipped()
		}
		datalst[i].Partlst = wopartlst

		// 加载属性列表
		woatcodlst = []Pln_workorder_atcodlst{}
		if err = e.Where("finr = ? and workordernr = ?", G_FINR, datalst[i].Workordernr).OrderBy("pos").Find(&woatcodlst); err != nil {
			return
		}
		for k, _ = range woatcodlst {
			woatcodlst[k].Clipped()
		}
		datalst[i].Attrlst = woatcodlst
	}

	return
}

// 查询外部二级件的非排序订单数据, 不考虑取消的订单，指考虑未完成的订单
func (t *Pln_workorder) GetExternalTier2NOSEQOrders() (datalst []Pln_workorder, err error) {
	var (
		i, j, k           int
		linelst           []Workline
		lines, workorders []string
		wopartlst         []Pln_workorder_partlst
		woatcodlst        []Pln_workorder_atcodlst
	)

	e := G_DbEngine
	// 查询供应商类别的外部产线
	if err = e.Where("finr = ? and linetype = ?",
		G_FINR, common.LINE_TYPE_VENDOR).Find(&linelst);
		err != nil {
		return
	}

	lines = []string{}
	for i = 0; i < len(linelst); i++ {
		lines = append(lines, strings.TrimSpace(linelst[i].Worklineid))
	}

	// 查询关联产线的生产订单
	if err = e.Where("finr = ? and ordertype != ? and status = ?",
		G_FINR, common.WO_TYPE_SEQ, common.WO_STATUS_UNPLANED).In("worklineid", lines).OrderBy("workordernr").Find(&datalst);
		err != nil {
		return
	}
	for i = 0; i < len(datalst); i++ {
		datalst[i].Clipped()
		//获取workorder集合
		workorders = append(workorders, datalst[i].Workordernr)
		// 加载零件列表
		wopartlst = []Pln_workorder_partlst{}
		if err = e.Where("finr = ? and workordernr = ?", G_FINR, datalst[i].Workordernr).OrderBy("partid").Find(&wopartlst); err != nil {
			return
		}
		for j, _ = range wopartlst {
			wopartlst[j].Clipped()
		}
		datalst[i].Partlst = wopartlst

		// 加载属性列表
		woatcodlst = []Pln_workorder_atcodlst{}
		if err = e.Where("finr = ? and workordernr = ?", G_FINR, datalst[i].Workordernr).OrderBy("pos").Find(&woatcodlst); err != nil {
			return
		}
		for k, _ = range woatcodlst {
			woatcodlst[k].Clipped()
		}
		datalst[i].Attrlst = woatcodlst
	}
	//更新对应的workordernrs
	if _, err = e.Table("pln_workorder").In("workordernr", workorders).Where("finr = ? ", G_FINR).Update(map[string]interface{}{"status": common.WO_STATUS_PLANNED}); err != nil {
		return
	}
	return
}

func (t *Pln_workorder) GetAttributeValue(iatcod int) string {
	var i int

	for i = 0; i < len(t.Attrlst); i++ {
		if t.Attrlst[i].Attrcode == iatcod {
			return t.Attrlst[i].Attrvalue
		}
	}

	return ""
}

// 基于Grammer ASN信息获取对应的生产订单
func (t *Pln_workorder) GetListByASN(session *xorm.Session, projnr string, oemseqlist []int, sglist []string) (datalst []Pln_workorder, err error) {
	var i int

	if err = session.Where("finr = ? and projnr = ?", G_FINR, projnr).In("oemseq",
		oemseqlist).In("supplygroupid", sglist).Find(&datalst); err != nil {
		return
	}
	for i = 0; i < len(datalst); i++ {
		datalst[i].Clipped()
	}

	return
}
