package mes

import (
	"fmt"
	"github.com/go-xorm/xorm"
	"leit.com/leit_seat_aps/common"
	"leit.com/leit_seat_aps/db"
	"leit.com/leit_seat_aps/service"
	"testing"
	"time"
)

// 测试基于派工下达计划生成生产包装
func TestGenProductionPack(t *testing.T) {
	var (
		connstring   	string
		err          	error
		i, m, n 		int
		projtab      	db.Prod_project
		projtablst   	[]db.Prod_project
		pproj 			service.ProdProject
		wotab			db.Pln_workorder
		wotablst		[]db.Pln_workorder
		sgid,sortway	string
		exist, ok, relstat    	bool
		bl_pakordlst 	[]service.BL_ProdPackOrder
		bl_pakord		service.BL_ProdPackOrder
		session      	*xorm.Session
	)

	// 连接数据库
	connstring = fmt.Sprintf("server=%s;user id=%s;password=%s;database=%s;port=%d;encrypt=disable",
		`localhost`, "lapp", "123fis", "SJA_APS", 1433)
	if err = db.InitMssqlDb(connstring); err != nil {
		t.Errorf("Failed to connect db due to: %v", err)
		return
	}
	defer db.G_DbEngine.Close()

	projtab = db.Prod_project{}
	if projtablst, err = projtab.GetAllActive(); err != nil {
		t.Errorf("Failed to get project list!")
		return
	}

	for i, _ = range projtablst {
		pproj 				= service.ProdProject{}
		pproj.Projecttab 	= projtablst[i]
		pproj.Projectid 	= projtablst[i].Projectid
		fmt.Println("Ready to load data for project:", projtablst[i].Projectid)

		// 加载项目主数据
		if err = pproj.LoadMasterData(); err != nil {
			t.Errorf("Failed to load project master data due to: %v!", err)
			return
		}

		// 从指定起始OEMSEQ获取未进行生产包装的工单列表
		if wotablst, err = pproj.GetUnpackedWorkorderListByOemseq(); err != nil {
			t.Errorf("获取项目%s指定供应组%s的工单失败: %v!",pproj.Projectid, sgid, err)
			return
		}

		// 遍历工单
		for _, wotab = range wotablst {
			fmt.Println("==========匹配工单==>",wotab.Workordernr,"--OEMSEQ--",common.ValueToInt(wotab.Oemseq, 0), "--SupplyGroup--",wotab.Supplygroupid)
			// 获取工单供应组对应的排序方式
			sortway = pproj.GetSupplyGroupPackSortWay(wotab.Supplygroupid)
			switch sortway {
			case common.MESPACK_SORT_OEMSEQ:
				// 基于OEMSEQ和工单的供应组ID查找包装单
				if exist, bl_pakord, err = pproj.GetOpenPackOrderByOemseqSupplyGroup(wotab); err != nil{
					t.Errorf("无法获取工单 %s 的包装订单，原因是：%v", wotab.Workordernr, err)
					return
				}
			case common.MESPACK_SORT_RELSEQ:
				// 基于空包装单及supplygroup
				if exist, bl_pakord, err = pproj.GetOpenPackOrderBySupplyGroup(wotab.Supplygroupid); err != nil{
					t.Errorf("无法获取工单 %s 的包装订单，原因是：%v", wotab.Workordernr, err)
					return
				}
			}

			session = db.G_DbEngine.NewSession()
			defer session.Close()
			if err = session.Begin(); err != nil {
				return
			}
			// 不存在则新建包装单
			if !exist {
				if bl_pakordlst, err = pproj.CreatePackOrderByOemseq(session, wotab, sortway); err != nil {
					fmt.Println("创建开口包装单失败，原因：", err)
					session.Rollback()
					continue
				}
				// 获取当前OEMSEQ 或 checksequence的包装单
				ok = false
				for m = 0; m < len(bl_pakordlst); m++ {
					for n = 0; n < len(bl_pakordlst[m].Pkotab.Itemlst); n++ {
						switch sortway {
						case common.MESPACK_SORT_OEMSEQ:
							if (bl_pakordlst[m].Pkotab.Itemlst[n].Supplygroupid == wotab.Supplygroupid) && bl_pakordlst[m].Pkotab.Itemlst[n].Oemseq == common.ValueToInt(wotab.Oemseq, 0) {
								ok = true
								bl_pakord = bl_pakordlst[m]
								break
							}
						case common.MESPACK_SORT_RELSEQ:
							if bl_pakordlst[m].Pkotab.Itemlst[n].Supplygroupid == wotab.Supplygroupid {
								ok = true
								bl_pakord = bl_pakordlst[m]
								break
							}
						}
					}
					if ok {
						break
					}
				}
				if !ok {
					fmt.Println("无法为当前工单找到包装单！")
					session.Rollback()
					continue
				}
			}

			// 将工单加入到当前包装单并尝试下达包装单
			if ok, err = pproj.AddWOToPackOrder(wotab, &bl_pakord, sortway, session); err != nil{
				t.Errorf("添加工单 %s 到包装单 %s 出错，原因是 %v", wotab.Workordernr,bl_pakord.PackorderId, err)
				session.Rollback()
				return
			}

			// 添加成功尝试下达生产包装单
			if ok {
				fmt.Println("成功添加工单",wotab.Workordernr,"到包装单",bl_pakord.PackorderId)
				// 关闭当前包装
				if relstat, err = pproj.ReleasePackOrder(session, bl_pakord); err != nil{
					t.Errorf("下达包装单 %s 出错，原因是 %v", bl_pakord.PackorderId, err)
					session.Rollback()
					return
				}
				if relstat{
					// 生成包装单消息
					if err = pproj.CreatePackOrderRelMsg(bl_pakord.PackorderId, session); err != nil{
						t.Errorf("创建包装单下达消息 %s 出错，原因是 %v", bl_pakord.PackorderId, err)
						session.Rollback()
						return
					}
					fmt.Println("成功下达包装单",bl_pakord.PackorderId)
				}
			}else{
				fmt.Println("！！！无法添加工单",wotab.Workordernr,"到包装单",bl_pakord.PackorderId)
			}

			session.Commit()
		}
	}
}

// 测试基于包装下达消息解析并生成包装单打印数据
func TestParsePackReleaseMsg(t *testing.T) {
	var (
		connstring   	string
		err          	error
		//session      	*xorm.Session
	)

	// 连接数据库
	connstring = fmt.Sprintf("server=%s;user id=%s;password=%s;database=%s;port=%d;encrypt=disable",
		`localhost`, "lapp", "123fis", "SJA_APS", 1433)
	if err = db.InitMssqlDb(connstring); err != nil {
		t.Errorf("Failed to connect db due to: %v", err)
		return
	}
	defer db.G_DbEngine.Close()

	// 加载打印所需模板数据

	// 启动解析循环
	for {
		fmt.Println("准备解析生产包装单打印消息！")
		/**if err = ParseOpenPrintMsg(&pe); err != nil {
			t.Errorf("解析生产包装单打印消息失败：%v", err)
			return
		}**/
		fmt.Println("解析生产包装单打印消息完成！")
		time.Sleep(10 * time.Second)
	}

	return
}
