package msg

import (
	"fmt"
	"leit.com/leit_seat_aps/db"
	"leit.com/leit_seat_aps/service"
	"testing"
	"time"
)

func TestMsg(t *testing.T) {
	var (
		connstring string
		err        error
		pe         service.BL_PrintEngine
	)

	// 连接数据库
	connstring = fmt.Sprintf("server=%s;user id=%s;password=%s;database=%s;port=%d;encrypt=disable",
		`DESKTOP-S4G95G5\\MLINK`, "lapp", "123fis", "LAPP_JITS", 1433)
	if err = db.InitMssqlDb(connstring); err != nil {
		t.Errorf("Failed to connect db due to: %v", err)
		return
	}
	defer db.G_DbEngine.Close()

	// 加载打印引擎所需模板数据
	pe = service.BL_PrintEngine{}
	if err = pe.LoadPickingTemplates(); err != nil {
		t.Errorf("加载拣料单模板失败，原因是：%v", err)
		return
	}
	if err = pe.LoadPackageTemplate(); err != nil {
		t.Errorf("加载包装单模板失败，原因是：%v", err)
		return
	}
	if err = pe.LoadShipCarTemplate(); err != nil {
		t.Errorf("加载发运单模板失败，原因是：%v", err)
		return
	}
	if err = pe.LoadAttributes(); err != nil {
		t.Errorf("加载属性主数据失败，原因是：%v", err)
		return
	}

	// 启动解析循环
	for {
		fmt.Println("准备解析打印消息！")
		if err = ParseOpenPrintMsg(&pe); err != nil {
			t.Errorf("解析打印消息失败：%v", err)
			return
		}
		fmt.Println("解析完成！")
		time.Sleep(10 * time.Second)
	}

	return
}

func TestEdiNew(t *testing.T) {
	var (
		connstring string
		err        error
		filefolder string
	)

	// 连接数据库
	connstring = fmt.Sprintf("server=%s;user id=%s;password=%s;database=%s;port=%d;encrypt=disable",
		`DESKTOP-S4G95G5\\MLINK`, "lapp", "123fis", "LAPP_JITS", 1433)
	if err = db.InitMssqlDb(connstring); err != nil {
		t.Errorf("Failed to connect db due to: %v", err)
		return
	}
	defer db.G_DbEngine.Close()

	filefolder = "C:\\MES\\"

	if err = Grammer_GenerateSEQOrderEDIFile(filefolder); err != nil {
		t.Errorf("生成EDI文件失败：%v", err)
	}

	if err = Grammer_GenerateNOSEQOrderEDIFile(filefolder); err != nil {
		t.Errorf("生成EDI文件失败：%v", err)
	}
}

func TestAsn(t *testing.T) {
	var (
		connstring string
		err        error
	)

	// 连接数据库
	connstring = fmt.Sprintf("server=%s;user id=%s;password=%s;database=%s;port=%d;encrypt=disable",
		`DESKTOP-S4G95G5\\MLINK`, "lapp", "123fis", "LAPP_JITS", 1433)
	if err = db.InitMssqlDb(connstring); err != nil {
		t.Errorf("Failed to connect db due to: %v", err)
		return
	}
	defer db.G_DbEngine.Close()

	if err = ParseShipOrderMsg(); err != nil {
		t.Errorf("解析发运单消息失败，无法生成ASN文件: %v", err)
		return
	}
}
