package service

import (
	"errors"
	"fmt"
	"github.com/go-xorm/xorm"
	"leit.com/leit_seat_aps/common"
	"leit.com/leit_seat_aps/db"
	"leit.com/leit_seat_aps/glog"
	"strconv"
	"strings"
	"time"
)

// 客户订单业务对象
type BL_CustOrder struct {
	Custordernr    string // 系统内部客户订单号
	Oemordernr     string // 客户订单号
	Projnr         string // 项目号
	Partfamilyid   string
	SeqType        int    // 3=新建；1=变更；2=取消
	Parsed         int    // 0=未解析；1=已解析；2=需要解析
	Edifile        string // EDI 文件名
	Carmodelid     string
	Ordertime      time.Time
	Swet           time.Time
	Bl_sgdict      map[string]BL_SupplyGroup // 供应组业务对象字典(未分配共用件）
	Bl_usgdict     map[string]BL_SupplyGroup // 分配完共用供应组零件后的供应组清单
	Bl_wodict      map[string]BL_WorkOrder   // 生产订单业务对象
	Bl_ordattrdict map[int]BL_OrdAttribute   // 客户订单属性清单字典
	Bl_ordpartdict map[string]BL_SgPart      // 客户订单零件清单字典
	Bl_verrdict    map[string]BL_VerifyError // 校验报错信息
	Custordertab   db.Pln_custorder          // 客户订单记录本身
}

// 创建客户订单头
func (bl_co *BL_CustOrder) CreateCOHead(session *xorm.Session, cosnr string) (err error) {
	var (
		cotab db.Pln_custorder
		conr  string
	)

	// 基于设置获取客户订单号
	if strings.TrimSpace(cosnr) != "" {
		if conr, err = SN_GetNextSnrBySession(strings.TrimSpace(cosnr), session); err != nil {
			return
		}
	} else {
		if conr, err = SN_GetNextSnrBySession("CUSTORDER", session); err != nil {
			return
		}
	}

	// 创建客户订单头
	cotab = db.Pln_custorder{}
	cotab.Finr = db.G_FINR
	cotab.Custordernr = conr
	cotab.Oemordernr = bl_co.Oemordernr
	cotab.Partfamilyid = bl_co.Partfamilyid
	cotab.Projnr = bl_co.Projnr
	cotab.Status = common.CO_STATUS_UNPLANED
	cotab.Handlestatus = "IT" // 初始化
	cotab.Ordertime = common.Date(bl_co.Ordertime.Unix(), "YYYYMMDDHHmmss")
	cotab.Swet = common.Date(bl_co.Swet.Unix(), "YYYYMMDDHHmmss")
	cotab.Lastuser = "service"
	cotab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	if err = cotab.Insert(session); err != nil {
		return
	}
	/**if _, err = session.Insert(&cotab); err != nil {
		return err
	}**/

	bl_co.Custordernr = conr
	bl_co.Custordertab = cotab

	return
}

// 获取客户订单的供应组零件清单
// 从客户订单的sgpartlst中读取各供应组的零件数据
func (bl_co *BL_CustOrder) GetSupplyGroupPartList(tp *TodProject) (err error) {
	var (
		cotab         db.Pln_custorder
		cosgpartablst []db.Pln_custorder_supplygroup_partlst
		i             int
		ok            bool
		verr          BL_VerifyError
		bl_sg         BL_SupplyGroup
		bl_sgpt       BL_SgPart
		cosgptlst     []db.Pln_custorder_supplygroup_partlst
	)

	cotab = db.Pln_custorder{Custordernr: bl_co.Custordernr}
	if cosgpartablst, err = cotab.GetSGPartlst(); err != nil {
		return
	}

	// 初始化客户订单的供应组字典
	bl_co.Bl_sgdict = make(map[string]BL_SupplyGroup)

	// 基于客户订单版本的供应组零件数据创建供应组并将零件分配到供应组
	for i = 0; i < len(cosgpartablst); i++ {
		// 判断供应组是否有效
		if _, ok = tp.Supplygrpdict[strings.TrimSpace(cosgpartablst[i].Supplygroupid)]; !ok {
			// SupplyGroup不存在
			verr = BL_VerifyError{}
			verr.Errortype = common.CO_VERIFY_SUPPLYGROUP_NOTEXIST
			verr.Supplygroupid = cosgpartablst[i].Supplygroupid
			verr.Errorinfo = fmt.Sprintf("Supplygroup: %s was not existing!", cosgptlst[i].Supplygroupid)
			bl_co.Bl_verrdict[verr.GetDictKey()] = verr
			continue
		}

		// 判断供应组在当前客户订单是否存在
		if bl_sg, ok = bl_co.Bl_sgdict[strings.TrimSpace(cosgpartablst[i].Supplygroupid)]; !ok {
			bl_sg = BL_SupplyGroup{}
			// 从全局供应组变量复制供应组的零件和属性信息
			if bl_sg, err = tp.CopySupplyGroup(cosgpartablst[i].Supplygroupid); err != nil {
				return
			}
			bl_sg.Bl_sgpartDict = make(map[string]BL_SgPart)
		}

		// 忽略数量为0的零件
		if cosgpartablst[i].Partqty <= 0 {
			continue
		}

		// 将零件加入到客户订单的供应组
		bl_sgpt = BL_SgPart{}
		bl_sgpt.Supplygroupid = cosgpartablst[i].Supplygroupid
		bl_sgpt.Originalsgid = cosgpartablst[i].Supplygroupid
		bl_sgpt.Partid = cosgpartablst[i].Partid
		bl_sgpt.Qty = cosgpartablst[i].Partqty
		bl_sg.Bl_sgpartDict[bl_sgpt.Partid] = bl_sgpt

		// 将供应组更新回客户订单
		bl_co.Bl_sgdict[bl_sg.Supplygroupid] = bl_sg
	}

	return
}

// 基于客户订单的供应组零件清单获取它的完整零件清单
func (bl_co *BL_CustOrder) GetPartListFromSupplyGroup(tp *TodProject) (err error) {
	var (
		bl_sg   BL_SupplyGroup
		bl_sgpt BL_SgPart
		verr    BL_VerifyError
		ok      bool
	)

	// 初始化客户订单的零件字典
	bl_co.Bl_ordpartdict = make(map[string]BL_SgPart)

	// 遍历客户订单的SupplyGroup，将其零件添加到订单零件列表中
	for _, bl_sg = range bl_co.Bl_usgdict {
		// 遍历SG的零件列表
		for _, bl_sgpt = range bl_sg.Bl_sgpartDict {
			if _, ok = tp.Partdict[bl_sgpt.Partid]; !ok {
				// 零件不存在
				verr = BL_VerifyError{}
				verr.Errortype = common.CO_VERIFY_PART_NOTEXIST
				verr.Partid = bl_sgpt.Partid
				verr.Supplygroupid = bl_sg.Supplygroupid
				verr.Errorinfo = fmt.Sprintf("Part: %s in Supplygroup: %s was not existing!", bl_sgpt.Partid, bl_sg.Supplygroupid)
				bl_co.Bl_verrdict[verr.GetDictKey()] = verr
				continue
			}
			// 将零件添加到客户订单的零件字典中
			bl_co.Bl_ordpartdict[bl_sgpt.Partid] = bl_sgpt
		}
	}

	return
}

// 基于客户订单的零件清单获取它的完整属性清单
func (bl_co *BL_CustOrder) GetAtcodListFromPartList(tp *TodProject) (err error) {
	var (
		bl_sg       BL_SupplyGroup
		bl_sgpt     BL_SgPart
		bl_pt       BL_Part
		bl_partattr BL_PartAttribute
		bl_ordattr  BL_OrdAttribute
		verr        BL_VerifyError
		ok          bool
	)

	// 初始化客户订单的属性字典
	bl_co.Bl_ordattrdict = make(map[int]BL_OrdAttribute)

	// 遍历客户订单的零件清单，判断零件是否存在
	for _, bl_sgpt = range bl_co.Bl_ordpartdict {
		if bl_pt, ok = tp.Partdict[bl_sgpt.Partid]; !ok {
			// 零件不存在
			verr = BL_VerifyError{}
			verr.Errortype = common.CO_VERIFY_PART_NOTEXIST
			verr.Partid = bl_sgpt.Partid
			verr.Supplygroupid = bl_sgpt.Supplygroupid
			verr.Errorinfo = fmt.Sprintf("Part: %s in Supplygroup: %s was not existing!", bl_sgpt.Partid, bl_sg.Supplygroupid)
			bl_co.Bl_verrdict[verr.GetDictKey()] = verr
			continue
		}
		// 遍历零件属性并添加到客户订单的属性列表中
		for _, bl_partattr = range bl_pt.AtcodDict {
			if bl_ordattr, ok = bl_co.Bl_ordattrdict[bl_partattr.Attrcode]; !ok {
				bl_ordattr = BL_OrdAttribute{}
				bl_ordattr.Attrcode = bl_partattr.Attrcode
				bl_ordattr.Attrvaldict = make(map[string]string)
			}
			bl_ordattr.Attrvaldict[bl_partattr.Attrvalue] = bl_partattr.Attrvalue
			bl_co.Bl_ordattrdict[bl_partattr.Attrcode] = bl_ordattr

			continue
		}
	}

	return
}

// 将客户订单的共用SG的零件根据规则分配到其它SG中
func (bl_co *BL_CustOrder) AssignNeutralPart(tp *TodProject) (err error) {
	var (
		bl_sg           BL_SupplyGroup
		bl_pt           BL_Part
		sgpt            BL_SgPart
		sgptnew         BL_SgPart
		verr            BL_VerifyError
		bl_part_rule    BL_PartAssignRule
		partruledatatab db.Me_partrule_qtyassign
		key             string
		ok              bool
	)

	// 遍历客户订单的共用的SupplyGroup，将其零件先分配到对应的SupplyGroup中
	for _, bl_sg = range bl_co.Bl_usgdict {
		if _, ok = tp.Supplygrpdict[strings.TrimSpace(bl_sg.Supplygroupid)]; !ok {
			// SupplyGroup不存在
			verr = BL_VerifyError{}
			verr.Errortype = common.CO_VERIFY_SUPPLYGROUP_NOTEXIST
			verr.Supplygroupid = bl_sg.Supplygroupid
			verr.Errorinfo = fmt.Sprintf("Supplygroup: %s was not existing!", bl_sg.Supplygroupid)
			bl_co.Bl_verrdict[verr.GetDictKey()] = verr
			continue
		}

		// 如果不是共用件SupplyGroup则跳过
		if tp.Supplygrpdict[strings.TrimSpace(bl_sg.Supplygroupid)].Sgtype != common.SG_TYPE_NEUTRAL {
			continue
		}
		// 遍历零件进行分配
		for _, sgpt = range bl_sg.Bl_sgpartDict {
			// 零件分配规则：零件ID+数量
			key = strings.TrimSpace(sgpt.Partid) + "+" + strconv.Itoa(sgpt.Qty)
			if bl_part_rule, ok = tp.Partruledict[key]; !ok {
				if sgpt.Qty > 1 {
					// 生成错误提示信息挂载到客户订单的错误信息列表中
					verr = BL_VerifyError{}
					verr.Errortype = common.CO_VERIFY_PARTRULE_NOTEXIST
					verr.Partid = sgpt.Partid
					verr.Supplygroupid = bl_sg.Supplygroupid
					verr.Errorinfo = fmt.Sprintf("Neutral Part: %s with Qty: %d assignment rule was not existing!", sgpt.Partid, sgpt.Qty)
					bl_co.Bl_verrdict[verr.GetDictKey()] = verr
				} else {
					// 对于没有属性分配的成本号和小件包零件不做处理
					if bl_pt, ok = tp.Partdict[sgpt.Partid]; !ok {
						// 报错零件不存在
						verr = BL_VerifyError{}
						verr.Errortype = common.CO_VERIFY_PART_NOTEXIST
						verr.Partid = sgpt.Partid
						verr.Supplygroupid = bl_sg.Supplygroupid
						verr.Errorinfo = fmt.Sprintf("Part: %s in SupplyGroup: %s was not existing!", sgpt.Partid, bl_sg.Supplygroupid)
						bl_co.Bl_verrdict[verr.GetDictKey()] = verr
					} else {
						if len(bl_pt.AtcodDict) > 0 {
							// 报错零件分配规则不存在
							verr = BL_VerifyError{}
							verr.Errortype = common.CO_VERIFY_PARTRULE_NOTEXIST
							verr.Partid = sgpt.Partid
							verr.Supplygroupid = bl_sg.Supplygroupid
							verr.Errorinfo = fmt.Sprintf("Neutral Part: %s with Qty: %d assignment rule was not existing!", sgpt.Partid, sgpt.Qty)
							bl_co.Bl_verrdict[verr.GetDictKey()] = verr
						}
					}
				}
				continue
			}
			// 存在则按照指定的SupplyGroup分配
			for _, partruledatatab = range bl_part_rule.Assignnruledict {
				// 判断零件分配规则指定的SupplyGroup是否存在，不存在则报错
				if _, ok = tp.Supplygrpdict[partruledatatab.Supplygroupid]; !ok {
					// 生成错误提示信息挂载到客户订单的错误信息列表中
					verr = BL_VerifyError{}
					verr.Errortype = common.CO_VERIFY_SUPPLYGROUP_NOTEXIST
					verr.Supplygroupid = partruledatatab.Supplygroupid
					verr.Errorinfo = fmt.Sprintf("Neutral Part: %s with Qty: %d assignment rule's supplygroup: %s was not existing!",
						partruledatatab.Partid, partruledatatab.Assignqty, partruledatatab.Supplygroupid)
					bl_co.Bl_verrdict[verr.GetDictKey()] = verr
					continue
				}
				// 判断零件分配规则指定的SupplyGroup在当前客户订单中是否存在，不存在则跳过
				if _, ok = bl_co.Bl_usgdict[partruledatatab.Supplygroupid]; !ok {
					// 生成错误提示信息挂载到客户订单的错误信息列表中
					continue
				}
				// 判断供应组是否没有零件，没有相当于取消，则不再分配
				if bl_co.SeqType == 1 && len(bl_co.Bl_usgdict[partruledatatab.Supplygroupid].Bl_sgpartDict) <= 0 {
					continue
				}

				// 存在则将分配数量更新到客户订单对应的SupplyGroup中
				sgptnew = BL_SgPart{}
				sgptnew.Partid = partruledatatab.Partid
				sgptnew.Qty = partruledatatab.Assignqty
				sgptnew.Supplygroupid = partruledatatab.Supplygroupid // 保留共用件原供应组ID信息
				sgptnew.Originalsgid = bl_sg.Supplygroupid
				bl_co.Bl_usgdict[sgptnew.Supplygroupid].Bl_sgpartDict[sgptnew.Partid] = sgptnew
			}
		}
	}

	return
}

// 将客户订单的不生成工单的SG的零件根据规则分配到其它SG中
func (bl_co *BL_CustOrder) AssignPartsToShippableSG(tp *TodProject) (err error) {
	var (
		bl_cosg     BL_SupplyGroup
		bl_sg       BL_SupplyGroup
		bl_asnsg    BL_SupplyGroup
		me_sgasntab db.Me_supplygroup_assignment
		sgpt        BL_SgPart
		sgptnew     BL_SgPart
		verr        BL_VerifyError
		ok          bool
	)

	// 遍历客户订单的共用的SupplyGroup，将其零件先分配到对应的SupplyGroup中
	for _, bl_cosg = range bl_co.Bl_usgdict {
		if bl_sg, ok = tp.Supplygrpdict[strings.TrimSpace(bl_cosg.Supplygroupid)]; !ok {
			// SupplyGroup不存在
			verr = BL_VerifyError{}
			verr.Errortype = common.CO_VERIFY_SUPPLYGROUP_NOTEXIST
			verr.Supplygroupid = bl_cosg.Supplygroupid
			verr.Errorinfo = fmt.Sprintf("Supplygroup: %s was not existing!", bl_cosg.Supplygroupid)
			bl_co.Bl_verrdict[verr.GetDictKey()] = verr
			continue
		}

		// 跳过特殊供应组
		if bl_sg.Sgtype != common.SG_TYPE_NORMAL {
			continue
		}
		if bl_sg.Shippablesg > 0 {
			continue
		}
		if len(bl_sg.SgAssigndict) <= 0 {
			continue
		}
		// 遍历零件进行分配
		for _, sgpt = range bl_cosg.Bl_sgpartDict {
			for _, me_sgasntab = range bl_sg.SgAssigndict {
				bl_asnsg, ok = bl_co.Bl_usgdict[me_sgasntab.Fsupplygroupid]
				// 对于SEQ+1通用件分配的供应组可能因为工单取消而不存在
				if bl_co.SeqType == 3 && !ok {
					// SupplyGroup不存在
					verr = BL_VerifyError{}
					verr.Errortype = common.CO_VERIFY_SUPPLYGROUP_NOTEXIST
					verr.Supplygroupid = me_sgasntab.Fsupplygroupid
					verr.Errorinfo = fmt.Sprintf("Supplygroup assignment fsupplygroupid: %s was not existing!", me_sgasntab.Fsupplygroupid)
					bl_co.Bl_verrdict[verr.GetDictKey()] = verr
					continue
				}
				// 对于SEQ+1且父组零件数为0的不分配
				if bl_co.SeqType == 1 && len(bl_co.Bl_usgdict[bl_asnsg.Supplygroupid].Bl_sgpartDict) <= 0 {
					continue
				}
				// 存在则将分配数量更新到客户订单对应的SupplyGroup中
				sgptnew = BL_SgPart{}
				sgptnew.Partid = sgpt.Partid
				if me_sgasntab.Assignfactor1 > 0 && me_sgasntab.Assignfactor2 > 0 {
					sgptnew.Qty = sgpt.Qty * me_sgasntab.Assignfactor1 / me_sgasntab.Assignfactor2
				} else {
					sgptnew.Qty = sgpt.Qty
				}
				sgptnew.Supplygroupid = bl_asnsg.Supplygroupid
				sgptnew.Originalsgid = bl_cosg.Supplygroupid
				bl_asnsg.Bl_sgpartDict[sgptnew.Partid] = sgptnew
				bl_co.Bl_usgdict[bl_asnsg.Supplygroupid] = bl_asnsg
			}
		}
	}

	return
}

// 基于客户订单SG的零件获取该SG的属性
func (bl_co *BL_CustOrder) GenerateSupplyGroupAtcod(tp *TodProject) (err error) {
	var (
		ok         bool
		verr       BL_VerifyError
		bl_sg      BL_SupplyGroup
		bl_sgpt    BL_SgPart
		bl_pt      BL_Part
		bl_ptattr  BL_PartAttribute
		bl_ordattr BL_OrdAttribute
	)

	// 遍历客户订单供应组
	for _, bl_sg = range bl_co.Bl_usgdict {
		if _, ok = tp.Supplygrpdict[strings.TrimSpace(bl_sg.Supplygroupid)]; !ok {
			// SupplyGroup不存在
			verr = BL_VerifyError{}
			verr.Errortype = common.CO_VERIFY_SUPPLYGROUP_NOTEXIST
			verr.Supplygroupid = bl_sg.Supplygroupid
			verr.Errorinfo = fmt.Sprintf("Supplygroup: %s was not existing!", bl_sg.Supplygroupid)
			bl_co.Bl_verrdict[verr.GetDictKey()] = verr
			continue
		}
		// 如果是共用件SupplyGroup则跳过
		if tp.Supplygrpdict[strings.TrimSpace(bl_sg.Supplygroupid)].Sgtype == common.SG_TYPE_NEUTRAL {
			continue
		}
		// 初始化客户订单供应组的属性字典
		bl_sg.Bl_ordattrdict = make(map[int]BL_OrdAttribute)
		for _, bl_sgpt = range bl_sg.Bl_sgpartDict {
			if bl_pt, ok = tp.Partdict[bl_sgpt.Partid]; !ok {
				// 零件不存在
				verr = BL_VerifyError{}
				verr.Errortype = common.CO_VERIFY_PART_NOTEXIST
				verr.Partid = bl_sgpt.Partid
				verr.Supplygroupid = bl_sg.Supplygroupid
				verr.Errorinfo = fmt.Sprintf("Part: %s in Supplygroup: %s was not existing!", bl_sgpt.Partid, bl_sg.Supplygroupid)
				bl_co.Bl_verrdict[verr.GetDictKey()] = verr
				continue
			}
			// 遍历零件的属性清单
			for _, bl_ptattr = range bl_pt.AtcodDict {
				if bl_ordattr, ok = bl_sg.Bl_ordattrdict[bl_ptattr.Attrcode]; !ok {
					bl_ordattr = BL_OrdAttribute{}
					bl_ordattr.Attrcode = bl_ptattr.Attrcode
					bl_ordattr.Attrvaldict = make(map[string]string)
				}
				bl_ordattr.Attrvaldict[bl_ptattr.Attrvalue] = bl_ptattr.Attrvalue
				bl_sg.Bl_ordattrdict[bl_ptattr.Attrcode] = bl_ordattr
			}
		}
		// 将供应组零件加入到客户订单
		bl_co.Bl_usgdict[bl_sg.Supplygroupid] = bl_sg
	}

	return
}

// 验证客户订单的零件是否存在
func (bl_co *BL_CustOrder) ValidatePartList(tp *TodProject) (err error) {
	var (
		sg   BL_SupplyGroup
		sgpt BL_SgPart
		verr BL_VerifyError
		ok   bool
	)

	// 遍历客户订单下的每一个SupplyGroup中的每一个零件，校验零件是否存在
	for _, sg = range bl_co.Bl_sgdict {
		if _, ok = tp.Supplygrpdict[strings.TrimSpace(sg.Supplygroupid)]; !ok {
			// SupplyGroup不存在
			verr = BL_VerifyError{}
			verr.Errortype = common.CO_VERIFY_SUPPLYGROUP_NOTEXIST
			verr.Supplygroupid = sg.Supplygroupid
			verr.Errorinfo = fmt.Sprintf("Supplygroup: %s was not existing!", sg.Supplygroupid)
			bl_co.Bl_verrdict[verr.GetDictKey()] = verr
		}
		for _, sgpt = range sg.Bl_sgpartDict {
			if _, ok = tp.Partdict[strings.TrimSpace(sgpt.Partid)]; !ok {
				// 零件不存在
				verr = BL_VerifyError{}
				verr.Errortype = common.CO_VERIFY_PART_NOTEXIST
				verr.Partid = sgpt.Partid
				verr.Supplygroupid = sg.Supplygroupid
				verr.Errorinfo = fmt.Sprintf("Part: %s in SupplyGroup: %s was not existing!", sgpt.Partid, sg.Supplygroupid)
				bl_co.Bl_verrdict[verr.GetDictKey()] = verr
			}
		}
	}

	return
}

// 验证客户订单供应组属性是否满足要求
func (bl_co *BL_CustOrder) ValidateSupplyGroupAtcodList(tp *TodProject) (err error) {
	var (
		bl_sg   BL_SupplyGroup
		sgatcod BL_SupplyGroup_Atcod
		coatcod BL_OrdAttribute
		verr    BL_VerifyError
		ok      bool
	)

	// 遍历客户订单的供应组
	for _, bl_sg = range bl_co.Bl_usgdict {
		// 如果是共用件SupplyGroup则跳过（因为它的零件已分配到非共用件Supply Group）
		if bl_sg.Sgtype == common.SG_TYPE_NEUTRAL {
			continue
		}
		// 验证供应组是否存在，如不存在，则报错
		if _, ok = tp.Supplygrpdict[strings.TrimSpace(bl_sg.Supplygroupid)]; !ok {
			verr = BL_VerifyError{}
			verr.Supplygroupid = bl_sg.Supplygroupid
			verr.Errortype = common.CO_VERIFY_SUPPLYGROUP_NOTEXIST
			verr.Errorinfo = fmt.Sprintf("Supplygroup: %s in Custorder: %s was not existing!", bl_sg.Supplygroupid, bl_co.Custordernr)
			bl_co.Bl_verrdict[verr.GetDictKey()] = verr
			return
		}
		// 遍历供应组内的验证属性并和客户订单在该供应组下解析所得的属性进行相互校验
		for _, sgatcod = range tp.Supplygrpdict[bl_sg.Supplygroupid].Sg_attr_dict {
			// 是否强制校验属性是否存在
			if sgatcod.Mandatory == 1 {
				if coatcod, ok = bl_sg.Bl_ordattrdict[sgatcod.Attrcode]; !ok {
					verr = BL_VerifyError{}
					verr.Errortype = common.CO_VERIFY_SG_ATCOD_NOT_SATISFY
					verr.Supplygroupid = bl_sg.Supplygroupid
					verr.Attrcode = sgatcod.Attrcode
					verr.Errorinfo = fmt.Sprintf("Custorder: %s doesn't have the required attribute: %d of supplygroup: %s!", bl_co.Custordernr, sgatcod.Attrcode, bl_sg.Supplygroupid)
					bl_co.Bl_verrdict[verr.GetDictKey()] = verr
					continue
				}
			}
			// 校验属性值
			switch sgatcod.Verifyrule {
			// 属性值唯一
			case common.VERIFY_RULE_ATVAL_UNIQ:
				// 如果有多个属性值
				if len(coatcod.Attrvaldict) > 1 {
					verr = BL_VerifyError{}
					verr.Errortype = common.CO_VERIFY_ATCOD_VALUE_CONFLICT
					verr.Supplygroupid = bl_sg.Supplygroupid
					verr.Attrcode = sgatcod.Attrcode
					verr.Errorinfo = fmt.Sprintf("Attribute: %d got multi values which conflicted with supplygroup: %s verfication rule!",
						sgatcod.Attrcode, sgatcod.Supplygroupid)
					bl_co.Bl_verrdict[verr.GetDictKey()] = verr
					break
				}
				// 其它规则
			}
		}
	}

	return
}

// 验证客户订单零件族的属性是否满足要求
func (bl_co *BL_CustOrder) ValidatePartFamilyAtcodList(tp *TodProject) (err error) {
	var (
		bl_pf   BL_PartFamily
		pfatcod BL_PartFamily_Atcod
		coatcod BL_OrdAttribute
		verr    BL_VerifyError
		ok      bool
	)

	// 验证客户订单的零件族
	if bl_pf, ok = tp.Partfamdict[strings.TrimSpace(bl_co.Custordertab.Partfamilyid)]; !ok {
		verr = BL_VerifyError{}
		verr.Errortype = common.CO_VERIFY_PARTFAMILY_NOTEXIST
		verr.Partfamilyid = bl_pf.Partfamilyid
		verr.Errorinfo = fmt.Sprintf("Partfamily: %s in Custorder: %s was not existing!", bl_co.Custordertab.Partfamilyid, bl_co.Custordernr)
		bl_co.Bl_verrdict[verr.GetDictKey()] = verr
		return
	}

	// 遍历零件族内的验证属性并和客户订单解析所得的属性进行相互校验
	for _, pfatcod = range bl_pf.Pf_attr_dict {
		// 是否强制校验属性是否存在
		if pfatcod.Mandatory == 1 {
			if coatcod, ok = bl_co.Bl_ordattrdict[pfatcod.Attrcode]; !ok {
				verr = BL_VerifyError{}
				verr.Errortype = common.CO_VERIFY_PF_ATCOD_NOT_SATISFY
				verr.Partfamilyid = bl_pf.Partfamilyid
				verr.Attrcode = pfatcod.Attrcode
				verr.Errorinfo = fmt.Sprintf("Custorder: %s doesn't have the required attribute: %d of partfamily: %s!", bl_co.Custordernr, pfatcod.Attrcode, bl_pf.Partfamilyid)
				bl_co.Bl_verrdict[verr.GetDictKey()] = verr
				continue
			}
		}
		// 校验属性值
		switch pfatcod.Verifyrule {
		// 属性值唯一
		case common.VERIFY_RULE_ATVAL_UNIQ:
			// 如果有多个属性值
			if len(coatcod.Attrvaldict) > 1 {
				verr = BL_VerifyError{}
				verr.Errortype = common.CO_VERIFY_ATCOD_VALUE_CONFLICT
				verr.Partfamilyid = bl_pf.Partfamilyid
				verr.Attrcode = pfatcod.Attrcode
				verr.Errorinfo = fmt.Sprintf("Attribute: %d got multi values which conflicted with partfamily: %s verfication rule!",
					pfatcod.Attrcode, bl_pf.Partfamilyid)
				bl_co.Bl_verrdict[verr.GetDictKey()] = verr
				break
			}
			// 其它规则
		}
	}

	return
}

// 保存客户订单版本，包括零件数据
func (bl_co *BL_CustOrder) SaveCOHeadVersion(session *xorm.Session) (err error) {
	var (
		cotab   db.Pln_custorder
		covtab  db.Pln_custorder_ver
		covptab db.Pln_custorder_ver_partlst
		bl_sg   BL_SupplyGroup
		bl_sgpt BL_SgPart
		ver     int
	)

	cotab = db.Pln_custorder{Custordernr: bl_co.Custordernr}
	// 获取版本号
	if ver, err = cotab.GetLatestVersionBySession(session); err != nil {
		return
	}

	// 新增客户订单版本
	covtab = db.Pln_custorder_ver{}
	covtab.Finr = db.G_FINR
	covtab.Custordernr = bl_co.Custordernr
	covtab.Oemordernr = bl_co.Oemordernr
	covtab.Partfamilyid = bl_co.Partfamilyid
	covtab.Version = ver + 1
	covtab.Parsed = 0
	covtab.Seqmode = bl_co.SeqType
	covtab.Ordertime = common.Date(bl_co.Ordertime.Unix(), "YYYYMMDDHHmmss")
	covtab.Swet = common.Date(bl_co.Swet.Unix(), "YYYYMMDDHHmmss")
	covtab.Edifiletype = "TOD"
	covtab.Edifile = bl_co.Edifile
	covtab.Lastuser = "service"
	covtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	if err = covtab.Insert(session); err != nil {
		return
	}

	// 遍历客户订单的SupplyGroup
	for _, bl_sg = range bl_co.Bl_sgdict {
		// 遍历零件列表并存储
		for _, bl_sgpt = range bl_sg.Bl_sgpartDict {
			//先删除
			tem := new(db.Pln_custorder_ver_partlst)
			tem.Finr = covtab.Finr
			tem.Custordernr = covtab.Custordernr
			tem.Version = covtab.Version
			tem.Supplygroupid = bl_sg.Supplygroupid
			tem.Partid = bl_sgpt.Partid
			tem.DelSession(session)

			covptab = db.Pln_custorder_ver_partlst{}
			covptab.Finr = covtab.Finr
			covptab.Custordernr = covtab.Custordernr
			covptab.Version = covtab.Version
			covptab.Supplygroupid = bl_sg.Supplygroupid
			covptab.Partid = bl_sgpt.Partid
			covptab.Partqty = bl_sgpt.Qty
			covptab.Lastuser = "service"
			covptab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
			if err = covptab.Insert(session); err != nil {
				return
			}
		}
	}

	return
}

// 基于客户订单的供应组在内存中生成生产订单
func (bl_co *BL_CustOrder) GenerateWorkorderBySupplyGroup(tp *TodProject) (err error) {
	var (
		bl_sg     BL_SupplyGroup
		bl_tpsg   BL_SupplyGroup
		bl_wopt   BL_WorkOrder_Part
		sgpt      BL_SgPart
		bl_wo     BL_WorkOrder
		i, factor int
		verr      BL_VerifyError
		ok        bool
	)

	// 初始化客户订单的生产订单字典
	bl_co.Bl_wodict = make(map[string]BL_WorkOrder)

	// 遍历客户订单的SupplyGroup，对于非共用件的创建生产订单
	for _, bl_sg = range bl_co.Bl_usgdict {
		if bl_tpsg, ok = tp.Supplygrpdict[strings.TrimSpace(bl_sg.Supplygroupid)]; !ok {
			// SupplyGroup不存在
			verr = BL_VerifyError{}
			verr.Errortype = common.CO_VERIFY_SUPPLYGROUP_NOTEXIST
			verr.Supplygroupid = bl_sg.Supplygroupid
			verr.Errorinfo = fmt.Sprintf("Supplygroup: %s was not existing!", bl_sg.Supplygroupid)
			bl_co.Bl_verrdict[verr.GetDictKey()] = verr
			continue
		}

		// 如果是共用件SupplyGroup则跳过
		if bl_tpsg.Sgtype == common.SG_TYPE_NEUTRAL {
			continue
		}
		// 跳过不生成工单的Supplygroup
		if !bl_tpsg.Multioutput {
			continue
		}
		// 如果零件数为0则取消该WO
		if len(bl_sg.Bl_sgpartDict) <= 0 {
			bl_wo = BL_WorkOrder{}
			bl_wo.Bl_wopartdict = make(map[string]BL_WorkOrder_Part)
			bl_wo.Bl_ordattrdict = make(map[int]BL_OrdAttribute)
			bl_wo.Bl_verrlist = []BL_VerifyError{}
			bl_wo.Workordernr = bl_sg.Supplygroupid // 内存中的客户订单的生产订单号先用SGID代替
			bl_wo.Custordernr = bl_co.Custordernr
			bl_wo.Partfamilyid = bl_sg.Partfamilyid
			bl_wo.Supplygroupid = bl_sg.Supplygroupid
			bl_wo.Worklineid = bl_sg.Worklineid
			bl_wo.Projnr = bl_co.Projnr
			bl_wo.Shippable = bl_tpsg.Shippablesg
			bl_wo.Status = common.WO_STATUS_CANCELED

			bl_co.Bl_wodict[bl_wo.Workordernr] = bl_wo
			continue
		}
		// 初始化
		// 如果是一对多
		if bl_tpsg.Multioutput && bl_tpsg.Modfactor > 0 {
			factor = bl_tpsg.Modfactor
			for i = 0; i < factor; i++ {
				bl_wo = BL_WorkOrder{}
				bl_wo.Bl_wopartdict = make(map[string]BL_WorkOrder_Part)
				bl_wo.Bl_ordattrdict = make(map[int]BL_OrdAttribute)
				bl_wo.Bl_verrlist = []BL_VerifyError{}
				bl_wo.Workordernr = bl_sg.Supplygroupid + strconv.Itoa(i+1) // 内存中的客户订单的生产订单号先用SGID代替
				bl_wo.Custordernr = bl_co.Custordernr
				bl_wo.Partfamilyid = bl_sg.Partfamilyid
				bl_wo.Supplygroupid = bl_sg.Supplygroupid
				bl_wo.Worklineid = bl_sg.Worklineid
				bl_wo.Projnr = bl_co.Projnr
				bl_wo.Shippable = bl_tpsg.Shippablesg
				bl_wo.Status = common.WO_STATUS_UNPLANED
				// 将供应组的零件清单复制到生产订单
				for _, sgpt = range bl_sg.Bl_sgpartDict {
					if sgpt.Qty%factor != 0 {
						continue
					}
					bl_wopt = BL_WorkOrder_Part{}
					bl_wopt.Workordernr = bl_wo.Workordernr
					bl_wopt.Partid = sgpt.Partid
					bl_wopt.Qty = sgpt.Qty / factor
					bl_wo.Bl_wopartdict[sgpt.Partid] = bl_wopt
				}
				bl_co.Bl_wodict[bl_wo.Workordernr] = bl_wo
			}
		} else {
			bl_wo = BL_WorkOrder{}
			bl_wo.Bl_wopartdict = make(map[string]BL_WorkOrder_Part)
			bl_wo.Bl_ordattrdict = make(map[int]BL_OrdAttribute)
			bl_wo.Bl_verrlist = []BL_VerifyError{}
			bl_wo.Workordernr = bl_sg.Supplygroupid // 内存中的客户订单的生产订单号先用SGID代替
			bl_wo.Custordernr = bl_co.Custordernr
			bl_wo.Partfamilyid = bl_sg.Partfamilyid
			bl_wo.Supplygroupid = bl_sg.Supplygroupid
			bl_wo.Worklineid = bl_sg.Worklineid
			bl_wo.Projnr = bl_co.Projnr
			bl_wo.Shippable = bl_tpsg.Shippablesg
			bl_wo.Status = common.WO_STATUS_UNPLANED

			// 将供应组的零件清单复制到生产订单
			for _, sgpt = range bl_sg.Bl_sgpartDict {
				bl_wopt = BL_WorkOrder_Part{}
				bl_wopt.Workordernr = bl_wo.Workordernr
				bl_wopt.Partid = sgpt.Partid
				bl_wopt.Qty = sgpt.Qty
				bl_wo.Bl_wopartdict[sgpt.Partid] = bl_wopt
			}
			bl_co.Bl_wodict[bl_wo.Workordernr] = bl_wo
		}
	}
	return
}

// 基于生产订单零件生成生产订单属性
func (bl_co *BL_CustOrder) GenerateWorkorderAtcodlst(tp *TodProject) (err error) {
	var (
		bl_wo       BL_WorkOrder
		bl_wopt     BL_WorkOrder_Part
		bl_pt       BL_Part
		bl_partattr BL_PartAttribute
		bl_ordattr  BL_OrdAttribute
		verr        BL_VerifyError
		ok          bool
	)
	// 遍历客户订单
	for _, bl_wo = range bl_co.Bl_wodict {
		bl_wo.Bl_ordattrdict = make(map[int]BL_OrdAttribute)
		// 遍历它的零件清单
		for _, bl_wopt = range bl_wo.Bl_wopartdict {
			if bl_pt, ok = tp.Partdict[bl_wopt.Partid]; !ok {
				// 零件不存在
				verr = BL_VerifyError{}
				verr.Errortype = common.CO_VERIFY_PART_NOTEXIST
				verr.Partid = bl_wopt.Partid
				verr.Supplygroupid = bl_wo.Supplygroupid
				verr.Errorinfo = fmt.Sprintf("Part: %s in custorder: %s was not existing!", bl_wopt.Partid, bl_co.Custordernr)
				bl_co.Bl_verrdict[verr.GetDictKey()] = verr
				continue
			}

			// 遍历零件属性并添加到生产订单的属性列表中
			for _, bl_partattr = range bl_pt.AtcodDict {
				if bl_ordattr, ok = bl_wo.Bl_ordattrdict[bl_partattr.Attrcode]; !ok {
					bl_ordattr = BL_OrdAttribute{}
					bl_ordattr.Attrcode = bl_partattr.Attrcode
					bl_ordattr.Attrvaldict = make(map[string]string)
				}
				bl_ordattr.Attrvaldict[bl_partattr.Attrvalue] = bl_partattr.Attrvalue
				bl_wo.Bl_ordattrdict[bl_partattr.Attrcode] = bl_ordattr
				continue
			}
		}
		// 将生产订单刷新到客户订单中
		bl_co.Bl_wodict[bl_wo.Workordernr] = bl_wo
	}

	return
}

// 保存客户订单的解析错误
func (bl_co *BL_CustOrder) SaveParseErrors(session *xorm.Session) (err error) {
	var (
		cotab    db.Pln_custorder
		coerrtab db.Pln_custorder_errorlst
		bl_verr  BL_VerifyError
		idx      int
	)

	// 获取索引号
	cotab = db.Pln_custorder{Custordernr: bl_co.Custordernr}
	if idx, err = cotab.GetLatestErrorIndexBySession(session); err != nil {
		return
	}

	// 遍历客户订单的错误列表
	for _, bl_verr = range bl_co.Bl_verrdict {
		idx = idx + 1
		coerrtab = db.Pln_custorder_errorlst{}
		coerrtab.Finr = db.G_FINR
		coerrtab.Custordernr = bl_co.Custordernr
		coerrtab.Pos = idx
		coerrtab.Errortype = bl_verr.Errortype
		coerrtab.Errorinfo = bl_verr.Errorinfo
		coerrtab.Partid = bl_verr.Partid
		coerrtab.Supplygroupid = bl_verr.Supplygroupid
		coerrtab.Partfamilyid = bl_verr.Partfamilyid
		coerrtab.Attrcode = bl_verr.Attrcode
		coerrtab.Errorstatus = "O"
		coerrtab.Lastuser = "service"
		coerrtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
		if err = coerrtab.Insert(session); err != nil {
			return err
		}
	}
	return
}

// 保存客户订单的零件和属性清单（先删除后新增）
func (bl_co *BL_CustOrder) SavePartAndAtcodData(session *xorm.Session, tp *TodProject) (err error) {
	var (
		cotab      db.Pln_custorder
		bl_sg      BL_SupplyGroup
		bl_sgpt    BL_SgPart
		cosgptlst  db.Pln_custorder_supplygroup_partlst
		bl_ordattr BL_OrdAttribute
		coptlst    db.Pln_custorder_partlst
		coatlst    db.Pln_custorder_atcodlst
		attrval    string
		idx        int
	)

	// 遍历客户订单供应组
	for _, bl_sg = range bl_co.Bl_sgdict {
		// 删除客户订单的零件清单和属性清单数据
		cotab = db.Pln_custorder{Custordernr: bl_co.Custordernr}
		if !cotab.DelSGPartlst(session, bl_sg.Supplygroupid) {
			err = errors.New(fmt.Sprintf("Failed to delete cust order: %s sgpart list!", bl_co.Custordernr))
			return
		}
		for _, bl_sgpt = range bl_sg.Bl_sgpartDict {
			cosgptlst = db.Pln_custorder_supplygroup_partlst{}
			cosgptlst.Finr = db.G_FINR
			cosgptlst.Custordernr = bl_co.Custordernr
			cosgptlst.Supplygroupid = bl_sgpt.Supplygroupid
			cosgptlst.Partfamilyid = bl_sg.Partfamilyid
			cosgptlst.Partid = bl_sgpt.Partid
			cosgptlst.Partqty = bl_sgpt.Qty
			cosgptlst.Lastuser = "service"
			cosgptlst.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
			if err = cosgptlst.Insert(session); err != nil {
				return err
			}
		}
	}

	// 删除客户订单的零件清单和属性清单数据
	if !cotab.DelPartlst(session) {
		err = errors.New(fmt.Sprintf("Failed to delete cust order: %s part list!", bl_co.Custordernr))
		return
	}
	if !cotab.DelAttributelst(session) {
		err = errors.New(fmt.Sprintf("Failed to delete cust order: %s attribute list!", bl_co.Custordernr))
		return
	}

	// 遍历客户订单已分配的零件列表
	for _, bl_sg = range bl_co.Bl_sgdict {
		for _, bl_sgpt = range bl_sg.Bl_sgpartDict {
			coptlst = db.Pln_custorder_partlst{}
			coptlst.Finr = db.G_FINR
			coptlst.Custordernr = bl_co.Custordernr
			coptlst.Supplygroupid = bl_sgpt.Supplygroupid
			coptlst.Originalsgid = bl_sgpt.Originalsgid
			coptlst.Partfamilyid = bl_sg.Partfamilyid
			coptlst.Partid = bl_sgpt.Partid
			coptlst.Partqty = bl_sgpt.Qty
			coptlst.Lastuser = "service"
			coptlst.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
			if err = coptlst.Insert(session); err != nil {
				return err
			}
		}
	}

	// 遍历属性列表并存储
	idx = 0
	for _, bl_ordattr = range bl_co.Bl_ordattrdict {
		coatlst = db.Pln_custorder_atcodlst{}
		coatlst.Finr = db.G_FINR
		coatlst.Custordernr = bl_co.Custordernr
		coatlst.Attrcode = bl_ordattr.Attrcode
		for _, attrval = range bl_ordattr.Attrvaldict {
			idx++
			coatlst.Pos = idx
			coatlst.Attrvalue = attrval
			coatlst.Lastuser = "service"
			coatlst.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
			if err = coatlst.Insert(session); err != nil {
				return err
			}
		}
	}

	return
}

// 取消客户订单（将客户订单和它的生产订单状态置位98取消）
func (bl_co *BL_CustOrder) Cancel(session *xorm.Session) (err error) {
	var (
		cotab    db.Pln_custorder
		wotablst []db.Pln_workorder
		i        int
	)

	cotab = db.Pln_custorder{Custordernr: bl_co.Custordernr}
	if cotab, err = cotab.SelectOne(); err != nil {
		return
	}
	if wotablst, err = cotab.GetWorkorderlst(); err != nil {
		return
	}

	cotab.Status = common.CO_STATUS_CANCELED
	cotab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	if err = cotab.UpdateFields(session, "status, lastmodif"); err != nil {
		return
	}

	for i, _ = range wotablst {
		wotablst[i].Status = common.WO_STATUS_CANCELED
		wotablst[i].Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
		if err = wotablst[i].UpdateFields(session, "status, lastmodif"); err != nil {
			return
		}
	}

	return
}

// 保存客户订单的生产订单
func (bl_co *BL_CustOrder) SaveWorkorderData(session *xorm.Session, tp *TodProject) (err error) {
	var (
		bl_wo     BL_WorkOrder
		bl_sg     BL_SupplyGroup
		wotab     db.Pln_workorder
		wotablst  []db.Pln_workorder
		exist, ok bool
		i         int
	)

	for _, bl_wo = range bl_co.Bl_wodict {
		if bl_sg, ok = tp.Supplygrpdict[bl_wo.Supplygroupid]; !ok {
			err = errors.New(fmt.Sprintf("Workorder: %s supplygroup: %s is not existing in global dict!", bl_wo.Workordernr, bl_wo.Supplygroupid))
			return
		}

		wotab = db.Pln_workorder{}
		if exist, wotablst, err = wotab.GetByCOAndSGBySession(session, bl_wo.Custordernr, bl_wo.Supplygroupid); err != nil {
			return
		}

		// 不存在且不是取消则新建
		if !exist {
			if bl_wo.Status != common.WO_STATUS_CANCELED {
				if err = bl_wo.CreateHead(session, tp); err != nil {
					return
				}
			}
		} else {
			// 存在且取消则取消工单
			if bl_wo.Status == common.WO_STATUS_CANCELED {
				// 取消这个生产订单
				bl_wo.Workordertab = wotablst[0]
				bl_wo.Workordertab.Status = common.WO_STATUS_CANCELED
				bl_wo.Workordertab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
				if err = bl_wo.Workordertab.Update("status, lastmodif"); err != nil {
					return
				}
			}
			if bl_sg.Multioutput && bl_sg.Modfactor > 0 {
				if len(wotablst) < bl_sg.Modfactor {
					if err = bl_wo.CreateHead(session, tp); err != nil {
						return
					}
				}
			} else {
				bl_wo.Workordertab = wotablst[0]
				bl_wo.Workordernr = wotablst[0].Workordernr
			}
		}

		// 删除并替换生产订单原来的零件和属性清单数据
		if bl_sg.Multioutput && bl_sg.Modfactor > 0 {
			for i = 0; i < len(wotablst); i++ {
				bl_wo.Workordernr = wotablst[i].Workordernr
				bl_wo.Workordertab = wotablst[i]
				if err = bl_wo.SavePartAndAtcodList(session); err != nil {
					return
				}
			}
		} else {
			if err = bl_wo.SavePartAndAtcodList(session); err != nil {
				return
			}
		}
	}

	return
}

// 用COV中的零件数据替换客户订单已有SG的零件
func (bl_co *BL_CustOrder) ReplaceSGPartListByCOVData(covtab db.Pln_custorder_ver, tp *TodProject) (err error) {
	var (
		covptab   db.Pln_custorder_ver_partlst
		sgdict    map[string]string
		bl_sg     BL_SupplyGroup
		bl_usg    BL_SupplyGroup
		bl_sgcopy BL_SupplyGroup
		bl_csg    BL_SupplyGroup
		bl_sgpt   BL_SgPart
		sgname    string
		i         int
		ok        bool
	)

	// 1. 获取COV中的供应组名
	sgdict = make(map[string]string)
	for i = 0; i < len(covtab.Partlst); i++ {
		covtab.Partlst[i].Clipped()
		sgdict[covtab.Partlst[i].Supplygroupid] = covtab.Partlst[i].Supplygroupid
	}
	// 2. 删除客户订单对应供应组的零件数据
	for sgname, _ = range sgdict {
		if bl_sg, ok = tp.Supplygrpdict[sgname]; !ok {
			err = errors.New(fmt.Sprintf("covtab's supply group %s is not existing in global dict!", sgname))
			return
		}
		bl_sgcopy = BL_SupplyGroup{}
		if bl_sgcopy, err = bl_sg.Copy(); err != nil {
			return
		}
		// 初始化供应组的零件清单
		bl_sgcopy.Bl_sgpartDict = make(map[string]BL_SgPart)
		// 将供应组更新回客户订单
		bl_co.Bl_sgdict[bl_sg.Supplygroupid] = bl_sgcopy
		//fmt.Println("Add suggply group:",bl_sg.Supplygroupid," to custorder :",bl_co.Custordernr)
	}
	// 3. 将COV中的零件添加到客户订单
	for i = 0; i < len(covtab.Partlst); i++ {
		if covtab.Partlst[i].Partqty <= 0 {
			continue
		}
		covptab = covtab.Partlst[i]
		covptab.Clipped()
		bl_sgpt = BL_SgPart{}
		bl_sgpt.Supplygroupid = covptab.Supplygroupid
		bl_sgpt.Originalsgid = covptab.Supplygroupid
		bl_sgpt.Partid = covptab.Partid
		bl_sgpt.Qty = covptab.Partqty
		if bl_sg, ok = bl_co.Bl_sgdict[covptab.Supplygroupid]; !ok {
			err = errors.New(fmt.Sprintf("covtab's supply group %s is not existing in global dict!", sgname))
			return
		}

		bl_sg.Bl_sgpartDict[bl_sgpt.Partid] = bl_sgpt
		bl_co.Bl_sgdict[bl_sg.Supplygroupid] = bl_sg
		//fmt.Println("        Add partid:",bl_sgpt.Partid," of supplygroup:",bl_sgpt.Supplygroupid)
	}

	// 将覆盖后的客户订单的SG的零件清单复制到 bl_co.Bl_usgdict 对象
	bl_co.Bl_usgdict = make(map[string]BL_SupplyGroup)
	for _, bl_sg = range bl_co.Bl_sgdict {
		bl_sgcopy = BL_SupplyGroup{}
		if bl_sgcopy, err = bl_sg.Copy(); err != nil {
			return
		}
		bl_co.Bl_usgdict[bl_sgcopy.Supplygroupid] = bl_sgcopy
	}

	// 判断供应组是否有子组，如果有则添加到客户订单
	for _, bl_usg = range bl_co.Bl_usgdict {
		if bl_sg, ok = tp.Supplygrpdict[bl_usg.Supplygroupid]; !ok {
			err = errors.New(fmt.Sprintf("covtab's supply group %s is not existing in global dict!", sgname))
			return
		}
		if bl_sg.Sgtype == common.SG_TYPE_NORMAL {
			for _, sgname = range bl_sg.Sgchildrendict {
				bl_csg = BL_SupplyGroup{}
				if bl_csg, err = tp.CopySupplyGroup(sgname); err != nil {
					return
				}
				bl_csg.Bl_sgpartDict = make(map[string]BL_SgPart)
				// 将子供应组更新回客户订单
				bl_co.Bl_usgdict[bl_csg.Supplygroupid] = bl_csg
			}
		}
	}

	return
}

// GetFullData: 获取客户订单完整数据(零件和属性清单）
func (bl_co *BL_CustOrder) GetFullData() (err error) {
	var (
		cotab         db.Pln_custorder
		bl_wo         BL_WorkOrder    // 生产订单业务对象
		bl_ordattr    BL_OrdAttribute // 客户订单属性清单字典
		bl_sgpt       BL_SgPart       // 客户订单零件清单字典
		bl_wopt       BL_WorkOrder_Part
		i, j, k, l, m int
		ok            bool
	)

	cotab = db.Pln_custorder{Custordernr: bl_co.Custordernr}
	if cotab, err = cotab.GetFullData(); err != nil {
		return
	}

	// 遍历客户订单的零件数据
	bl_co.Bl_ordpartdict = make(map[string]BL_SgPart)
	for i = 0; i < len(cotab.Partlst); i++ {
		bl_sgpt = BL_SgPart{}
		bl_sgpt.Supplygroupid = cotab.Partlst[i].Supplygroupid
		bl_sgpt.Partid = cotab.Partlst[i].Partid
		bl_sgpt.Qty = cotab.Partlst[i].Partqty
		bl_co.Bl_ordpartdict[bl_sgpt.Partid] = bl_sgpt
	}

	// 遍历客户订单的属性数据
	bl_co.Bl_ordattrdict = make(map[int]BL_OrdAttribute)
	for j = 0; j < len(cotab.Attrlst); j++ {
		if bl_ordattr, ok = bl_co.Bl_ordattrdict[cotab.Attrlst[j].Attrcode]; !ok {
			// 不存在
			bl_ordattr = BL_OrdAttribute{}
			bl_ordattr.Attrcode = cotab.Attrlst[j].Attrcode
			bl_ordattr.Attrvaldict = make(map[string]string)
		}
		bl_ordattr.Attrvaldict[cotab.Attrlst[j].Attrvalue] = cotab.Attrlst[j].Attrvalue
	}

	// 遍历客户订单的生产订单数据
	bl_co.Bl_wodict = make(map[string]BL_WorkOrder)
	for k = 0; k < len(cotab.Wotablst); k++ {
		bl_wo = BL_WorkOrder{}
		bl_wo.Workordernr = cotab.Wotablst[k].Workordernr
		bl_wo.Supplygroupid = cotab.Wotablst[k].Supplygroupid
		bl_wo.Partfamilyid = cotab.Wotablst[k].Partfamilyid
		bl_wo.Custordernr = cotab.Wotablst[k].Custordernr
		bl_wo.Oemseq = common.ValueToInt(cotab.Wotablst[k].Oemseq, 0)
		bl_wo.Status = cotab.Wotablst[k].Status
		bl_wo.Schedseq = cotab.Wotablst[k].Schedseq
		bl_wo.Workordertab = cotab.Wotablst[k]
		bl_wo.Bl_wopartdict = make(map[string]BL_WorkOrder_Part)
		bl_wo.Bl_ordattrdict = make(map[int]BL_OrdAttribute)

		// 遍历生产订单零件数据
		for l = 0; l < len(cotab.Wotablst[k].Partlst); l++ {
			bl_wopt = BL_WorkOrder_Part{}
			bl_wopt.Partid = cotab.Wotablst[k].Partlst[l].Partid
			bl_wopt.Qty = cotab.Wotablst[k].Partlst[l].Partqty
			bl_wo.Bl_wopartdict[bl_wopt.Partid] = bl_wopt
		}

		// 遍历生产订单属性数据
		for m = 0; m < len(cotab.Wotablst[k].Attrlst); m++ {
			if bl_ordattr, ok = bl_wo.Bl_ordattrdict[cotab.Wotablst[k].Attrlst[m].Attrcode]; !ok {
				bl_ordattr = BL_OrdAttribute{}
				bl_ordattr.Attrcode = cotab.Wotablst[k].Attrlst[m].Attrcode
				bl_ordattr.Attrvaldict = make(map[string]string)
			}
			bl_ordattr.Attrvaldict[cotab.Wotablst[k].Attrlst[m].Attrvalue] = cotab.Wotablst[k].Attrlst[m].Attrvalue
			bl_wo.Bl_ordattrdict[bl_ordattr.Attrcode] = bl_ordattr
		}

		bl_co.Bl_wodict[bl_wo.Workordernr] = bl_wo
	}

	return
}

// GetFullData: 获取客户订单的生产订单头数据
func (bl_co *BL_CustOrder) GetWorkOrderList() (err error) {
	var (
		k        int
		wotablst []db.Pln_workorder
		bl_wo    BL_WorkOrder // 生产订单业务对象
	)

	// 获取客户订单的生产订单数据
	if err = db.G_DbEngine.Where("finr = ? and custordernr = ?", db.G_FINR,
		bl_co.Custordernr).Asc("workordernr").Find(&wotablst); err != nil {
		return
	}
	bl_co.Bl_wodict = make(map[string]BL_WorkOrder)
	for k = 0; k < len(wotablst); k++ {
		wotablst[k].Clipped()
		bl_wo = BL_WorkOrder{}
		bl_wo.Workordernr = wotablst[k].Workordernr
		bl_wo.Supplygroupid = wotablst[k].Supplygroupid
		bl_wo.Partfamilyid = wotablst[k].Partfamilyid
		bl_wo.Custordernr = wotablst[k].Custordernr
		bl_wo.Oemseq = common.ValueToInt(wotablst[k].Oemseq, 0)
		bl_wo.Status = wotablst[k].Status
		bl_wo.Schedseq = wotablst[k].Schedseq
		bl_wo.Workordertab = wotablst[k]
		bl_wo.Bl_wopartdict = make(map[string]BL_WorkOrder_Part)
		bl_wo.Bl_ordattrdict = make(map[int]BL_OrdAttribute)
		bl_co.Bl_wodict[bl_wo.Workordernr] = bl_wo
	}

	return
}

// GetFullData: 获取客户订单的生产订单头数据
func (bl_co *BL_CustOrder) GetWorkOrderListBySession(session *xorm.Session) (err error) {
	var (
		k        int
		wotablst []db.Pln_workorder
		bl_wo    BL_WorkOrder // 生产订单业务对象
	)

	// 获取客户订单的生产订单数据
	if err = session.Where("finr = ? and custordernr = ?", db.G_FINR,
		bl_co.Custordernr).Asc("workordernr").Find(&wotablst); err != nil {
		return
	}
	bl_co.Bl_wodict = make(map[string]BL_WorkOrder)
	for k = 0; k < len(wotablst); k++ {
		wotablst[k].Clipped()
		bl_wo = BL_WorkOrder{}
		bl_wo.Workordernr = wotablst[k].Workordernr
		bl_wo.Supplygroupid = wotablst[k].Supplygroupid
		bl_wo.Partfamilyid = wotablst[k].Partfamilyid
		bl_wo.Custordernr = wotablst[k].Custordernr
		bl_wo.Oemseq = common.ValueToInt(wotablst[k].Oemseq, 0)
		bl_wo.Status = wotablst[k].Status
		bl_wo.Schedseq = wotablst[k].Schedseq
		bl_wo.Workordertab = wotablst[k]
		bl_wo.Bl_wopartdict = make(map[string]BL_WorkOrder_Part)
		bl_wo.Bl_ordattrdict = make(map[int]BL_OrdAttribute)
		bl_co.Bl_wodict[bl_wo.Workordernr] = bl_wo
	}

	return
}

// GetFullData: 获取客户订单的生产订单头数据
func (bl_co *BL_CustOrder) IsAllPackItemPackedBySession(session *xorm.Session) (packed bool, err error) {
	var bl_wo BL_WorkOrder

	// 获取客户订单的所有生产订单
	if err = bl_co.GetWorkOrderListBySession(session); err != nil {
		return
	}
	// 遍历需要包装发运的生产订单，确认是否已包装
	packed = true
	for _, bl_wo = range bl_co.Bl_wodict {
		if bl_wo.Workordertab.Shippable == 1 && bl_wo.Workordertab.Packstatus == common.WO_STATUS_UNPACKED {
			packed = false
			return
		}
	}

	return
}

// UpdateSEQData: 基于解析的SEQ数据更新客户订单和它的生产订单
func (bl_co *BL_CustOrder) UpdateSEQData(seqlandtab db.Pln_seqdata_landing) (err error) {
	var (
		bl_wo BL_WorkOrder
	)

	// 更新客户订单排序和状态信息
	bl_co.Custordertab.Oemseq = seqlandtab.Oemseq
	if bl_co.Custordertab.Oemseq >= 9999999999 {
		bl_co.Custordertab.Orderinfo = "TCAR"
	} else {
		bl_co.Custordertab.Orderinfo = ""
	}
	bl_co.Custordertab.Unloadingpoint = seqlandtab.Unloadingpoint
	bl_co.Custordertab.Vin = seqlandtab.Vin
	if bl_co.Custordertab.Status < common.CO_STATUS_PLANNED {
		bl_co.Custordertab.Status = common.CO_STATUS_PLANNED
	}
	bl_co.Custordertab.Seqswet = seqlandtab.Swet
	bl_co.Custordertab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	if err = bl_co.Custordertab.UpdateToFieldsWithoutSession("oemseq, unloadingpoint, vin, status, seqswet, orderinfo, lastmodif"); err != nil {
		glog.InfoExtln("seq更新", "seqlandtab.Oemseq", seqlandtab.Oemseq)
		glog.InfoExtln("seq更新", "seqlandtab.Unloadingpoint", seqlandtab.Unloadingpoint)
		glog.InfoExtln("seq更新", "seqlandtab.Vin", seqlandtab.Vin)
		glog.InfoExtln("seq更新", "seqlandtab.Swet", seqlandtab.Swet)
		return
	}

	// 更新客户订单的生产订单信息
	for _, bl_wo = range bl_co.Bl_wodict {
		bl_wo.Workordertab.Mrpstatus = 0
		bl_wo.Workordertab.Oemseq = seqlandtab.Oemseq
		bl_wo.Workordertab.Preschedseq = bl_wo.Workordertab.Oemseq * 100
		if bl_wo.Workordertab.Status < common.WO_STATUS_PLANNED {
			bl_wo.Workordertab.Schedseq = 0
		}
		bl_wo.Workordertab.Swet = seqlandtab.Swet
		if bl_wo.Workordertab.Status < common.WO_STATUS_PLANNED {
			bl_wo.Workordertab.Status = common.WO_STATUS_PLANNED
		}
		bl_wo.Workordertab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
		if err = bl_wo.Workordertab.Update("oemseq, preschedseq, status, swet, lastmodif"); err != nil {
			glog.InfoExtln("seq更新", "bl_wo.Workordertab..Oemseq", bl_wo.Workordertab.Oemseq)
			glog.InfoExtln("seq更新", "bl_wo.Workordertab.Preschedseq", bl_wo.Workordertab.Preschedseq)
			glog.InfoExtln("seq更新", "bl_wo.Workordertab.Swet", seqlandtab.Swet)
			return
		}
	}

	return
}

// UpdateSEQData: 基于解析的SEQ数据更新客户订单和它的生产订单
func (bl_co *BL_CustOrder) UpdateSEQDataSession(session *xorm.Session, seqlandtab db.Pln_seqdata_landing) (err error) {
	var (
		bl_wo BL_WorkOrder
	)

	// 更新客户订单排序和状态信息
	bl_co.Custordertab.Oemseq = seqlandtab.Oemseq
	if bl_co.Custordertab.Oemseq >= 9999999999 {
		bl_co.Custordertab.Orderinfo = "TCAR"
	} else {
		bl_co.Custordertab.Orderinfo = ""
	}
	bl_co.Custordertab.Unloadingpoint = seqlandtab.Unloadingpoint
	bl_co.Custordertab.Vin = seqlandtab.Vin
	if bl_co.Custordertab.Status < common.CO_STATUS_PLANNED {
		bl_co.Custordertab.Status = common.CO_STATUS_PLANNED
	}
	bl_co.Custordertab.Seqswet = seqlandtab.Swet
	bl_co.Custordertab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	if err = bl_co.Custordertab.UpdateFields(session, "oemseq, unloadingpoint, vin, status, seqswet, orderinfo, lastmodif"); err != nil {
		glog.InfoExtln("seq更新", "seqlandtab.Oemseq", seqlandtab.Oemseq)
		glog.InfoExtln("seq更新", "seqlandtab.Unloadingpoint", seqlandtab.Unloadingpoint)
		glog.InfoExtln("seq更新", "seqlandtab.Vin", seqlandtab.Vin)
		glog.InfoExtln("seq更新", "seqlandtab.Swet", seqlandtab.Swet)
		return
	}

	// 更新客户订单的生产订单信息
	for _, bl_wo = range bl_co.Bl_wodict {
		bl_wo.Workordertab.Mrpstatus = 0
		bl_wo.Workordertab.Oemseq = seqlandtab.Oemseq
		bl_wo.Workordertab.Preschedseq = bl_wo.Workordertab.Oemseq * 100
		if bl_wo.Workordertab.Status <= common.WO_STATUS_PLANNED {
			bl_wo.Workordertab.Schedseq = 0
		}
		bl_wo.Workordertab.Swet = seqlandtab.Swet
		if bl_wo.Workordertab.Status < common.WO_STATUS_PLANNED {
			bl_wo.Workordertab.Status = common.WO_STATUS_PLANNED
		}
		bl_wo.Workordertab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
		if err = bl_wo.Workordertab.Update("oemseq, preschedseq, status, swet, lastmodif"); err != nil {
			glog.InfoExtln("seq更新", "bl_wo.Workordertab..Oemseq", bl_wo.Workordertab.Oemseq)
			glog.InfoExtln("seq更新", "bl_wo.Workordertab.Preschedseq", bl_wo.Workordertab.Preschedseq)
			glog.InfoExtln("seq更新", "bl_wo.Workordertab.Swet", seqlandtab.Swet)
			return
		}
	}

	return
}

// 检查当前客户订单的OEMSEQ是否连续
func (bl_co *BL_CustOrder) IsOemSeqContinued(seqlandtab db.Pln_seqdata_landing) (ct bool, err error) {
	var (
		maxseq int
		conr   string
	)

	ct = false
	if maxseq, conr, err = bl_co.Custordertab.GetMaxOemSeqByPartFamily(); err != nil {
		return
	}

	if maxseq <= 0 {
		ct = true
		return
	}
	// 检查当前客户订单序号是否和上一个序号连续
	if strings.TrimSpace(bl_co.Custordernr) != strings.TrimSpace(conr) {
		if maxseq+1 != common.ValueToInt(seqlandtab.Oemseq, 0) {
			return
		}
	} else {
		if maxseq != common.ValueToInt(seqlandtab.Oemseq, 0) {
			return
		}
	}

	ct = true
	return
}

// SaveSeqCOV: 保存客户订单的SEQ版本数据
func (bl_co *BL_CustOrder) SaveSeqCOV(seqlandtab db.Pln_seqdata_landing) (err error) {
	var (
		cotab  db.Pln_custorder
		covtab db.Pln_custorder_ver
		ver    int
	)

	cotab = db.Pln_custorder{Custordernr: bl_co.Custordernr}
	// 获取版本号
	if ver, err = cotab.GetLatestVersion(); err != nil {
		return
	}

	// 新增客户订单版本
	covtab = db.Pln_custorder_ver{}
	covtab.Finr = db.G_FINR
	covtab.Custordernr = bl_co.Custordernr
	covtab.Oemordernr = bl_co.Oemordernr
	covtab.Partfamilyid = bl_co.Partfamilyid
	covtab.Version = ver + 1
	covtab.Parsed = common.EDI_PARSE_YES
	covtab.Seqmode = seqlandtab.Seqmode
	covtab.Edifiletype = common.EDI_SEQ
	covtab.Edifile = seqlandtab.Edifile
	covtab.Lastuser = "service"
	covtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	if err = covtab.Add(); err != nil {
		return
	}

	return
}

// SaveSeqCOV: 保存客户订单的SEQ版本数据
func (bl_co *BL_CustOrder) SaveSeqCOVSession(session *xorm.Session, seqlandtab db.Pln_seqdata_landing) (err error) {
	var (
		cotab  db.Pln_custorder
		covtab db.Pln_custorder_ver
		ver    int
	)

	cotab = db.Pln_custorder{Custordernr: bl_co.Custordernr}
	// 获取版本号
	if ver, err = cotab.GetLatestVersionBySession(session); err != nil {
		return
	}

	// 新增客户订单版本
	covtab = db.Pln_custorder_ver{}
	covtab.Finr = db.G_FINR
	covtab.Custordernr = bl_co.Custordernr
	covtab.Oemordernr = bl_co.Oemordernr
	covtab.Partfamilyid = bl_co.Partfamilyid
	covtab.Version = ver + 1
	covtab.Parsed = common.EDI_PARSE_YES
	covtab.Seqmode = seqlandtab.Seqmode
	covtab.Edifiletype = common.EDI_SEQ
	covtab.Edifile = seqlandtab.Edifile
	covtab.Lastuser = "service"
	covtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	if err = covtab.Insert(session); err != nil {
		return
	}

	return
}

// SaveCalloffCOV: 保存客户订单的SEQ版本数据
func (bl_co *BL_CustOrder) SaveCalloffCOV(session *xorm.Session, callofflandtab db.Pln_calloffdata_landing) (err error) {
	var (
		cotab  db.Pln_custorder
		covtab db.Pln_custorder_ver
		ver    int
	)

	cotab = db.Pln_custorder{Custordernr: bl_co.Custordernr}
	// 获取版本号
	if ver, err = cotab.GetLatestVersionBySession(session); err != nil {
		return
	}

	// 新增客户订单版本
	covtab = db.Pln_custorder_ver{}
	covtab.Finr = db.G_FINR
	covtab.Custordernr = bl_co.Custordernr
	covtab.Oemordernr = bl_co.Oemordernr
	covtab.Partfamilyid = bl_co.Partfamilyid
	covtab.Version = ver + 1
	covtab.Parsed = common.EDI_PARSE_YES
	covtab.Seqmode = callofflandtab.Seqmode
	covtab.Edifiletype = common.EDI_CALLOFF
	covtab.Edifile = callofflandtab.Edifile
	covtab.Lastuser = "service"
	covtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	if err = covtab.Insert(session); err != nil {
		return
	}

	return
}

// GetPartFamilyList: 基于客户订单的供应组获取当前客户订单关联的零件族字典
func (bl_co *BL_CustOrder) GetPartFamilyDict(bl_proj *BL_Project) (pfDict map[string]string, err error) {
	var (
		key   string
		pfKey string
		ok    bool
	)

	pfDict = make(map[string]string)
	for key, _ = range bl_co.Bl_sgdict {
		if _, ok = bl_proj.Supplygrpdict[key]; !ok {
			continue
		}
		pfKey = bl_proj.Supplygrpdict[key].Partfamilyid
		pfDict[pfKey] = pfKey
	}

	return
}

// 基于订单的项目号、OEM订单号和零件族查询客户订单及其工作订单列表
func (bl_co *BL_CustOrder) GetCalloffOemorder() (exist bool, err error) {
	var cotab db.Pln_custorder

	cotab = db.Pln_custorder{Projnr: bl_co.Projnr, Oemordernr: bl_co.Oemordernr, Partfamilyid: bl_co.Partfamilyid}
	if exist, cotab, err = cotab.GetByOemOrderNr(); err != nil {
		return
	}
	bl_co.Custordernr = cotab.Custordernr
	bl_co.Oemordernr = cotab.Oemordernr
	bl_co.Custordertab = cotab

	// 获取客户订单的生产订单数据
	if err = bl_co.GetWorkOrderList(); err != nil {
		return
	}

	return
}

// 获取客户订单的车型零件族
func (bl_co *BL_CustOrder) GetOemorderModelSpecPfdict() (sgdict map[string]string, err error) {
	var (
		i           int
		cocarmodtab db.Pln_custorder_carmodelver
		carmodelid  string
		carmodtab   db.Me_carmodel
	)

	// 初始化
	sgdict = make(map[string]string)
	// 获取客户订单的OEMORDER的车型
	if carmodelid, err = cocarmodtab.GetCarModelForOemorder(bl_co.Custordertab.Projnr, bl_co.Custordertab.Oemordernr); err != nil {
		return
	}
	if carmodelid != "" {
		carmodtab = db.Me_carmodel{Carmodelid: carmodelid}
		if carmodtab, err = carmodtab.SelectOneWithPartfamily(); err != nil {
			return
		}
	}

	// 获取车型的零件族
	for i = 0; i < len(carmodtab.Pflst); i++ {
		sgdict[carmodtab.Pflst[i].Partfamilyid] = carmodtab.Pflst[i].Partfamilyid
		fmt.Println(i, "-->", carmodtab.Pflst[i].Partfamilyid)
	}
	return
}

// 基于订单的项目号、OEM订单号和零件族查询客户订单及其工作订单列表
func (bl_co *BL_CustOrder) IsCarmodelSpecAudited() (audit bool, err error) {
	var (
		cotab   db.Pln_custorder
		vcoslst []db.VCustorderSpec
		i       int
	)

	cotab = db.Pln_custorder{Projnr: bl_co.Projnr, Oemordernr: bl_co.Oemordernr}
	if vcoslst, err = cotab.GetCarModelSpec(); err != nil {
		return
	}
	// 如果不存在整车副规格版本，直接报错
	if len(vcoslst) <= 0 {
		audit = false
		err = errors.New(fmt.Sprintf("项目：%s的OEM订单：%s的车型规格版本不存在!", bl_co.Projnr, bl_co.Oemordernr))
		return
	}
	// 如果存在，遍历所有版本并判断审核状态
	audit = true
	for i = 0; i < len(vcoslst); i++ {
		if vcoslst[i].Me_carmodel_ver.Status <= 0 {
			audit = false
			return
		}
	}

	return
}
