package service

import (
	"fmt"
	"github.com/go-xorm/xorm"
	"leit.com/leit_seat_aps/common"
	"leit.com/leit_seat_aps/db"
	"leit.com/leit_seat_aps/glog"
	"strconv"
	"strings"
	"time"
)

// 订单消息服务，包括拣料单、包装单和发运单
type BL_Ordmsg struct {
	MsgType   string // 消息类型，PICK 拣料单 PACK 包装单 SHIP 发运单
	MsgObjid  string // 消息对象ID
	MsgEvent  string // 消息事件
	MsgiPara1 int
	MsgiPara2 int
	MsgcPara1 string
	MsgcPara2 string
	Ordmsgtab db.Order_msg
}

// 创建订单消息
func (bl_ordmsg *BL_Ordmsg) Create(session *xorm.Session) (err error) {
	var (
		ordsn, prnsn string
		msgid        string
		ordmsgtab    db.Order_msg
	)

	// 获取拣料单序列号
	if ordsn, err = GetSysStringParameter(1, "*", "ORDERMSG_SN"); err != nil {
		return
	}
	if strings.TrimSpace(ordsn) == "" {
		ordsn = "ORDERMSG" // 默认流水ID
	}

	// 获取打印表单头流水号
	if prnsn, err = GetSysStringParameter(1, "*", "PRNSHEETHEAD_SN"); err != nil {
		return
	}
	if strings.TrimSpace(prnsn) == "" {
		prnsn = "PRNSHEETHEAD" // 默认流水ID
	}

	// 获取消息号
	if msgid, err = SN_SyncGetNextSnr(strings.TrimSpace(ordsn)); err != nil {
		return
	}

	// 创建订单消息并返回
	ordmsgtab = db.Order_msg{}
	ordmsgtab.Finr = db.G_FINR
	ordmsgtab.Msgid = msgid
	ordmsgtab.Status = common.MSG_STATUS_PLAN
	ordmsgtab.Printheadsn = prnsn
	ordmsgtab.Msgtype = bl_ordmsg.MsgType
	ordmsgtab.Msgobjid = bl_ordmsg.MsgObjid
	ordmsgtab.Msgevent = bl_ordmsg.MsgEvent
	ordmsgtab.Msg_cpara1 = bl_ordmsg.MsgcPara1
	ordmsgtab.Msg_cpara2 = bl_ordmsg.MsgcPara2
	ordmsgtab.Msg_ipara1 = bl_ordmsg.MsgiPara1
	ordmsgtab.Msg_ipara2 = bl_ordmsg.MsgiPara2
	ordmsgtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	ordmsgtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	ordmsgtab.Lastuser = "service"
	if err = ordmsgtab.Insert(session); err != nil {
		return
	}
	return
}

// 创建订单消息
func (bl_ordmsg *BL_Ordmsg) Add() (err error) {
	var (
		ordsn, prnsn string
		msgid        string
		ordmsgtab    db.Order_msg
	)

	// 获取拣料单序列号
	if ordsn, err = GetSysStringParameter(1, "*", "ORDERMSG_SN"); err != nil {
		return
	}
	if strings.TrimSpace(ordsn) == "" {
		ordsn = "ORDERMSG" // 默认流水ID
	}

	// 获取打印表单头流水号
	if prnsn, err = GetSysStringParameter(1, "*", "PRNSHEETHEAD_SN"); err != nil {
		return
	}
	if strings.TrimSpace(prnsn) == "" {
		prnsn = "PRNSHEETHEAD" // 默认流水ID
	}

	// 获取消息号
	if msgid, err = SN_SyncGetNextSnr(strings.TrimSpace(ordsn)); err != nil {
		return
	}

	// 创建订单消息并返回
	ordmsgtab = db.Order_msg{}
	ordmsgtab.Finr = db.G_FINR
	ordmsgtab.Msgid = msgid
	ordmsgtab.Status = common.MSG_STATUS_PLAN
	ordmsgtab.Printheadsn = prnsn
	ordmsgtab.Msgtype = bl_ordmsg.MsgType
	ordmsgtab.Msgobjid = bl_ordmsg.MsgObjid
	ordmsgtab.Msgevent = bl_ordmsg.MsgEvent
	ordmsgtab.Msg_cpara1 = bl_ordmsg.MsgcPara1
	ordmsgtab.Msg_cpara2 = bl_ordmsg.MsgcPara2
	ordmsgtab.Msg_ipara1 = bl_ordmsg.MsgiPara1
	ordmsgtab.Msg_ipara2 = bl_ordmsg.MsgiPara2
	ordmsgtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	ordmsgtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	ordmsgtab.Lastuser = "service"
	if err = ordmsgtab.Add(); err != nil {
		return
	}

	return
}

// 为客户订单创建消息，如果客户订单消息已存在且未处理则不再创建，如果已存在且已处理或不存在，则新建消息
func (bl_ordmsg *BL_Ordmsg) CreateForOemorderToParseCarmodels() (err error) {
	var (
		ordsn        string
		msgid        string
		ordmsgtab    db.Order_msg
		ordmsgtablst []db.Order_msg
	)

	// 查找是否有未关闭的该客户订单的消息，如果有则返回，如果没有则新建
	ordmsgtab = db.Order_msg{}
	if ordmsgtablst, err = ordmsgtab.GetOemorderOpenCarmodelMessages(bl_ordmsg.MsgObjid); err != nil {
		glog.InfoExtln("解析并读取SEQ", "为客户订单创建车型解析消息失败: ", err)
		return
	}
	if len(ordmsgtablst) > 0 {
		return
	}

	// 获取拣料单序列号
	if ordsn, err = GetSysStringParameter(1, "*", "ORDERMSG_SN"); err != nil {
		return
	}
	if strings.TrimSpace(ordsn) == "" {
		ordsn = "ORDERMSG" // 默认流水ID
	}

	// 获取消息号
	if msgid, err = SN_SyncGetNextSnr(strings.TrimSpace(ordsn)); err != nil {
		glog.InfoExtln("解析并读取SEQ", "为客户订单创建车型解析消息失败: ", err)
		return
	}

	// 创建订单消息并返回
	ordmsgtab = db.Order_msg{}
	ordmsgtab.Finr = db.G_FINR
	ordmsgtab.Msgid = msgid
	ordmsgtab.Status = common.MSG_STATUS_PLAN
	ordmsgtab.Msgtype = bl_ordmsg.MsgType
	ordmsgtab.Msgobjid = bl_ordmsg.MsgObjid
	ordmsgtab.Msgevent = bl_ordmsg.MsgEvent
	ordmsgtab.Msg_cpara1 = bl_ordmsg.MsgcPara1
	ordmsgtab.Msg_cpara2 = bl_ordmsg.MsgcPara2
	ordmsgtab.Msg_ipara1 = bl_ordmsg.MsgiPara1
	ordmsgtab.Msg_ipara2 = bl_ordmsg.MsgiPara2
	ordmsgtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	ordmsgtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	ordmsgtab.Lastuser = "service"
	if err = ordmsgtab.Add(); err != nil {
		glog.InfoExtln("解析并读取SEQ", "为客户订单创建车型解析消息失败: ", err)
		return
	}

	return
}

// 为客户订单创建消息，如果客户订单消息已存在且未处理则不再创建，如果已存在且已处理或不存在，则新建消息
func (bl_ordmsg *BL_Ordmsg) CreateForOemorderToParseCarmodel(session *xorm.Session) (err error) {
	var (
		ordsn        string
		msgid        string
		ordmsgtab    db.Order_msg
		ordmsgtablst []db.Order_msg
	)

	// 查找是否有未关闭的该客户订单的消息，如果有则返回，如果没有则新建
	ordmsgtab = db.Order_msg{}
	if ordmsgtablst, err = ordmsgtab.GetOemorderOpenCarmodelMessages(bl_ordmsg.MsgObjid); err != nil {
		glog.InfoExtln("解析并读取SEQ", "为客户订单创建车型解析消息失败: ", err)
		return
	}
	if len(ordmsgtablst) > 0 {
		return
	}

	// 获取拣料单序列号
	if ordsn, err = GetSysStringParameter(1, "*", "ORDERMSG_SN"); err != nil {
		return
	}
	if strings.TrimSpace(ordsn) == "" {
		ordsn = "ORDERMSG" // 默认流水ID
	}

	// 获取消息号
	if msgid, err = SN_SyncGetNextSnr(strings.TrimSpace(ordsn)); err != nil {
		glog.InfoExtln("解析并读取SEQ", "为客户订单创建车型解析消息失败: ", err)
		return
	}

	// 创建订单消息并返回
	ordmsgtab = db.Order_msg{}
	ordmsgtab.Finr = db.G_FINR
	ordmsgtab.Msgid = msgid
	ordmsgtab.Status = common.MSG_STATUS_PLAN
	ordmsgtab.Msgtype = bl_ordmsg.MsgType
	ordmsgtab.Msgobjid = bl_ordmsg.MsgObjid
	ordmsgtab.Msgevent = bl_ordmsg.MsgEvent
	ordmsgtab.Msg_cpara1 = bl_ordmsg.MsgcPara1
	ordmsgtab.Msg_cpara2 = bl_ordmsg.MsgcPara2
	ordmsgtab.Msg_ipara1 = bl_ordmsg.MsgiPara1
	ordmsgtab.Msg_ipara2 = bl_ordmsg.MsgiPara2
	ordmsgtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	ordmsgtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	ordmsgtab.Lastuser = "service"
	if err = ordmsgtab.Insert(session); err != nil {
		glog.InfoExtln("解析并读取SEQ", "为客户订单创建车型解析消息失败: ", err)
		return
	}

	return
}

// 为拣料单生成打印数据
func (bl_ordmsg *BL_Ordmsg) GeneratePrintDataForPickOrder(session *xorm.Session, pickordtab db.Pln_pickorder, pe *BL_PrintEngine) (err error) {
	var (
		bl_picktmp                                  BL_PickingTemplate
		printheadtab                                db.Printheadtab
		printdetailtab                              db.Printdetailtab
		wotab                                       db.Pln_workorder
		cotab                                       db.Pln_custorder
		atcodtab                                    db.Me_attribute
		atvaltab                                    db.Me_attribute_valst
		ok                                          bool
		bl_sheet                                    BL_2DSheet
		kv, varlst                                  []string
		col, printheadid, val, tmp, colpos          string
		i, j, ibeg, iend, istep, iloop, idx, rowpos int
		bufdata                                     []interface{}
	)

	// 获取模板
	if bl_picktmp, ok = pe.PickingTmpDict[pickordtab.Picktemplateid]; !ok {
		return
	}

	// 获取打印头流水号
	if strings.TrimSpace(bl_ordmsg.Ordmsgtab.Printheadsn) != "" {
		if printheadid, err = SN_GetNextSnrBySession(strings.TrimSpace(bl_ordmsg.Ordmsgtab.Printheadsn), session); err != nil {
			return
		}
	} else {
		if printheadid, err = SN_GetNextSnrBySession("PRNSHEETHEAD", session); err != nil {
			return
		}
	}
	// 创建打印头
	printheadtab = db.Printheadtab{}
	printheadtab.Finr = db.G_FINR
	printheadtab.Printheadid = printheadid
	printheadtab.Status = common.PRN_HEAD_STATUS_NEW
	printheadtab.Printerid = bl_picktmp.Picktemplatetab.Printerid
	printheadtab.Printobjtype = common.MSG_TYPE_PICK
	printheadtab.Printfiletype = "EXCEL"
	printheadtab.Printobjid = pickordtab.Pickorderid
	printheadtab.Templatefile = bl_picktmp.Picktemplatetab.Templatefile
	printheadtab.Orientation = bl_picktmp.Picktemplatetab.Orientation
	printheadtab.Printcopies = common.Max(bl_ordmsg.Ordmsgtab.Printcopies, bl_picktmp.Picktemplatetab.Printqty, 1)
	printheadtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	printheadtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	printheadtab.Lastuser = "service"
	if err = printheadtab.Insert(session); err != nil {
		return
	}

	idx = 1

	// 如果是二维表，初始化变量
	if bl_picktmp.Picktemplatetab.Twodsheet > 0 {
		bl_sheet = BL_2DSheet{}
		bl_sheet.colkeylist = make(map[string]BL_2DSheetKey)
		bl_sheet.rowkeylist = make(map[string]BL_2DSheetKey)
		bl_sheet.colvarvallist = make(map[string]string)
		bl_sheet.rowvarvallist = make(map[string]string)
		if err = bl_sheet.GetColRowKeyData(&bl_picktmp); err != nil {
			return
		}

		// 添加拣料单头
		for i = 0; i < len(pickordtab.Itemlst); i++ {
			// 定义暂存数据的map
			bufdata = []interface{}{}
			// 加载拣料单头
			bufdata = append(bufdata, pickordtab)
			// 加载拣料单行
			bufdata = append(bufdata, pickordtab.Itemlst[i])
			// 加载拣料单行对应的生产订单
			wotab = db.Pln_workorder{Finr: db.G_FINR, Workordernr: pickordtab.Itemlst[i].Serialno}
			if wotab, err = wotab.SelectOne(); err != nil {
				fmt.Printf("拣料单:%s 拣料单项：%s 对应的生产订单: %s 不存在！", pickordtab.Pickorderid, pickordtab.Itemlst[i].Partid, pickordtab.Itemlst[i].Serialno)
				continue
			}
			bufdata = append(bufdata, wotab)
			// 加载生产订单对应的客户订单
			cotab = db.Pln_custorder{Finr: db.G_FINR, Custordernr: wotab.Custordernr}
			if cotab, err = cotab.SelectOne(); err != nil {
				fmt.Printf("生产订单：%s 的客户订单： %s不存在！", wotab.Workordernr, wotab.Custordernr)
				continue
			}
			bufdata = append(bufdata, cotab)

			for j = 0; j < len(bl_picktmp.Picktemplatetab.Printlst); j++ {
				printdetailtab = db.Printdetailtab{}
				printdetailtab.Finr = db.G_FINR
				printdetailtab.Printheadid = printheadid
				printdetailtab.Varname = bl_picktmp.Picktemplatetab.Printlst[j].Varname
				switch bl_picktmp.Picktemplatetab.Printlst[j].Vartype {
				case common.PRN_VAR_TYP_2DCOL: // 二维表列变量
					// 获取拣料单项额外的内存数据，包括生产订单、客户订单和属性
					// 获取基于指定属性在生产订单中对应的属性对象及其值对象
					if bl_picktmp.Picktemplatetab.Printlst[j].Keyval != "" {
						atcodtab, atvaltab = pe.GetPickOrderItemAttributeData(&bl_picktmp.Picktemplatetab.Printlst[j], &wotab)
						// 将拣料单行数据写入缓存数据
						bufdata = append(bufdata, atcodtab)
						bufdata = append(bufdata, atvaltab)
					}

					kvmap := common.StructToMap(bufdata)

					// 获取模板列打印变量的行坐标值和实际值
					if ok, rowpos, val = bl_sheet.GetColCurrRowPos(&bl_picktmp.Picktemplatetab.Printlst[j], kvmap); !ok {
						continue
					}
					kv = strings.Split(bl_picktmp.Picktemplatetab.Printlst[j].Varpos, ":")
					if len(kv) < 3 {
						continue
					}
					col = kv[0] + strconv.Itoa(rowpos)

					// 判断是否已存在
					if _, ok = bl_sheet.colvarvallist[col]; ok {
						continue
					} else {
						bl_sheet.colvarvallist[col] = val
					}

					printdetailtab.Varpos = col
					if val, err = GetValueByPicture(val, bl_picktmp.Picktemplatetab.Printlst[j].Picture, bl_picktmp.Picktemplatetab.Printlst[j].Varpos); err != nil {
						return
					}
					printdetailtab.Pos = idx
					printdetailtab.Varvalue = val
					printdetailtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
					printdetailtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
					printdetailtab.Lastuser = "service"
					if err = printdetailtab.Insert(session); err != nil {
						return
					}
					idx++

				case common.PRN_VAR_TYP_2DROW: // 二维表行变量
					// 获取基于指定属性在生产订单中对应的属性对象及其值对象
					if bl_picktmp.Picktemplatetab.Printlst[j].Keyval != "" {
						atcodtab, atvaltab = pe.GetPickOrderItemAttributeData(&bl_picktmp.Picktemplatetab.Printlst[j], &wotab)
						// 将拣料单行数据写入缓存数据
						bufdata = append(bufdata, atcodtab)
						bufdata = append(bufdata, atvaltab)
					}

					kvmap := common.StructToMap(bufdata)

					// 获取模板列打印变量的行坐标值和实际值
					if ok, colpos, val = bl_sheet.GetRowCurrColPos(&bl_picktmp.Picktemplatetab.Printlst[j], kvmap); !ok {
						continue
					}
					kv = strings.Split(bl_picktmp.Picktemplatetab.Printlst[j].Varpos, ":")
					if len(kv) < 3 {
						continue
					}
					col = colpos + kv[0]

					// 判断是否已存在
					if _, ok = bl_sheet.rowvarvallist[col]; ok {
						continue
					} else {
						bl_sheet.rowvarvallist[col] = val
					}

					printdetailtab.Varpos = col
					if val, err = GetValueByPicture(val, bl_picktmp.Picktemplatetab.Printlst[j].Picture, bl_picktmp.Picktemplatetab.Printlst[j].Varpos); err != nil {
						return
					}
					printdetailtab.Pos = idx
					printdetailtab.Varvalue = val
					printdetailtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
					printdetailtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
					printdetailtab.Lastuser = "service"
					if err = printdetailtab.Insert(session); err != nil {
						return
					}
					idx++

				case common.PRN_VAR_TYP_2DCELL: // 二维表单元变量
					// 将拣料单行数据写入缓存数据
					bufdata = append(bufdata, pickordtab.Itemlst[i])
					kvmap := common.StructToMap(bufdata)

					// 获取模板列打印变量的行坐标值和实际值
					colpos, rowpos, val = bl_sheet.GetCellCurrPos(&bl_picktmp.Picktemplatetab.Printlst[j], kvmap)

					col = colpos + strconv.Itoa(rowpos)
					printdetailtab.Varpos = col
					if val, err = GetValueByPicture(val, bl_picktmp.Picktemplatetab.Printlst[j].Picture, bl_picktmp.Picktemplatetab.Printlst[j].Varpos); err != nil {
						return
					}
					printdetailtab.Pos = idx
					printdetailtab.Varvalue = val
					printdetailtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
					printdetailtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
					printdetailtab.Lastuser = "service"
					if err = printdetailtab.Insert(session); err != nil {
						return
					}
					idx++

				case common.PRN_VAR_TYP_FORMULAR: //公式求和
					//example := "J:7:30",最后求和:(i+1) == len(pickordtab.Itemlst)
					if (i + 1) == len(pickordtab.Itemlst) {
						// 获取起始坐标
						kv := strings.Split(bl_picktmp.Picktemplatetab.Printlst[j].Varpos, ":")
						if len(kv) >= 3 {
							ibeg, _ = strconv.Atoi(kv[1])
							iend, _ = strconv.Atoi(kv[2])
							for i := ibeg; i <= iend; i++ {
								col = kv[0] + common.ValueToString(i, "")
								// 判断是否已存在
								if _, ok = bl_sheet.rowvarvallist[col]; ok {
									continue
								} else {
									bl_sheet.rowvarvallist[col] = val
								}
								printdetailtab.Varpos = col
								printdetailtab.Pos = idx
								printdetailtab.Varvalue = "0"
								printdetailtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
								printdetailtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
								printdetailtab.Lastuser = "service"
								if err = printdetailtab.Insert(session); err != nil {
									glog.InfoExtln("", "PICK公式求和", "err is :", err)
									continue
								}
								idx++
							}
						}

					}
				default:
					continue
				}
			}
		}
	}

	// 非二维表
	for i = 0; i < len(bl_picktmp.Picktemplatetab.Printlst); i++ {
		// 定义暂存数据的map
		bufdata := []interface{}{}
		// 添加拣料单头
		bufdata = append(bufdata, pickordtab)
		kvmap := common.StructToMap(bufdata)

		// 初始化打印明细数据对象
		printdetailtab = db.Printdetailtab{
			Finr:        db.G_FINR,
			Printheadid: printheadid,
			Pos:         idx,
			Varname:     bl_picktmp.Picktemplatetab.Printlst[i].Varname,
			Varpos:      bl_picktmp.Picktemplatetab.Printlst[i].Varpos}
		val = ""

		switch bl_picktmp.Picktemplatetab.Printlst[i].Vartype {
		// 一维表常量
		case common.PRN_VAR_TYP_VALUE: // 静态值
			val = bl_picktmp.Picktemplatetab.Printlst[i].Varvalue
		// 一维表动态遍历
		case common.PRN_VAR_TYP_VARIABLE: // 动态变量
			if val, ok = kvmap[bl_picktmp.Picktemplatetab.Printlst[i].Varvalue]; !ok {
				val = ""
			}
		// 一维表系统遍历
		case common.PRN_VAR_TYP_SYSVAR: // 系统参数
			switch bl_picktmp.Picktemplatetab.Printlst[i].Varvalue {
			case "DATE()":
				val = common.Date(time.Now().Unix(), "YYYYMMDD")
			case "TIME()":
				val = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
			}
		// 一维表变量集
		case common.PRN_VAR_TYP_VARSET: // 动态变量集
			varlst = strings.Split(bl_picktmp.Picktemplatetab.Printlst[i].Varvalue, ";") // 获取变量集
			if len(varlst) <= 0 {
				continue
			} else {
				for j = 0; j < len(varlst); j++ {
					if tmp, ok = kvmap[varlst[j]]; ok {
						val = val + tmp
					}
				}
			}
		// 一维表循环遍历
		case common.PRN_VAR_TYP_LOOPVAR: // 循环变量
			for j = 0; j < len(pickordtab.Itemlst); j++ {
				// 将拣料单行数据写入缓存数据
				bufdata = append(bufdata, pickordtab.Itemlst[j])
				// 加载拣料单行对应的生产订单
				wotab = db.Pln_workorder{Finr: db.G_FINR, Workordernr: pickordtab.Itemlst[j].Serialno}
				if wotab, err = wotab.SelectOne(); err != nil {
					fmt.Printf("拣料单:%s 拣料单项：%s 对应的生产订单: %s 不存在！", pickordtab.Pickorderid, pickordtab.Itemlst[j].Partid, pickordtab.Itemlst[j].Serialno)
					continue
				}
				bufdata = append(bufdata, wotab)
				// 加载生产订单对应的客户订单

				cotab = db.Pln_custorder{Finr: db.G_FINR, Custordernr: wotab.Custordernr}
				if cotab, err = cotab.SelectOne(); err != nil {
					fmt.Printf("生产订单：%s 的客户订单： %s不存在！", wotab.Workordernr, wotab.Custordernr)
					continue
				}
				bufdata = append(bufdata, cotab)
				kvmap := common.StructToMap(bufdata)
				/**for k,v := range kvmap {
					fmt.Println("KV value:",k,"==>",v)
				}**/
				// 获取起始坐标
				kv = strings.Split(bl_picktmp.Picktemplatetab.Printlst[i].Varpos, ":")
				if len(kv) >= 3 {
					ibeg, _ = strconv.Atoi(kv[1])
					iend, _ = strconv.Atoi(kv[2])
				}
				if len(kv) >= 4 {
					istep, _ = strconv.Atoi(kv[3])
				} else {
					istep = 0
				}
				if j > (iend - ibeg) {
					if istep <= 0 {
						break
					} else {
						iloop = (j) / (iend - ibeg + 1)
						col = common.AddExcelColLen(kv[0], istep*iloop) + strconv.Itoa(ibeg+j-iloop*(iend-ibeg+1))
					}
				} else {
					col = kv[0] + strconv.Itoa(ibeg+j)
				}

				printdetailtab.Varpos = col
				if val, ok = kvmap[bl_picktmp.Picktemplatetab.Printlst[i].Varvalue]; !ok {
					val = ""
				}
				if val, err = GetValueByPicture(val, bl_picktmp.Picktemplatetab.Printlst[i].Picture, bl_picktmp.Picktemplatetab.Printlst[i].Varpos); err != nil {
					return
				}
				printdetailtab.Pos = idx
				printdetailtab.Varvalue = val
				printdetailtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
				printdetailtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
				printdetailtab.Lastuser = "service"
				if err = printdetailtab.Insert(session); err != nil {
					return
				}
				idx++
			}
			continue
			// 如果是其它类型的变量则跳过不做处理
		default:
			continue
		}
		// 创建打印行记录
		if val, err = GetValueByPicture(val, bl_picktmp.Picktemplatetab.Printlst[i].Picture, bl_picktmp.Picktemplatetab.Printlst[i].Varpos); err != nil {
			return
		}
		printdetailtab.Varvalue = val
		printdetailtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
		printdetailtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
		printdetailtab.Lastuser = "service"
		if err = printdetailtab.Insert(session); err != nil {
			return
		}

		idx++
	}

	return
}

// 为包装单生成打印数据
func (bl_ordmsg *BL_Ordmsg) GeneratePrintDataForPackOrder(session *xorm.Session, packordtab db.Jit_packorder, pe *BL_PrintEngine) (err error) {
	var (
		bl_packtmp                                          BL_Package
		printheadtab                                        db.Printheadtab
		printdetailtab                                      db.Printdetailtab
		wotab                                               db.Pln_workorder
		cotab                                               db.Pln_custorder
		supplygroup                                         db.Me_supplygroup
		printheadid, val, tmp, col                          string
		kv, varlst                                          []string
		ok                                                  bool
		i, j, ibeg, iend, idx, istep, iloop, icopy, icopies int
	)
	// 获取模板
	if bl_packtmp, ok = pe.PackTmpDict[packordtab.Packtemplateid]; !ok {
		return
	}
	// 如果打印多份
	icopies = common.Max(bl_ordmsg.Ordmsgtab.Printcopies, bl_packtmp.Paktmpltab.Printqty, 1)
	for icopy = 0; icopy < icopies; icopy++ {
		// 获取打印头流水号
		if strings.TrimSpace(bl_ordmsg.Ordmsgtab.Printheadsn) != "" {
			if printheadid, err = SN_GetNextSnrBySession(strings.TrimSpace(bl_ordmsg.Ordmsgtab.Printheadsn), session); err != nil {
				return
			}
		} else {
			if printheadid, err = SN_GetNextSnrBySession("PRNSHEETHEAD", session); err != nil {
				return
			}
		}
		// 创建打印头
		printheadtab = db.Printheadtab{}
		printheadtab.Finr = db.G_FINR
		printheadtab.Printheadid = printheadid
		printheadtab.Status = common.PRN_HEAD_STATUS_NEW
		printheadtab.Printerid = bl_packtmp.Paktmpltab.Printerid
		printheadtab.Printobjtype = common.MSG_TYPE_PACK
		printheadtab.Printobjid = packordtab.Packorderid
		printheadtab.Printfiletype = "EXCEL"
		if strings.TrimSpace(packordtab.Templatefile) == "" {
			printheadtab.Templatefile = bl_packtmp.Paktmpltab.Templatefile
		} else {
			printheadtab.Templatefile = packordtab.Templatefile
		}
		printheadtab.Orientation = bl_packtmp.Paktmpltab.Orientation
		printheadtab.Printcopies = common.Max(bl_ordmsg.Ordmsgtab.Printcopies, bl_packtmp.Paktmpltab.Printqty, 1)
		printheadtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
		printheadtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
		printheadtab.Lastuser = "service"
		if err = printheadtab.Insert(session); err != nil {
			return
		}
		// 遍历打印设置条码
		idx = 1
		detailmap := make(map[string]string)
		for i = 0; i < len(bl_packtmp.Paktmpltab.Printlst); i++ {
			// 定义暂存数据的map
			bufdata := []interface{}{}
			bufdata = append(bufdata, packordtab)
			kvmap := common.StructToMap(bufdata)
			// 初始化打印明细数据对象
			printdetailtab = db.Printdetailtab{
				Finr:        db.G_FINR,
				Printheadid: printheadid,
				Pos:         idx,
				Varname:     bl_packtmp.Paktmpltab.Printlst[i].Varname,
				Varpos:      bl_packtmp.Paktmpltab.Printlst[i].Varpos}
			val = ""
			if len(packordtab.Itemlst) == 0{
				glog.InfoExtln("packordtab","packordtab.orderid",packordtab.Packorderid)
				continue
			}
			if packordtab.Itemlst[0].Workordernr != "" {
				wotab = db.Pln_workorder{Finr: db.G_FINR, Workordernr: packordtab.Itemlst[0].Workordernr}
				if wotab, err = wotab.SelectOne(); err != nil {
					return
				}
				bufdata = append(bufdata, wotab)
				co := db.Pln_custorder{Finr: db.G_FINR, Custordernr: wotab.Custordernr}
				if cotab, err = co.SelectOne(); err != nil {
					return
				}
				bufdata = append(bufdata, cotab)
				kvmap = common.StructToMap(bufdata)
			}
			switch bl_packtmp.Paktmpltab.Printlst[i].Vartype {
			case common.PRN_VAR_TYP_VALUE: // 静态值
				val = bl_packtmp.Paktmpltab.Printlst[i].Varvalue

			case common.PRN_VAR_TYP_VARIABLE: // 动态变量
				if val, ok = kvmap[bl_packtmp.Paktmpltab.Printlst[i].Varvalue]; !ok {
					val = ""
				}
			case common.PRN_VAR_TYP_VARSET: // 动态变量集
				for k, v := range detailmap {
					key := strings.TrimSpace(k)
					value := strings.TrimSpace(v)
					kvmap[key] = value
				}
				// 获取变量集
				varlst = strings.Split(bl_packtmp.Paktmpltab.Printlst[i].Varvalue, ";")
				if len(varlst) <= 0 {
					continue
				} else {
					for j = 0; j < len(varlst); j++ {
						if tmp, ok = kvmap[varlst[j]]; ok {
							val = val + tmp
						}
					}
				}
			case common.PRN_VAR_TYP_LOOPVAR: // 循环变量
				for j = 0; j < len(packordtab.Itemlst); j++ {
					//加载模板
					bufdata = append(bufdata, bl_packtmp.Paktmpltab)
					// 将拣料单行数据写入缓存数据
					bufdata = append(bufdata, packordtab.Itemlst[j])
					wotab = db.Pln_workorder{Finr: db.G_FINR, Workordernr: packordtab.Itemlst[j].Workordernr}
					if wotab, err = wotab.SelectOne(); err != nil {
						return
					}
					bufdata = append(bufdata, wotab)
					supplygroup = db.Me_supplygroup{Finr: db.G_FINR, Supplygroupid: wotab.Supplygroupid}
					if supplygroup, err = supplygroup.SelectOne(); err != nil {
						return
					}
					bufdata = append(bufdata, supplygroup)
					co := db.Pln_custorder{Finr: db.G_FINR, Custordernr: wotab.Custordernr}
					if cotab, err = co.SelectOne(); err != nil {
						return
					}
					bufdata = append(bufdata, cotab)
					kvmap := common.StructToMap(bufdata)
					if common.ValueIsEmpty(packordtab.Itemlst[j].Workordernr) {
						kvmap["pln_workorder.oemseq"] = ""
					}
					/**for k, v := range kvmap {
						fmt.Println("KV value:", k, "==>", v)
					}**/
					// 获取起始坐标
					kv = strings.Split(bl_packtmp.Paktmpltab.Printlst[i].Varpos, ":")
					if len(kv) >= 3 {
						ibeg, _ = strconv.Atoi(kv[1])
						iend, _ = strconv.Atoi(kv[2])
					}
					if len(kv) >= 4 {
						istep, _ = strconv.Atoi(kv[3])
					} else {
						istep = 0
					}
					if j > (iend - ibeg) {
						if istep <= 0 {
							break
						} else {
							iloop = (j) / (iend - ibeg + 1)
							col = common.AddExcelColLen(kv[0], istep*iloop) + strconv.Itoa(ibeg+j-iloop*(iend-ibeg+1))
						}
					} else {
						col = kv[0] + strconv.Itoa(ibeg+j)
					}
					printdetailtab.Varpos = col
					if val, ok = kvmap[bl_packtmp.Paktmpltab.Printlst[i].Varvalue]; !ok {
						val = ""
					}
					if val, err = GetValueByPicture(val, bl_packtmp.Paktmpltab.Printlst[i].Picture, bl_packtmp.Paktmpltab.Printlst[i].Varpos); err != nil {
						return
					}
					printdetailtab.Pos = idx
					printdetailtab.Varvalue = val
					printdetailtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
					printdetailtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
					printdetailtab.Lastuser = "service"
					if err = printdetailtab.Insert(session); err != nil {
						return
					}
					idx++
				}
				continue

			case common.PRN_VAR_TYP_SYSVAR: // 系统参数2
				switch bl_packtmp.Paktmpltab.Printlst[i].Varvalue {
				case "DATE()":
					val = common.Date(time.Now().Unix(), "YYYYMMDD")
				case "TIME()":
					val = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
				case "COPY":
					val = GetPrintSysCopyValue(icopy+1, bl_packtmp.Paktmpltab.Printlst[i].Picture)
				}

			default:
				continue
			}

			if val, err = GetValueByPicture(val, bl_packtmp.Paktmpltab.Printlst[i].Picture, bl_packtmp.Paktmpltab.Printlst[i].Varpos); err != nil {
				return
			}

			printdetailtab.Varvalue = val
			printdetailtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
			printdetailtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
			printdetailtab.Lastuser = "service"
			if err = printdetailtab.Insert(session); err != nil {
				return
			}
			varname := strings.TrimSpace(bl_packtmp.Paktmpltab.Printlst[i].Varname)
			detailmap[varname] = val
			idx++
		}
	}
	return
}

// 为发运单生成打印数据
func (bl_ordmsg *BL_Ordmsg) GeneratePrintDataForShipOrder(session *xorm.Session, shipordtab db.Jit_shiporder, pe *BL_PrintEngine) (err error) {
	var (
		bl_shiptmp                          BL_Shipcar
		printheadtab                        db.Printheadtab
		printdetailtab                      db.Printdetailtab
		kv, varlst                          []string
		ok                                  bool
		printheadid, val, tmp, col          string
		i, j, ibeg, iend, idx, istep, iloop int
	)

	// 获取模板
	if bl_shiptmp, ok = pe.ShipTmpDict[shipordtab.Shiptemplateid]; !ok {
		return
	}
	// 获取打印头流水号
	if strings.TrimSpace(bl_ordmsg.Ordmsgtab.Printheadsn) != "" {
		if printheadid, err = SN_GetNextSnrBySession(strings.TrimSpace(bl_ordmsg.Ordmsgtab.Printheadsn), session); err != nil {
			return
		}
	} else {
		if printheadid, err = SN_GetNextSnrBySession("PRNSHEETHEAD", session); err != nil {
			return
		}
	}
	// 创建打印头
	printheadtab = db.Printheadtab{}
	printheadtab.Finr = db.G_FINR
	printheadtab.Printheadid = printheadid
	printheadtab.Status = common.PRN_HEAD_STATUS_NEW
	printheadtab.Printerid = bl_shiptmp.Shipcartab.Printerid
	printheadtab.Printobjtype = common.MSG_TYPE_PICK
	printheadtab.Printobjid = shipordtab.Shiporderid
	printheadtab.Printfiletype = "EXCEL"
	if shipordtab.Shiptype == "REO" {
		printheadtab.Templatefile = "REO_" + bl_shiptmp.Shipcartab.Templatefile
	} else {
		printheadtab.Templatefile = bl_shiptmp.Shipcartab.Templatefile
	}
	printheadtab.Orientation = bl_shiptmp.Shipcartab.Orientation
	printheadtab.Printcopies = common.Max(bl_ordmsg.Ordmsgtab.Printcopies, bl_shiptmp.Shipcartab.Printqty, 1)
	printheadtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	printheadtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	printheadtab.Lastuser = "service"
	if err = printheadtab.Insert(session); err != nil {
		return
	}

	// 遍历打印设置条码
	idx = 1
	for i = 0; i < len(bl_shiptmp.Shipcartab.Printlst); i++ {
		// 定义暂存数据的map
		bufdata := []interface{}{}
		bufdata = append(bufdata, shipordtab)
		kvmap := common.StructToMap(bufdata)

		// 初始化打印明细数据对象
		printdetailtab = db.Printdetailtab{
			Finr:        db.G_FINR,
			Printheadid: printheadid,
			Pos:         idx,
			Varname:     bl_shiptmp.Shipcartab.Printlst[i].Varname,
			Varpos:      bl_shiptmp.Shipcartab.Printlst[i].Varpos}
		val = ""

		switch bl_shiptmp.Shipcartab.Printlst[i].Vartype {
		case common.PRN_VAR_TYP_VALUE: // 静态值
			val = bl_shiptmp.Shipcartab.Printlst[i].Varvalue

		case common.PRN_VAR_TYP_VARIABLE: // 动态变量
			if val, ok = kvmap[bl_shiptmp.Shipcartab.Printlst[i].Varvalue]; !ok {
				val = ""
			}

		case common.PRN_VAR_TYP_VARSET: // 动态变量集
			// 获取变量集
			varlst = strings.Split(bl_shiptmp.Shipcartab.Printlst[i].Varvalue, ";")
			if len(varlst) <= 0 {
				continue
			} else {
				for j = 0; j < len(varlst); j++ {
					if tmp, ok = kvmap[varlst[j]]; ok {
						val = val + tmp
					}
				}
			}

		case common.PRN_VAR_TYP_LOOPVAR: // 循环变量
			for j = 0; j < len(shipordtab.Itemlst); j++ {
				// 将拣料单行数据写入缓存数据
				bufdata = append(bufdata, shipordtab.Itemlst[j])
				kvmap := common.StructToMap(bufdata)
				/**for k,v := range kvmap {
					fmt.Println("KV value:",k,"==>",v)
				}**/
				// 获取起始坐标
				kv = strings.Split(bl_shiptmp.Shipcartab.Printlst[i].Varpos, ":")
				if len(kv) >= 3 {
					ibeg, _ = strconv.Atoi(kv[1])
					iend, _ = strconv.Atoi(kv[2])
				}
				if len(kv) >= 4 {
					istep, _ = strconv.Atoi(kv[3])
				} else {
					istep = 0
				}

				if j > (iend - ibeg) {
					if istep <= 0 {
						break
					} else {
						iloop = (j) / (iend - ibeg + 1)
						col = common.AddExcelColLen(kv[0], istep*iloop) + strconv.Itoa(ibeg+j-iloop*(iend-ibeg+1))
					}
				} else {
					col = kv[0] + strconv.Itoa(ibeg+j)
				}

				printdetailtab.Varpos = col
				if val, ok = kvmap[bl_shiptmp.Shipcartab.Printlst[i].Varvalue]; !ok {
					val = ""
				}
				if val, err = GetValueByPicture(val, bl_shiptmp.Shipcartab.Printlst[i].Picture, bl_shiptmp.Shipcartab.Printlst[i].Varpos); err != nil {
					return
				}
				printdetailtab.Pos = idx
				printdetailtab.Varvalue = val
				printdetailtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
				printdetailtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
				printdetailtab.Lastuser = "service"
				if err = printdetailtab.Insert(session); err != nil {
					return
				}
				idx++
			}
			continue

		case common.PRN_VAR_TYP_SYSVAR: // 系统参数
			switch bl_shiptmp.Shipcartab.Printlst[i].Varvalue {
			case "DATE()":
				val = common.Date(time.Now().Unix(), "YYYYMMDD")
			case "TIME()":
				val = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
			}

		default:
			continue
		}

		if val, err = GetValueByPicture(val, bl_shiptmp.Shipcartab.Printlst[i].Picture, bl_shiptmp.Shipcartab.Printlst[i].Varpos); err != nil {
			return
		}

		printdetailtab.Varvalue = val
		printdetailtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
		printdetailtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
		printdetailtab.Lastuser = "service"
		if err = printdetailtab.Insert(session); err != nil {
			return
		}

		idx++
	}

	return
}

// 为BBA ASN生成打印数据
func (bl_ordmsg *BL_Ordmsg) GeneratePrintDataForBbaasn(session *xorm.Session, bbaasntab db.Jit_bbaasn, pe *BL_PrintEngine) (err error) {
	var (
		bbaasntmptab                               db.Jit_bbaasntemplate
		printheadtab                               db.Printheadtab
		printdetailtab                             db.Printdetailtab
		cust                                       db.Pln_custorder
		ok                                         bool
		kv, varlst                                 []string
		col, printheadid, val, tmp                 string
		i, j, ibeg, iend, istep, iloop, idx, bstep int
		bufdata                                    []interface{}
		shipordtab                                 db.Jit_shiporder
	)

	// 获取模板
	bbaasntmptab = db.Jit_bbaasntemplate{}
	if bbaasntmptab, ok = pe.BbaasnTmpDict[bbaasntab.Asntemplateid]; !ok {
		return
	}

	// 获取打印头流水号
	if strings.TrimSpace(bl_ordmsg.Ordmsgtab.Printheadsn) != "" {
		if printheadid, err = SN_GetNextSnrBySession(strings.TrimSpace(bl_ordmsg.Ordmsgtab.Printheadsn), session); err != nil {
			return
		}
	} else {
		if printheadid, err = SN_GetNextSnrBySession("PRNSHEETHEAD", session); err != nil {
			return
		}
	}
	//查询shipordtab信息
	shipordtab = db.Jit_shiporder{Shiporderid: bbaasntab.Shiporderid}
	if shipordtab, err = shipordtab.SelectOne(); err != nil {
		glog.InfoExtln("Msg解析", "解析打印消息失败：", err)
		return
	}
	// 创建打印头
	printheadtab = db.Printheadtab{}
	printheadtab.Finr = db.G_FINR
	printheadtab.Printheadid = printheadid
	printheadtab.Status = common.PRN_HEAD_STATUS_NEW
	printheadtab.Printerid = bbaasntmptab.Printerid
	printheadtab.Printobjtype = common.MSG_TYPE_BBAASN
	printheadtab.Printfiletype = "EXCEL"
	printheadtab.Printobjid = bbaasntab.Asn
	if shipordtab.Shiptype == "REO" {
		printheadtab.Templatefile = "REO_" + bbaasntmptab.Templatefile
	} else {
		printheadtab.Templatefile = bbaasntmptab.Templatefile
	}
	printheadtab.Orientation = bbaasntmptab.Orientation
	printheadtab.Printcopies = common.Max(bl_ordmsg.Ordmsgtab.Printcopies, bbaasntmptab.Printqty, 1)
	printheadtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	printheadtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	printheadtab.Lastuser = "service"
	if err = printheadtab.Insert(session); err != nil {
		glog.InfoExtln("Msg解析", "解析打印消息失败：", err)
		return
	}

	idx = 1

	// 如果是二维表，初始化变量

	// 非二维表
	for i = 0; i < len(bbaasntmptab.Printlst); i++ {
		// 定义暂存数据的map
		bufdata = []interface{}{}
		// 添加BBA ASN头
		bufdata = append(bufdata, bbaasntab)
		kvmap := common.StructToMap(bufdata)

		// 初始化打印明细数据对象
		printdetailtab = db.Printdetailtab{
			Finr:        db.G_FINR,
			Printheadid: printheadid,
			Pos:         idx,
			Varname:     bbaasntmptab.Printlst[i].Varname,
			Varpos:      bbaasntmptab.Printlst[i].Varpos}
		val = ""

		switch bbaasntmptab.Printlst[i].Vartype {
		// 一维表常量
		case common.PRN_VAR_TYP_VALUE: // 静态值
			val = bbaasntmptab.Printlst[i].Varvalue
		// 一维表动态遍历
		case common.PRN_VAR_TYP_VARIABLE: // 动态变量
			if val, ok = kvmap[bbaasntmptab.Printlst[i].Varvalue]; !ok {
				val = ""
			}
		// 一维表系统遍历
		case common.PRN_VAR_TYP_SYSVAR: // 系统参数
			switch bbaasntmptab.Printlst[i].Varvalue {
			case "DATE()":
				val = common.Date(time.Now().Unix(), "YYYYMMDD")
			case "TIME()":
				val = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
			case "HOUR()":
				val = common.Date(time.Now().Unix(), "HHmmss")
			}
		// 一维表变量集
		case common.PRN_VAR_TYP_VARSET: // 动态变量集
			varlst = strings.Split(bbaasntmptab.Printlst[i].Varvalue, ";") // 获取变量集
			if len(varlst) <= 0 {
				continue
			} else {
				for j = 0; j < len(varlst); j++ {
					if tmp, ok = kvmap[varlst[j]]; ok {
						val = val + tmp
					}
				}
			}
		// 一维表循环遍历
		case common.PRN_VAR_TYP_LOOPVAR: // 循环变量
			for j = 0; j < len(bbaasntab.Itemlst); j++ {
				// 将拣料单行数据写入缓存数据
				bufdata = append(bufdata, bbaasntab.Itemlst[j])
				//查询客户订单
				cust = db.Pln_custorder{Finr: db.G_FINR, Projnr: bbaasntab.Projnr, Oemordernr: bbaasntab.Itemlst[j].Oemordernr, Calloffnr: bbaasntab.Itemlst[j].Calloffnr}
				if cust, err = cust.SelectCustOne(); err != nil {
					return err
				}
				bufdata = append(bufdata, cust)
				kvmap := common.StructToMap(bufdata)
				/**for k,v := range kvmap {
					fmt.Println("KV value:",k,"==>",v)
				}**/
				// 获取起始坐标
				kv = strings.Split(bbaasntmptab.Printlst[i].Varpos, ":")
				if len(kv) >= 3 {
					ibeg, _ = strconv.Atoi(kv[1])
					iend, _ = strconv.Atoi(kv[2])
				}
				if len(kv) >= 4 {
					step := strings.Split(kv[3], "-")
					if len(step) > 1 {
						bstep, _ = strconv.Atoi(step[0])
						istep, _ = strconv.Atoi(step[1])
					} else {
						istep, _ = strconv.Atoi(kv[3])
						bstep = 0
					}
				} else {
					istep = 0
				}
				if j > (iend - ibeg) {
					if istep <= 0 {
						break
					} else {
						if bstep > 0 {
							if (ibeg + j) > bstep {
								col = kv[0] + strconv.Itoa(ibeg+j+istep)
							} else {
								col = kv[0] + strconv.Itoa(ibeg+j)
							}

						} else {
							iloop = (j) / (iend - ibeg + 1)
							col = common.AddExcelColLen(kv[0], istep*iloop) + strconv.Itoa(ibeg+j-iloop*(iend-ibeg+1))
						}
					}
				} else {
					if bstep > 0 {
						if (ibeg + j) > bstep {
							col = kv[0] + strconv.Itoa(ibeg+j+istep)
						} else {
							col = kv[0] + strconv.Itoa(ibeg+j)
						}

					} else {
						col = kv[0] + strconv.Itoa(ibeg+j)
					}
				}

				printdetailtab.Varpos = col
				if val, ok = kvmap[bbaasntmptab.Printlst[i].Varvalue]; !ok {
					val = ""
				}
				if val, err = GetValueByPicture(val, bbaasntmptab.Printlst[i].Picture, bbaasntmptab.Printlst[i].Varpos); err != nil {
					glog.InfoExtln("bbaasn打印消息", "bbaasntmptab.Printlst[i].Varpos:", bbaasntmptab.Printlst[i].Varpos)
					glog.InfoExtln("bbaasn打印消息", "GetValueByPicture err is:", err)
					return
				}
				printdetailtab.Pos = idx
				printdetailtab.Varvalue = val
				printdetailtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
				printdetailtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
				printdetailtab.Lastuser = "service"
				if err = printdetailtab.Insert(session); err != nil {
					glog.InfoExtln("bbaasn打印消息", "bbaasntmptab.Printlst[i].Varpos:", bbaasntmptab.Printlst[i].Varpos)
					glog.InfoExtln("bbaasn打印消息", "GetValueByPicture err is:", err)
					return
				}
				idx++
			}
			continue
		// 如果是其它类型的变量则跳过不做处理
		default:
			continue
		}
		// 创建打印行记录
		if val, err = GetValueByPicture(val, bbaasntmptab.Printlst[i].Picture, bbaasntmptab.Printlst[i].Varpos); err != nil {
			glog.InfoExtln("bbaasn打印消息", "bbaasntmptab.Printlst[i].Varpos:", bbaasntmptab.Printlst[i].Varpos)
			glog.InfoExtln("bbaasn打印消息", "GetValueByPicture err is:", err)
			return
		}
		printdetailtab.Varvalue = val
		printdetailtab.Credatuz = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
		printdetailtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
		printdetailtab.Lastuser = "service"
		if err = printdetailtab.Insert(session); err != nil {
			glog.InfoExtln("bbaasn打印消息", "bbaasntmptab.Printlst[i].Varpos:", bbaasntmptab.Printlst[i].Varpos)
			glog.InfoExtln("bbaasn打印消息", "GetValueByPicture err is:", err)
			return
		}

		idx++
	}

	return
}

// 关闭订单消息
func (bl_ordmsg *BL_Ordmsg) Close() (err error) {
	bl_ordmsg.Ordmsgtab.Status = common.MSG_STATUS_FINISHED
	bl_ordmsg.Ordmsgtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	bl_ordmsg.Ordmsgtab.Lastuser = "service"
	if err = bl_ordmsg.Ordmsgtab.UpdateFieldsWithoutSession("status,lastmodif,lastuser"); err != nil {
		return
	}
	return
}

// 挂起订单消息
func (bl_ordmsg *BL_Ordmsg) Pending(errinfo string) (err error) {
	bl_ordmsg.Ordmsgtab.Status = common.MSG_STATUS_PENDING
	bl_ordmsg.Ordmsgtab.Errinfo = errinfo
	bl_ordmsg.Ordmsgtab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
	bl_ordmsg.Ordmsgtab.Lastuser = "service"
	if err = bl_ordmsg.Ordmsgtab.UpdateFieldsWithoutSession("status,errinfo,lastmodif,lastuser"); err != nil {
		return
	}
	return
}

// 订单消息是否超时未处理，如果是则生成对应的报警信息
func (bl_ordmsg *BL_Ordmsg) IsNeedToAlert(overtime_second float64) (alert bool, alertinfo string, err error) {
	var (
		createtime time.Time
		dura       time.Duration
	)

	alert = false
	// 判断消息状态
	if bl_ordmsg.Ordmsgtab.Status >= common.MSG_STATUS_FINISHED {
		return
	}
	// 未完成消息是否超时
	if createtime, err = common.DateParse(bl_ordmsg.Ordmsgtab.Credatuz, "YmdHis"); err != nil {
		return
	}
	dura = time.Now().Sub(createtime)
	if dura.Seconds() > overtime_second {
		alert = true
		alertinfo = fmt.Sprintf("类型为：%s 对象为：%s 的消息未处理！", bl_ordmsg.Ordmsgtab.Msgtype, bl_ordmsg.Ordmsgtab.Msgobjid)
		return
	}
	return
}
