package tod

import (
	"errors"
	"fmt"
	"github.com/360EntSecGroup-Skylar/excelize"
	"github.com/go-xorm/xorm"
	"io/ioutil"
	"leit.com/leit_seat_aps/common"
	"leit.com/leit_seat_aps/db"
	"leit.com/leit_seat_aps/glog"
	"leit.com/leit_seat_aps/service"
	"os"
	"regexp"
	"strings"
	"testing"
	"time"
)

// 测试TOD解析缓存并创建客户订单头
func TestParseTodNew(t *testing.T) {
	var (
		connstring string
		err        error
		projtab    db.Me_project
		projtablst []db.Me_project
		todland    TodLandData
		todProj    service.TodProject
		rd         []os.FileInfo
		fi         os.FileInfo
		edi_file   string
		matched    bool
	)

	// 连接数据库
	connstring = fmt.Sprintf("server=%s;user id=%s;password=%s;database=%s;port=%d;encrypt=disable",
		`localhost`, "lapp", "123fis", "SJA_APS", 1433)
	//connstring = fmt.Sprintf("server=%s;user id=%s;password=%s;database=%s;port=%d;encrypt=disable",
	//	`10.114.64.19`, "lapp", "123fis", "APS", 1433)
	if err = db.InitMssqlDb(connstring); err != nil {

		t.Errorf("Failed to connect db due to: %v", err)
		return
	}
	defer db.G_DbEngine.Close()

	projtab = db.Me_project{}
	if projtablst, err = projtab.GetAllActive(); err != nil {
		t.Errorf("Failed to get project list!")
		return
	}

	for i, _ := range projtablst {
		todProj = service.TodProject{}
		todProj.Projecttab = projtablst[i]
		todProj.Projectid = projtablst[i].Projectid
		fmt.Println("Ready to load data for project:", projtablst[i].Projectid)
		if err = todProj.GetAttributeList(common.ATCOD_TYPE_PART); err != nil {
			t.Errorf("Failed to load attribute list for project:%s due to: %v", projtablst[i].Projectid, err)
			return
		}
		if err = todProj.GetPartFamilyList(); err != nil {
			t.Errorf("Failed to load partfamily list for project:%s due to: %v", projtablst[i].Projectid, err)
			return
		}
		if err = todProj.GetSupplyGroupList(); err != nil {
			t.Errorf("Failed to load supplygroup list for project:%s due to: %v", projtablst[i].Projectid, err)
			return
		}
		if err = todProj.GetPartList(); err != nil {
			t.Errorf("Failed to load part list for project:%s due to: %v", projtablst[i].Projectid, err)
			return
		}
		if err = todProj.GetCarModelList(); err != nil {
			t.Errorf("Failed to load carmodel list for project:%s due to: %v", projtablst[i].Projectid, err)
			return
		}
		if err = todProj.GetPartRuleList(); err != nil {
			t.Errorf("Failed to load partrule list for project:%s due to: %v", projtablst[i].Projectid, err)
			return
		}
		fmt.Println("Finish loading the project master data!")

		fmt.Println(projtablst[i].Projectid, " ==> Try to load file from folder:", projtablst[i].Tod_folder)
		if rd, err = ioutil.ReadDir(projtablst[i].Tod_folder + "/inbox"); err != nil {
			t.Errorf("Failed to read tod folder due to %v", err)
			return
		}
		for _, fi = range rd {
			if fi.IsDir() {
				continue
			}
			edi_file = projtablst[i].Tod_folder + "/inbox" + "/" + fi.Name()
			fmt.Println("Try to load tod file:", edi_file)
			// 判断文件的合规性
			if matched, err = regexp.MatchString(todProj.Projecttab.Tod_filename_regexp, fi.Name()); err != nil {
				t.Errorf("Failed to match EDI file name: %s due to: %v", fi.Name(), err)
				return
			}
			if !matched {
				t.Errorf("The specified EDI file: %s doesn't match the required name specification!", fi.Name())
				os.Rename(edi_file, todProj.Projecttab.Tod_folder+"/errbox"+"/"+fi.Name())
				return
			}

			fmt.Println("Ready to read tod data......")
			todland = TodLandData{}
			todland.Projnr = projtablst[i].Projectid
			todland.Edifile = edi_file
			if err = todland.ReadTodData(edi_file); err != nil {
				t.Errorf("Failed to ReadTodData for project %s: due to: %v", projtablst[i].Projectid, err)
				os.Rename(edi_file, todProj.Projecttab.Tod_folder+"/errbox"+"/"+fi.Name())
				return
			}
			if err = todland.SaveTodData(&todProj); err != nil {
				t.Errorf("Failed to SaveTodData for project: due to: %v,err is :%v", projtablst[i].Projectid, err)
				os.Rename(edi_file, todProj.Projecttab.Tod_folder+"/errbox"+"/"+fi.Name())
				return
			}
			// 将TOD文件移到已处理文件夹 outbox
			os.Rename(edi_file, todProj.Projecttab.Tod_folder+"/outbox"+"/"+fi.Name())
		}
	}
}

// 测试TOD解析创建生产订单
func TestParseTodCustorderNew(t *testing.T) {
	var (
		connstring string
		err        error
		projtab    db.Me_project
		projtablst []db.Me_project
		mesprojtab db.Prod_project
		todProj    service.TodProject
		covtab     db.Pln_custorder_ver
		vcovlst    []db.VCustorderVer
		bl_co      service.BL_CustOrder
		i, j       int
		bRet       bool
		session    *xorm.Session
	)

	// 连接数据库
	connstring = fmt.Sprintf("server=%s;user id=%s;password=%s;database=%s;port=%d;encrypt=disable",
		`localhost`, "lapp", "123fis", "SJA_APS", 1433)
	//connstring = fmt.Sprintf("server=%s;user id=%s;password=%s;database=%s;port=%d;encrypt=disable",
	//	`10.114.64.19`, "lapp", "123fis", "APS", 1433)

	if err = db.InitMssqlDb(connstring); err != nil {
		t.Errorf("Failed to connect db due to: %v", err)
		return
	}
	defer db.G_DbEngine.Close()

	projtab = db.Me_project{}
	if projtablst, err = projtab.GetAllActive(); err != nil {
		t.Errorf("Failed to get project list!")
		return
	}

	for i, _ = range projtablst {
		todProj = service.TodProject{}
		todProj.Projecttab = projtablst[i]
		todProj.Projectid = projtablst[i].Projectid
		// 获取生产项目表
		mesprojtab = db.Prod_project{Projectid: projtablst[i].Projectid}
		if todProj.Mesprojtab, err = mesprojtab.SelectOne(); err != nil{
			err = errors.New(fmt.Sprintf("获取MES项目%s表数据失败:%v",projtablst[i].Projectid, err))
			return
		}
		fmt.Println("Ready to load data for project:", projtablst[i].Projectid)
		if err = todProj.GetAttributeList(common.ATCOD_TYPE_PART); err != nil {
			t.Errorf("Failed to load attribute list for project:%s due to: %v", projtablst[i].Projectid, err)
			return
		}
		if err = todProj.GetPartFamilyList(); err != nil {
			t.Errorf("Failed to load partfamily list for project:%s due to: %v", projtablst[i].Projectid, err)
			return
		}
		if err = todProj.GetSupplyGroupList(); err != nil {
			t.Errorf("Failed to load supplygroup list for project:%s due to: %v", projtablst[i].Projectid, err)
			return
		}
		if err = todProj.GetPartList(); err != nil {
			t.Errorf("Failed to load part list for project:%s due to: %v", projtablst[i].Projectid, err)
			return
		}
		if err = todProj.GetPartRuleList(); err != nil {
			t.Errorf("Failed to load partrule list for project:%s due to: %v", projtablst[i].Projectid, err)
			return
		}
		fmt.Println("Finish loading the project master data!")

		// 获取待解析的订单版本头数据
		covtab = db.Pln_custorder_ver{}
		if vcovlst, err = covtab.GetProjectUnparsedTOD(todProj.Projectid); err != nil {
			t.Errorf("Failed to load unparsed coh list due to %v", err)
			return
		}

		// 加载客户订单进行校验
		for j, _ = range vcovlst {
			bl_co 				= service.BL_CustOrder{}
			bl_co.Custordernr 	= vcovlst[j].Pln_custorder.Custordernr
			bl_co.Oemordernr 	= vcovlst[j].Pln_custorder.Oemordernr
			bl_co.Projnr 		= vcovlst[j].Pln_custorder.Projnr
			bl_co.Partfamilyid 	= vcovlst[j].Pln_custorder_ver.Partfamilyid
			bl_co.SeqType 		= vcovlst[j].Pln_custorder_ver.Seqmode
			bl_co.Edifile 		= vcovlst[j].Pln_custorder_ver.Edifile
			bl_co.Custordertab 	= vcovlst[j].Pln_custorder

			// 初始化客户订单的错误清单
			bl_co.Bl_verrdict = make(map[string]service.BL_VerifyError)
			// 查询获得客户订单的供应组的零件清单
			if err = bl_co.GetSupplyGroupPartList(&todProj); err != nil {
				t.Errorf("Failed to get CO:%s supplygroup part list due to: %v", bl_co.Custordernr, err)
				return
			}
			// 用客户订单版本头中的零件数据替换已有的供应组零件
			if err = bl_co.ReplaceSGPartListByCOVData(vcovlst[j].Pln_custorder_ver, &todProj); err != nil {
				t.Errorf("Failed to replace CO:%s supplygroup part list due to: %v", bl_co.Custordernr, err)
				return
			}
			// 将共用供应组的零件进行分配
			if err = bl_co.AssignNeutralPart(&todProj); err != nil {
				t.Errorf("Failed to assign neutral parts for custorder:%s due to: %v", bl_co.Custordernr, err)
				return
			}
			// 将不生成工单的供应组（例如头枕HR）的零件分配到指定供应组
			if err = bl_co.AssignPartsToShippableSG(&todProj); err != nil {
				t.Errorf("Failed to assign neutral parts for custorder:%s due to: %v", bl_co.Custordernr, err)
				return
			}
			// 基于供应组零件清单获取客户订单的完整零件清单
			if err = bl_co.GetPartListFromSupplyGroup(&todProj); err != nil {
				t.Errorf("Failed to get Partlist by SG for custorder:%s due to: %v", bl_co.Custordernr, err)
				return
			}
			// 基于客户订单的零件清单获取订单的属性清单
			if err = bl_co.GetAtcodListFromPartList(&todProj); err != nil {
				t.Errorf("Failed to get Atcodlist by Partlist for custorder:%s due to: %v", bl_co.Custordernr, err)
				return
			}
			// 获取客户订单供应组的属性清单(不含共用件SG）
			if err = bl_co.GenerateSupplyGroupAtcod(&todProj); err != nil {
				t.Errorf("Failed to generate sg attribute list for custorder:%s due to: %v", bl_co.Custordernr, err)
				return
			}
			// 验证客户订单零件
			if err = bl_co.ValidatePartList(&todProj); err != nil {
				t.Errorf("Failed to verify CO:%s partlist due to: %v", bl_co.Custordernr, err)
				return
			}
			// 验证客户订单属性(按SupplyGroup）
			if todProj.Projecttab.Tod_verify_supplygroupattribute > 0 {
				if err = bl_co.ValidateSupplyGroupAtcodList(&todProj); err != nil {
					t.Errorf("Failed to verify CO:%s attributelist due to: %v", bl_co.Custordernr, err)
					return
				}
			}
			// 验证客户订单属性（按Part Family）
			if todProj.Projecttab.Tod_verify_partfamilyattribute > 0 {
				if err = bl_co.ValidatePartFamilyAtcodList(&todProj); err != nil {
					t.Errorf("Failed to verify CO:%s attributelist due to: %v", bl_co.Custordernr, err)
					return
				}
			}
			// 生成客户订单的生产订单数据
			if err = bl_co.GenerateWorkorderBySupplyGroup(&todProj); err != nil {
				t.Errorf("Failed to GenerateWorkorderBySupplyGroup for CO:%s due to: %v", bl_co.Custordernr, err)
				return
			}
			// 生成客户订单的生产订单属性数据
			if err = bl_co.GenerateWorkorderAtcodlst(&todProj); err != nil {
				t.Errorf("Failed to GenerateWorkorderAtcodlst for CO:%s due to: %v", bl_co.Custordernr, err)
				return
			}
			// 保存客户订单的解析结果数据
			session = db.G_DbEngine.NewSession()
			defer session.Close()
			if err := session.Begin(); err != nil {
				return
			}
			bRet = true

			// 如果是SEQ+2取消客户订单，则不再记录验证错误，直接更新客户订单及其生产订单的状态为98
			if bl_co.SeqType == 2 {
				if err = bl_co.Cancel(session); err != nil {
					bRet = false
				}
				// 更新COV头的解析状态
				vcovlst[j].Pln_custorder_ver.Parsed = 1
				vcovlst[j].Pln_custorder_ver.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
				if err = vcovlst[j].Pln_custorder_ver.UpdateFields(session, "parsed, lastmodif"); err != nil {
					bRet = false
				}
			}

			// 保存客户订单的解析错误
			// 如果有错误则不再更新后续其它数据
			if len(bl_co.Bl_verrdict) > 0 && bl_co.SeqType != 2 {
				if err = bl_co.SaveParseErrors(session); err != nil {
					t.Errorf("Failed to Save CO:%s parse result due to: %v", bl_co.Custordernr, err)
					bRet = false
				} else {
					// 更新客户订单的处理状态为ER并返回继续
					bl_co.Custordertab.Handlestatus = common.CO_PARSE_STATUS_ERROR
					bl_co.Custordertab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
					if err = bl_co.Custordertab.UpdateFields(session, "handlestatus,lastmodif"); err != nil {
						bRet = false
					}
					//session.Commit()
					//session.Close()
					//continue
				}
			} else {
				bl_co.Custordertab.Handlestatus = common.CO_PARSE_STATUS_OK
				bl_co.Custordertab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
				if err = bl_co.Custordertab.UpdateFields(session, "handlestatus,lastmodif"); err != nil {
					bRet = false
				}
				// 更新COV头的解析状态
				vcovlst[j].Pln_custorder_ver.Parsed = 1
				vcovlst[j].Pln_custorder_ver.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
				if err = vcovlst[j].Pln_custorder_ver.UpdateFields(session, "parsed, lastmodif"); err != nil {
					bRet = false
				}
			}

			// 保存客户订单的零件数据和属性数据
			if bRet {
				if err = bl_co.SavePartAndAtcodData(session, &todProj); err != nil {
					t.Errorf("Failed to Save CO:%s parse result due to: %v", bl_co.Custordernr, err)
					bRet = false
				}
			}

			// 保存客户订单下的生产订单头和它的零件及属性数据
			if bRet {
				if err = bl_co.SaveWorkorderData(session, &todProj); err != nil {
					t.Errorf("Failed to Save CO:%s workorder due to: %v", bl_co.Custordernr, err)
					bRet = false
				}
			}

			if bRet {
				session.Commit()
				session.Close()
			} else {
				session.Rollback()
				session.Close()
			}
		}
	}

	return
}

func TestReadTodXlsx(t *testing.T) {
	var (
		connstring string
		err        error
		projtab    db.Me_project
		projtablst []db.Me_project
		todland    TodLandData
		todProj    service.TodProject
		rd         []os.FileInfo
		fi         os.FileInfo
		edi_file   string
	)

	// 连接数据库
	connstring = fmt.Sprintf("server=%s;user id=%s;password=%s;database=%s;port=%d;encrypt=disable",
		`localhost`, "sa", "123456", "LAPP_JITS", 1433)
	//connstring = fmt.Sprintf("server=%s;user id=%s;password=%s;database=%s;port=%d;encrypt=disable",
	//	`10.114.64.19`, "lapp", "123fis", "APS", 1433)

	if err = db.InitMssqlDb(connstring); err != nil {
		t.Errorf("Failed to connect db due to: %v", err)
		return
	}
	defer db.G_DbEngine.Close()

	projtab = db.Me_project{}
	if projtablst, err = projtab.GetAllActive(); err != nil {
		glog.InfoExtln("tod解析", "Failed to get project list!", err)
		return
	}
	for i, _ := range projtablst {

		fmt.Println(projtablst[i].Projectid, " ==> Try to load file from folder:", projtablst[i].Tod_folder)

		if rd, err = ioutil.ReadDir(projtablst[i].Tod_folder + "/inxlsx"); err != nil {
			return
		}
		rds := common.SortByTime(rd)
		for _, fi = range rds {
			if fi.IsDir() {
				continue
			}
			edi_file = projtablst[i].Tod_folder + "/inxlsx" + "/" + fi.Name()

			fmt.Println(projtablst[i].Projectid, " ==> edi_file:", edi_file)
			xlsx, err := excelize.OpenFile(edi_file)
			if err != nil {
				return
			}
			todland = TodLandData{}
			todland.Projnr = projtablst[i].Projectid
			todland.Edifile = edi_file
			rows := xlsx.GetRows("Sheet1")
			// 初始化客户订单字典
			todland.Bl_CoDict = make(map[string]service.BL_CustOrder)
			for k, row := range rows {
				if k == 0 {
					//忽略第一行
					continue
				}

				partfamilyid := common.ValueToString(row[5], "")
				oemordernr := common.ValueToString(row[0], "")
				partid := common.ValueToString(row[2], "")
				supplygroup := common.ValueToString(row[1], "")
				qty := common.ValueToInt(row[3], 0)
				assemdate := common.ValueToString(row[4], "")
				if !common.ValueIsEmpty(assemdate) && len(assemdate) == 10 {
					//分割
					date := strings.Split(assemdate, ".")
					assemdate = date[2] + date[1] + date[0] + "000000"
				}
				// 遍历TOD文件将数据填充到订单字典中
				if bl_co, ok := todland.Bl_CoDict[oemordernr]; !ok {
					// 客户订单不存在，初始化一个新的客户订单
					bl_co = service.BL_CustOrder{}
					bl_co.Oemordernr = oemordernr
					bl_co.Projnr = todland.Projnr
					bl_co.Partfamilyid = partfamilyid
					bl_co.SeqType = 3
					bl_co.Ordertime = time.Now()
					bl_co.Swet = time.Now()
					bl_co.Edifile = todland.Edifile
					bl_co.Bl_sgdict = make(map[string]service.BL_SupplyGroup)
					bl_co.Bl_ordattrdict = make(map[int]service.BL_OrdAttribute)
					bl_co.Bl_verrdict = make(map[string]service.BL_VerifyError)
					todland.Bl_CoDict[partfamilyid] = bl_co
				} else {
					// 获取供应组下所有的零件信息
					sg := service.BL_SupplyGroup{}
					sg.Bl_sgpartDict = make(map[string]service.BL_SgPart)
					sg.Partfamilyid = partfamilyid
					sg.Supplygroupid = supplygroup
					// 获取供应组下所有的零件信息
					bl_sgpt := service.BL_SgPart{}
					PartNr := strings.Split(partid, "-")
					bl_sgpt.Partid = PartNr[0] + "-" + PartNr[1] // 零件号
					bl_sgpt.Ai = PartNr[1]                       // 零件版本号
					bl_sgpt.Qty = qty                            // 零件数量
					bl_sgpt.Supplygroupid = supplygroup
					bl_sgpt.Originalsgid = supplygroup

					bl_sgpt.AtcodDict = make(map[int]service.BL_Attribute)
					sg.Bl_sgpartDict[bl_sgpt.Partid] = bl_sgpt
					bl_co.Bl_sgdict[sg.Supplygroupid] = sg
					todland.Bl_CoDict[bl_co.Oemordernr] = bl_co
				}

			}
			fmt.Println(todland)
			err = todland.SaveTodData(&todProj)
			if err != nil {
				t.Errorf("todland err is:%v", err)
			}
			// 将TOD文件移到已处理文件夹 outbox
			os.Rename(edi_file, todProj.Projecttab.Tod_folder+"/outxlsx"+"/"+fi.Name())
		}
	}
}