package tod

import (
	"encoding/csv"
	"fmt"
	"github.com/go-xorm/xorm"
	"github.com/pkg/errors"
	"io"
	"io/ioutil"
	"leit.com/leit_seat_aps/common"
	"leit.com/leit_seat_aps/db"
	"leit.com/leit_seat_aps/etcd"
	"leit.com/leit_seat_aps/glog"
	"leit.com/leit_seat_aps/service"
	"os"
	"regexp"
	"strings"
	"time"
)

//解析tod文件缓存并创建客户订单头
func ParseTodNew() {
	for {
		var (
			err        error
			projtab    db.Me_project
			projtablst []db.Me_project
			todland    TodLandData
			todProj    service.TodProject
			rd         []os.FileInfo
			fi         os.FileInfo
			edi_file   string
			matched    bool
			flag       bool
		)
		projtab = db.Me_project{}
		if projtablst, err = projtab.GetAllActive(); err != nil {
			glog.InfoExtln("tod解析", "Failed to get project list!", err)
			return
		}

		for i, _ := range projtablst {
			flag = false
			fmt.Println(projtablst[i].Projectid, " ==> Try to load file from folder:", projtablst[i].Tod_folder)

			if rd, err = ioutil.ReadDir(projtablst[i].Tod_folder + "/inbox"); err != nil {
				return
			}
			rds := common.SortByTime(rd)
			for _, fi = range rds {
				if fi.IsDir() {
					continue
				}
				if !flag {
					todProj = service.TodProject{}
					todProj.Projecttab = projtablst[i]
					todProj.Projectid = projtablst[i].Projectid
					fmt.Println("Ready to load data for project:", projtablst[i].Projectid)
					if err = todProj.GetAttributeList(common.ATCOD_TYPE_PART); err != nil {
						glog.InfoExtln("tod解析", "Failed to load attribute list for project: due to: ", projtablst[i].Projectid, err)
						return
					}
					if err = todProj.GetPartFamilyList(); err != nil {
						glog.InfoExtln("tod解析", "Failed to load partfamily list for project is :", err)
						return
					}
					if err = todProj.GetSupplyGroupList(); err != nil {
						glog.InfoExtln("tod解析", "Failed to load supplygroup list for project is :", err)
						return
					}
					if err = todProj.GetPartList(); err != nil {
						glog.InfoExtln("tod解析", "Failed to load part list for project is :", err)
						return
					}
					if err = todProj.GetCarModelList(); err != nil {
						glog.InfoExtln("tod解析", "Failed to load carmodel list for project is :", err)
						return
					}
					if err = todProj.GetPartRuleList(); err != nil {
						glog.InfoExtln("tod解析", "Failed to load partrule list for project is :", err)
						return
					}
					flag = true
					fmt.Println("Finish loading the project master data!")
				}
				edi_file = projtablst[i].Tod_folder + "/inbox" + "/" + fi.Name()
				fmt.Println("Try to load tod file:", edi_file)
				// 判断文件的合规性
				if matched, err = regexp.MatchString(todProj.Projecttab.Tod_filename_regexp, fi.Name()); err != nil {
					glog.InfoExtln("tod解析", "doesn't match the required name specification! is :", err)
					return
				}
				if !matched {
					glog.InfoExtln("tod解析", "doesn't match the required name specification!")
					os.Rename(edi_file, projtablst[i].Tod_folder+"/errbox"+"/"+fi.Name())
					continue
				}

				fmt.Println("Ready to read tod data......")
				todland = TodLandData{}
				todland.Projnr = projtablst[i].Projectid
				todland.Edifile = edi_file
				if err = todland.ReadTodData(edi_file); err != nil {
					os.Rename(edi_file, todProj.Projecttab.Tod_folder+"/errbox"+"/"+fi.Name())
					glog.InfoExtln("tod解析", "Failed to ReadTodData for project is :", err)
					continue
				}
				if err = todland.SaveTodData(&todProj); err != nil {
					glog.InfoExtln("tod解析", "Failed to SaveTodData for project! is :", err)
					os.Rename(edi_file, todProj.Projecttab.Tod_folder+"/errbox"+"/"+fi.Name())
					continue
				}
				// 将TOD文件移到已处理文件夹 outbox
				os.Rename(edi_file, todProj.Projecttab.Tod_folder+"/outbox"+"/"+fi.Name())
			}
		}
		//等待十秒
		time.Sleep(10 * time.Second)
	}
}

// TOD解析创建生产订单
func ParseTodCustorderNew() {
	for {
		fmt.Println("TOD解析创建生产订单监听中!")
		var (
			err                   error
			projtab               db.Me_project
			projtablst            []db.Me_project
			mesprojtab            db.Prod_project
			todProj               service.TodProject
			covtab                db.Pln_custorder_ver
			vcovlst               []db.VCustorderVer
			bl_co                 service.BL_CustOrder
			cust                  db.Pln_custorder
			mailsubject, mailbody string
			verr                  service.BL_VerifyError
			i, j                  int
			bRet                  bool
			session               *xorm.Session
		)

		projtab = db.Me_project{}
		if projtablst, err = projtab.GetAllActive(); err != nil {
			glog.InfoExtln("TOD解析创建生产订单", "Failed to get project list!", err)
			return
		}

		for i, _ = range projtablst {
			// 获取待解析的订单版本头数据
			covtab = db.Pln_custorder_ver{}
			if vcovlst, err = covtab.GetProjectUnparsedTOD(projtablst[i].Projectid); err != nil {
				glog.InfoExtln("TOD解析创建生产订单", "Failed to load unparsed coh list due to :", err)
				return
			}
			if len(vcovlst) == 0 {
				glog.InfoExtln("TOD解析创建生产订单", "vcovlst is empty")
				continue
			}
			todProj = service.TodProject{}
			todProj.Projecttab = projtablst[i]
			todProj.Projectid = projtablst[i].Projectid
			fmt.Println("Ready to load data for project:", projtablst[i].Projectid)
			// 获取生产项目表
			mesprojtab = db.Prod_project{Projectid: projtablst[i].Projectid}
			if todProj.Mesprojtab, err = mesprojtab.SelectOne(); err != nil {
				err = errors.New(fmt.Sprintf("获取MES项目%s表数据失败:%v", projtablst[i].Projectid, err))
				glog.InfoExtln("TOD解析创建生产订单", err)
				return
			}
			if err = todProj.GetAttributeList(common.ATCOD_TYPE_PART); err != nil {
				glog.InfoExtln("TOD解析创建生产订单", "Failed to load attribute list for project: due to: ", projtablst[i].Projectid, err)
				return
			}
			if err = todProj.GetPartFamilyList(); err != nil {
				glog.InfoExtln("TOD解析创建生产订单", "Failed to load partfamily list for project: due to: ", projtablst[i].Projectid, err)
				return
			}
			if err = todProj.GetSupplyGroupList(); err != nil {
				glog.InfoExtln("TOD解析创建生产订单", "Failed to load supplygroup list for project: due to: ", projtablst[i].Projectid, err)
				return
			}
			if err = todProj.GetPartList(); err != nil {
				glog.InfoExtln("TOD解析创建生产订单", "Failed to load part list for project: due to: ", projtablst[i].Projectid, err)
				return
			}
			if err = todProj.GetPartRuleList(); err != nil {
				glog.InfoExtln("TOD解析创建生产订单", "Failed to load partrule list for project: due to: ", projtablst[i].Projectid, err)
				return
			}
			fmt.Println("Finish loading the project master data!")

			// 加载客户订单进行校验
			for j, _ = range vcovlst {
				//判断是否存在前一个版本，
				//如果存在前一个版本，判断是否解析
				if vcovlst[j].Pln_custorder_ver.Version > 1 {
					perInfo := new(db.Pln_custorder_ver)
					perInfo.Finr = vcovlst[j].Pln_custorder_ver.Finr
					perInfo.Custordernr = vcovlst[j].Pln_custorder_ver.Custordernr
					perInfo.Version = vcovlst[j].Pln_custorder_ver.Version
					res := perInfo.SelectInfo()
					if err != nil {
						glog.InfoExtln("TOD解析创建生产订单", "err is:", err)
						return
					}
					if res {
						glog.InfoExtln("TOD版本解析", "perInfo.Custordernr:", perInfo.Custordernr)
						glog.InfoExtln("TOD版本解析", "perInfo.Version:", perInfo.Version)
						continue
					}
				}

				/* 获取锁 */
			LOOP:
				err = etcd.G_jobLock.TryLock("lock")
				if err != nil {
					fmt.Println("tod groutine lock fail!")
					time.Sleep(10 * time.Millisecond)
					goto LOOP
				}

				fmt.Println("tod groutine lock success!")
				bl_co = service.BL_CustOrder{}
				bl_co.Custordernr = vcovlst[j].Pln_custorder.Custordernr
				bl_co.Oemordernr = vcovlst[j].Pln_custorder.Oemordernr
				bl_co.Projnr = vcovlst[j].Pln_custorder.Projnr
				bl_co.Partfamilyid = vcovlst[j].Pln_custorder_ver.Partfamilyid
				bl_co.SeqType = vcovlst[j].Pln_custorder_ver.Seqmode
				bl_co.Edifile = vcovlst[j].Pln_custorder_ver.Edifile
				bl_co.Custordertab = vcovlst[j].Pln_custorder
				bl_co.Parsed = vcovlst[j].Pln_custorder_ver.Parsed

				// 初始化客户订单的错误清单
				bl_co.Bl_verrdict = make(map[string]service.BL_VerifyError)

				// 对于TOD SEQ+3的重复订单，在半年内的要生成错误（参照客户订单错误生成逻辑）
				if covtab.Seqmode == 3 {
					//半年前的时间
					sixmonth := time.Now().AddDate(0, -6, 0)
					Expirationtime := common.TimeFormat(sixmonth, "yyyyMMddHHmmss")
					cust.Oemordernr = vcovlst[j].Pln_custorder.Oemordernr
					_, ok, err := cust.SelectExpirationtime(Expirationtime)
					if err != nil {
						glog.InfoExtln("TOD解析创建生产订单", "err is:", err)
						etcd.G_jobLock.UnLock()
						return
					}
					if ok {
						verr = service.BL_VerifyError{}
						verr.Errortype = common.REPEATORDER
						verr.Partfamilyid = vcovlst[j].Pln_custorder.Partfamilyid
						verr.Errorinfo = "TOD SEQ+3的重复订单:" + vcovlst[j].Pln_custorder.Oemordernr
						bl_co.Bl_verrdict[verr.GetDictKey()] = verr

						//添加邮件预警
						mailsubject = "客户订单号重复错误"
						mailbody = "项目号: " + todProj.Projectid + "\r\n客户订单号: " + bl_co.Oemordernr + "\r\nErrorinfo : 半年内重复！"
						if err = SendTodErrorEmail(projtablst[i], mailsubject, mailbody); err != nil {
							glog.InfoExtln("发送邮件", "err is :", err)
						}
					}
				}

				// 如果是指定供应组的SEQ+1发邮件预警
				AlertForCustOrderUpdateTodByRule(projtablst[i], bl_co, vcovlst[j])

				// 查询获得客户订单的供应组的零件清单
				if err = bl_co.GetSupplyGroupPartList(&todProj); err != nil {
					glog.InfoExtln("TOD解析创建生产订单", "Failed to get CO: supplygroup part list due to: ", bl_co.Custordernr, err)
					etcd.G_jobLock.UnLock()
					return
				}
				// 用客户订单版本头中的零件数据替换已有的供应组零件
				if err = bl_co.ReplaceSGPartListByCOVData(vcovlst[j].Pln_custorder_ver, &todProj); err != nil {
					glog.InfoExtln("TOD解析创建生产订单", "Failed to replace CO: supplygroup part list due to: ", bl_co.Custordernr, err)
					etcd.G_jobLock.UnLock()
					return
				}
				// 将共用供应组的零件进行分配
				if err = bl_co.AssignNeutralPart(&todProj); err != nil {
					glog.InfoExtln("TOD解析创建生产订单", "Failed to assign neutral parts for custorder: due to: ", bl_co.Custordernr, err)
					etcd.G_jobLock.UnLock()
					return
				}
				// 将不生成工单的供应组（例如头枕HR）的零件分配到指定供应组
				if err = bl_co.AssignPartsToShippableSG(&todProj); err != nil {
					glog.InfoExtln("TOD解析创建生产订单", "Failed to assign neutral parts for custorder: due to: ", bl_co.Custordernr, err)
					etcd.G_jobLock.UnLock()
					return
				}
				// 基于供应组零件清单获取客户订单的完整零件清单
				if err = bl_co.GetPartListFromSupplyGroup(&todProj); err != nil {
					glog.InfoExtln("TOD解析创建生产订单", "Failed to get Partlist by SG for custorder: due to: ", bl_co.Custordernr, err)
					etcd.G_jobLock.UnLock()
					return
				}
				// 基于客户订单的零件清单获取订单的属性清单
				if err = bl_co.GetAtcodListFromPartList(&todProj); err != nil {
					glog.InfoExtln("TOD解析创建生产订单", "Failed to get Atcodlist by Partlist for custorder: due to: ", bl_co.Custordernr, err)
					etcd.G_jobLock.UnLock()
					return
				}
				// 获取客户订单供应组的属性清单(不含共用件SG）
				if err = bl_co.GenerateSupplyGroupAtcod(&todProj); err != nil {
					glog.InfoExtln("TOD解析创建生产订单", "Failed to generate sg attribute list for custorder: due to: ", bl_co.Custordernr, err)
					etcd.G_jobLock.UnLock()
					return
				}
				// 验证客户订单零件
				if err = bl_co.ValidatePartList(&todProj); err != nil {
					glog.InfoExtln("TOD解析创建生产订单", "Failed to verify CO: partlist due to: ", bl_co.Custordernr, err)
					etcd.G_jobLock.UnLock()
					return
				}
				// 验证客户订单属性(按SupplyGroup）
				if todProj.Projecttab.Tod_verify_supplygroupattribute > 0 {
					if err = bl_co.ValidateSupplyGroupAtcodList(&todProj); err != nil {
						glog.InfoExtln("TOD解析创建生产订单", "Failed to verify CO: attributelist due to: ", bl_co.Custordernr, err)
						etcd.G_jobLock.UnLock()
						return
					}
				}
				// 验证客户订单属性（按Part Family）
				if todProj.Projecttab.Tod_verify_partfamilyattribute > 0 {
					if err = bl_co.ValidatePartFamilyAtcodList(&todProj); err != nil {
						glog.InfoExtln("TOD解析创建生产订单", "Failed to verify CO: attributelist due to: ", bl_co.Custordernr, err)
						etcd.G_jobLock.UnLock()
						return
					}
				}
				// 生成客户订单的生产订单数据
				if err = bl_co.GenerateWorkorderBySupplyGroup(&todProj); err != nil {
					glog.InfoExtln("TOD解析创建生产订单", "Failed to GenerateWorkorderBySupplyGroup for CO: due to: ", bl_co.Custordernr, err)
					etcd.G_jobLock.UnLock()
					return
				}
				// 生成客户订单的生产订单属性数据
				if err = bl_co.GenerateWorkorderAtcodlst(&todProj); err != nil {
					glog.InfoExtln("TOD解析创建生产订单", "Failed to GenerateWorkorderAtcodlst for CO: due to: ", bl_co.Custordernr, err)
					etcd.G_jobLock.UnLock()
					return
				}
				// 保存客户订单的解析结果数据
				session = db.G_DbEngine.NewSession()
				if err := session.Begin(); err != nil {
					session.Close()
					etcd.G_jobLock.UnLock()
					return
				}
				bRet = true
				// 保存客户订单的零件数据和属性数据
				if bRet {
					if err = bl_co.SavePartAndAtcodData(session, &todProj); err != nil {
						glog.InfoExtln("TOD解析创建生产订单", "Failed to Save CO: parse result due to: ", bl_co.Custordernr, err)
						bRet = false
					}
				}

				if bl_co.SeqType == 1 && bl_co.Custordertab.Status >= common.CO_STATUS_RELEASED {
					//添加邮件预警
					mailsubject = "SEQ+1的客户订单已下达"
					mailbody = "项目号: " + todProj.Projectid + "\r\n客户订单号: " + bl_co.Oemordernr + "\r\nErrorinfo : 已下达，SEQ+1更新出错！"
					if err = SendTodErrorEmail(projtablst[i], mailsubject, mailbody); err != nil {
						glog.InfoExtln("发送邮件", "err is :", err)
					}
				}

				// 如果是SEQ+2取消客户订单，则不再记录验证错误，直接更新客户订单及其生产订单的状态为98
				if bl_co.SeqType == 2 {
					if err = bl_co.Cancel(session); err != nil {
						bRet = false
					}
					// 更新COV头的解析状态
					vcovlst[j].Pln_custorder_ver.Parsed = 1
					vcovlst[j].Pln_custorder_ver.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
					if err = vcovlst[j].Pln_custorder_ver.UpdateFields(session, "parsed, lastmodif"); err != nil {
						bRet = false
					}

					//添加邮件预警
					mailsubject = "客户订单取消"
					mailbody = "项目号: " + todProj.Projectid + "\r\n客户订单号: " + bl_co.Oemordernr + "\r\n提示信息 : 客户订单接收到+2请求，取消！"
					if err = SendTodErrorEmail(projtablst[i], mailsubject, mailbody); err != nil {
						glog.InfoExtln("发送邮件", "err is :", err)
					}
				}

				// 保存客户订单的解析错误
				// 如果有错误则不再更新后续其它数据
				//if (len(bl_co.Bl_verrdict) > 0 && bl_co.SeqType != 2) || (bl_co.Parsed != 2 && bl_co.SeqType != 2) {
				if len(bl_co.Bl_verrdict) > 0 && bl_co.Parsed != 2 && bl_co.SeqType != 2 {
					if err = bl_co.SaveParseErrors(session); err != nil {
						glog.InfoExtln("TOD解析创建生产订单", "Failed to Save CO: parse result due to: ", bl_co.Custordernr, err)
						bRet = false
					} else {
						// 更新客户订单的处理状态为ER并返回继续
						bl_co.Custordertab.Handlestatus = common.CO_PARSE_STATUS_ERROR
						bl_co.Custordertab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
						if err = bl_co.Custordertab.UpdateFields(session, "handlestatus,lastmodif"); err != nil {
							bRet = false
						}
						if bl_co.SeqType != 3 {
							glog.InfoExtln("tod", "Custordernr：", bl_co.Custordertab.Custordernr)
							session.Commit()
							session.Close()
							etcd.G_jobLock.UnLock()
							continue
						}

					}
				} else {
					bl_co.Custordertab.Handlestatus = common.CO_PARSE_STATUS_OK
					bl_co.Custordertab.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
					if err = bl_co.Custordertab.UpdateFields(session, "handlestatus,lastmodif"); err != nil {
						bRet = false
					}
					// 更新COV头的解析状态
					vcovlst[j].Pln_custorder_ver.Parsed = 1
					vcovlst[j].Pln_custorder_ver.Lastmodif = common.Date(time.Now().Unix(), "YYYYMMDDHHmmss")
					if err = vcovlst[j].Pln_custorder_ver.UpdateFields(session, "parsed, lastmodif"); err != nil {
						bRet = false
					}
				}

				// 保存客户订单下的生产订单头和它的零件及属性数据
				if bRet {
					if err = bl_co.SaveWorkorderData(session, &todProj); err != nil {
						glog.InfoExtln("TOD解析创建生产订单", "Failed to Save CO: workorder due to: ", bl_co.Custordernr, err)
						bRet = false
					}
				}

				if bRet || bl_co.Parsed == 2 {
					session.Commit()
					session.Close()
				} else {
					session.Rollback()
					session.Close()
				}
				etcd.G_jobLock.UnLock()
				time.Sleep(10 * time.Millisecond)
			}
		}
		time.Sleep(10 * time.Second)
	}
}

//监听tod CSV
func ReadTodXlsx() {
	for {
		var (
			err        error
			projtab    db.Me_project
			projtablst []db.Me_project
			todland    TodLandData
			todProj    service.TodProject
			rd         []os.FileInfo
			fi         os.FileInfo
			edi_file   string
			flag       bool
			readline   int
			sg         service.BL_SupplyGroup
			bl_sgpt    service.BL_SgPart
		)
		flag = false

		projtab = db.Me_project{}
		if projtablst, err = projtab.GetAllActive(); err != nil {
			glog.InfoExtln("tod解析", "Failed to get project list!", err)
			return
		}
		for i, _ := range projtablst {

			fmt.Println(projtablst[i].Projectid, " ==> Try to load Tod xlsx from folder:", projtablst[i].Tod_folder)

			if rd, err = ioutil.ReadDir(projtablst[i].Tod_folder + "/inxlsx"); err != nil {
				glog.InfoExtln("tod解析", "Failed to load inxlsx: ", projtablst[i].Projectid, err)
				return
			}
			rds := common.SortByTime(rd)
			for _, fi = range rds {
				if fi.IsDir() {
					glog.InfoExtln("tod解析", "inxlsx is empty")
					continue
				}
				if !flag {
					todProj = service.TodProject{}
					todProj.Projecttab = projtablst[i]
					todProj.Projectid = projtablst[i].Projectid
					fmt.Println("Ready to load data for project:", projtablst[i].Projectid)
					if err = todProj.GetAttributeList(common.ATCOD_TYPE_PART); err != nil {
						glog.InfoExtln("tod解析", "Failed to load attribute list for project: due to: ", projtablst[i].Projectid, err)
						return
					}
					if err = todProj.GetPartFamilyList(); err != nil {
						glog.InfoExtln("tod解析", "Failed to load partfamily list for project is :", err)
						return
					}
					if err = todProj.GetSupplyGroupList(); err != nil {
						glog.InfoExtln("tod解析", "Failed to load supplygroup list for project is :", err)
						return
					}
					if err = todProj.GetPartList(); err != nil {
						glog.InfoExtln("tod解析", "Failed to load part list for project is :", err)
						return
					}
					if err = todProj.GetCarModelList(); err != nil {
						glog.InfoExtln("tod解析", "Failed to load carmodel list for project is :", err)
						return
					}
					if err = todProj.GetPartRuleList(); err != nil {
						glog.InfoExtln("tod解析", "Failed to load partrule list for project is :", err)
						return
					}
					flag = true
					fmt.Println("Finish loading the project master data!")
				}
				edi_file = projtablst[i].Tod_folder + "/inxlsx" + "/" + fi.Name()
				//准备读取文件
				fs, err := os.Open(edi_file)
				if err != nil {
					glog.InfoExtln("tod解析", "Failed to OpenFile!", err)
					return
				}
				if err != nil {
					glog.InfoExtln("tod解析", "Failed to OpenFile!", err)
					return
				}
				r := csv.NewReader(fs)
				todland = TodLandData{}
				todland.Projnr = projtablst[i].Projectid
				todland.Edifile = edi_file
				// 初始化客户订单字典
				todland.Bl_CoDict = make(map[string]service.BL_CustOrder)
				//针对大文件，一行一行的读取文件
				readline = 1
				for {
					row, err := r.Read()
					if err != nil && err != io.EOF {
						glog.InfoExtln("tod解析", "Failed to read file!", err)
						break
					}

					if err == io.EOF {
						glog.InfoExtln("tod解析", "文件读取完毕!")
						break
					}
					if readline == 1 {
						readline++
						//跨过第一行
						continue
					}
					if len(row) > 5 {
						readline++
						partfamilyid := strings.TrimSpace(common.ValueToString(row[5], ""))
						oemordernr := strings.TrimSpace(common.ValueToString(row[0], ""))
						partid := strings.TrimSpace(common.ValueToString(row[2], ""))
						supplygroup := strings.TrimSpace(common.ValueToString(row[1], ""))
						qty := common.ValueToInt(row[3], 0)
						assemdate := strings.TrimSpace(common.ValueToString(row[4], ""))
						if !common.ValueIsEmpty(assemdate) && len(assemdate) == 10 {
							//分割
							date := strings.Split(assemdate, ".")
							if len(date) > 2 {
								assemdate = date[2] + date[1] + date[0] + "000000"
							}
						}
						key := partfamilyid + "-" + oemordernr //零件族+客户订单号为键值
						bl_co, ok := todland.Bl_CoDict[key]
						if !ok {
							// 客户订单不存在，初始化一个新的客户订单
							bl_co = service.BL_CustOrder{}
							bl_co.Oemordernr = oemordernr
							bl_co.Projnr = todland.Projnr
							bl_co.Partfamilyid = partfamilyid
							bl_co.SeqType = 3
							bl_co.Ordertime = time.Now()
							bl_co.Swet = time.Now()
							bl_co.Edifile = todland.Edifile
							bl_co.Bl_sgdict = make(map[string]service.BL_SupplyGroup)
							bl_co.Bl_ordattrdict = make(map[int]service.BL_OrdAttribute)
							bl_co.Bl_verrdict = make(map[string]service.BL_VerifyError)
						}
						sgkey := supplygroup + "-" + partid //供应组+零件为键值
						// 获取供应组下所有的零件信息
						sg = service.BL_SupplyGroup{}
						sg.Bl_sgpartDict = make(map[string]service.BL_SgPart)
						sg.Partfamilyid = partfamilyid
						sg.Supplygroupid = supplygroup
						bl_sgpt = service.BL_SgPart{}
						PartNr := strings.Split(partid, "-")
						bl_sgpt.Partid = PartNr[0] + "-" + PartNr[1] // 零件号
						bl_sgpt.Ai = PartNr[1]                       // 零件版本号
						bl_sgpt.Qty = qty                            // 零件数量
						bl_sgpt.Supplygroupid = sg.Supplygroupid
						bl_sgpt.Originalsgid = sg.Supplygroupid
						bl_sgpt.AtcodDict = make(map[int]service.BL_Attribute)
						sg.Bl_sgpartDict[bl_sgpt.Partid] = bl_sgpt
						bl_co.Bl_sgdict[sgkey] = sg
						todland.Bl_CoDict[key] = bl_co
					}
				}
				//关闭文件
				err = fs.Close()
				if err != nil {
					glog.InfoExtln("tod Close", "Failed Close is :", err)
					return
				}
				if err = todland.SaveTodData(&todProj); err != nil {
					glog.InfoExtln("tod解析", "Failed to SaveTodData for project! is :", err)
					os.Rename(edi_file, todProj.Projecttab.Tod_folder+"/errxlsx"+"/"+fi.Name())
					continue
				}
				// 将TOD文件移到已处理文件夹 outbox
				err = os.Rename(edi_file, todProj.Projecttab.Tod_folder+"/outxlsx"+"/"+fi.Name())
				if err != nil {
					glog.InfoExtln("tod Rename解析", "Failed Rename is :", err)
				}
			}
		}
		//等待十秒
		time.Sleep(10 * time.Second)
	}
}

// 发送TOD解析错误或警示邮件
func SendTodErrorEmail(projtab db.Me_project, subject, errinfo string) (err error) {
	var mail db.Sendmailtab

	mail.Status = "waiting"
	mail.Body = errinfo
	mail.Subject = projtab.Projectid + "-->" + subject
	mail.Name = projtab.From_email
	mail.Tomail = projtab.Tod_error_to_recievers
	mail.Host = projtab.Emailserver_host + ":" + common.ValueToString(projtab.Emailserver_port, "")
	mail.Password = projtab.From_pwd
	mail.Username = projtab.From_email
	err = mail.Add()
	if err != nil {
		glog.InfoExtln("发送邮件", "err is :", err)
	}
	return
}

// 为项目指定的供应组发送SEQ+1的预警邮件(基于指定供应组）
func AlertForCustOrderUpdateTodByRule(projtab db.Me_project, bl_co service.BL_CustOrder, vcov db.VCustorderVer) {
	var (
		err                               error
		idx, seqtype, k                   int
		todrule, v, mailsubject, mailbody string
		kvlst, rulelst                    []string
		sgdict                            map[string]string
		exist                             bool
	)

	// 获取项目指定的TOD邮件规则
	if todrule, err = service.GetSysStringParameter(2, bl_co.Projnr, "TOD_EMAIL_RULE"); err != nil {
		return
	}
	kvlst = strings.Split(todrule, ";")
	if len(kvlst) > 0 {
		for idx, _ = range kvlst {
			rulelst = strings.Split(kvlst[idx], ":")
			// 蒋rulest[0]转化为整型与bl_co.SeqType 比较，如果相等，则比较1开始的供应组
			seqtype = common.ValueToInt(rulelst[0], 0)
			if seqtype == 1 {
				sgdict = make(map[string]string) //去除第一个元素
				for k, v = range rulelst {
					if k > 0 {
						sgdict[v] = v
					}
				}
				// 遍历客户订单的 Pln_custorder_ver_partlst 获取供应组
				exist = false
				for _, jdv := range vcov.Pln_custorder_ver.Partlst {
					if _, exist = sgdict[jdv.Supplygroupid]; exist {
						break
					}
				}
				if exist {
					mailsubject = "SEQ+1提醒"
					mailbody = "项目号: " + bl_co.Custordertab.Projnr + "\r\n客户订单号: " + bl_co.Oemordernr + "\r\n提示信息 : 客户订单接收到+1请求，取消！"
					if err = SendTodErrorEmail(projtab, mailsubject, mailbody); err != nil {
						glog.InfoExtln("发送邮件", "err is :", err)
					}
				}
			}
		}
	}
}
